function plotm( varargin)
% plotm( data, [options])
% Wrapper function which calls stem/plot/plot3 depending on the dimension of varargin{1}
% Also handles interval data
%
% Input:
%   data        dim x N array
%   [options]   everything which can be parsed by plot/plot2/plot3
%               if dim=1, stem is called instead of plot
%
% Options:
%   'box',val               Plots the hypercube of dimension val with volume 1
%   'rotate',val            (experimental) Rotates the current plot by 360 degree in steps of val degree.
%
% Options for 1-dim:
%       'height',val        scalar or 1x2 vector, default=[0 1],
%                           determines the length of the lines to be plotted
%                               scalar: Line goes from 0 to val
%                               vector: Line goes from val(1) to val(2) 
%
% Options for 2-dim:
%       'hull'              Plots also the convex hull
%       'boundary',val      (experminental) Plots also the boundary 
%                               val<= 1: computed with boundary
%                               val>1: points which have less neighbours are computed
%
% Options for 3-dim:
%       'contour'           Plots contour lines
%       'resolution',val    scalar, default depends on the input
%                           determines the resolution
%                               If val==0, the points are plotted
%                               otherwise val determines the resolution of the interpolated grid
%
% Info:
%   1-dim data:
%       Each point is plotted as a dot at height 1. 
%       The height can be changed by Option <'height',val>
%       To change the marker, the option <'Marker','something'> should be used
%
%   2-dim/3-dim data 
%       A point cloud is plotted
%
% E.g.: clf; hold on; clear all; load trimesh3d; plotm([x y z]','resolution',500); plotm([x y z]','resolution',0); view(3); plotm('rotate',0);
%       plotm(randn(2,40),'.-')
%       plotm(randn(1,40),'height',4);
%
% See also: surfm, plot, plot3, stem
%
% Written by: tommsch, 2016

% XX Write plotm4, using the commented out code below
% XX make plotm plot sequences
% XX hull for 3d plot


%Preprocessing
[rotateval,varargin]=parsem('rotate',varargin,[]);
[boxflag,varargin]=parsem('box',varargin,0); 
if(boxflag)
    plotm([mixvector([0 1],boxflag) [zeros(boxflag,1)]],'.-','point');
end

%Plot functions
if(~isempty(varargin)); 

    if(iscell(varargin{1}));
        dim=size(varargin{1}{1},1);
        if(dim==1); plotinterval1(varargin{:});
        else; fprintf('no intervalplot for dim>1\n');
        end
    elseif(isa(varargin{1},'sequence'))
        dim=ndims(varargin{1});
        if(dim==1); plotsequence1(varargin{:});
        elseif(dim==2); plotsequence2(varargin{:});
        else; fprintf('no sequenceplot for dim>1\n');
        end
    else
        dim=size(varargin{1},1);
        if(dim==1); plotm1(varargin{:});
        elseif(dim==2); plotm2(varargin{:}); 
        elseif(dim==3); plotm3(varargin{:});
        else; fprintf('no ''plotm'' for dim>3\n');
        end;
    end
end

%Post processing
if(~isempty(rotateval))
    rotateplot(rotateval);
end

end

function plotinterval1( Q, varargin )
    [verbose,varargin]=parsem('verbose',varargin,0); %Verbose level
    [height,varargin]=parsem('height',varargin,0); %height
    [~,varargin]=parsem('hull',varargin); %Remove hull
    [~,varargin]=parsem('boundary',varargin); %Remove boundary
    [~,varargin]=parsem('resolution', varargin,1); %Remove resolution
    [~,varargin]=parsem('contour',varargin); %Remove contour
    [~,varargin]=parsem('surface',varargin); %remove surface
    [~,varargin]=parsem('point',varargin); %remove pointcloud
    [~,varargin]=parsem('MarkerSize',varargin); %remove pointcloud
    
    n=numel(Q);
    if(numel(height)==1); height=[height height+1]; end;
    X=vertcat(Q{:})'; X=[X; flipud(X)];
    Y=[height(1)*ones(2,n); height(2)*ones(2,n)];
    try
        patch('XData',X,'YData',Y,varargin{:});
    catch
        patch('XData',X,'YData',Y);
    end
    
    if(verbose>=1);
        val1=sum(diff(vertcat(Q{:}),1,2));
        val2=sum(abs(diff(vertcat(Q{:}),1,2)));
        if(val1~=val2);
            fprintf('Area (oriented/unoriented): %i\n',val1,val2);
        else
            fprintf('Area: %i\n',val1);
        end
    end

end

function plotsequence1( c, varargin);
    plotm([((c.idx):(c.idx+numel(c)-1));(c.c).'],varargin{:});
end

function plotsequence2( c, varargin);
    Z=c.c;
    Z=Z(:).';
    Z(Z==0)=[];
    plotm([supp(c); Z],varargin{:});
end

function plotm1( Q, varargin )
    [~,varargin]=parsem('verbose',varargin,0); %Verbose level
    [height,varargin]=parsem('height',varargin,1);
    [~,varargin]=parsem('hull',varargin); %Remove hull
    [~,varargin]=parsem('boundary',varargin); %Remove boundary
    [~,varargin]=parsem('resolution', varargin,1); %Remove resolution
    [~,varargin]=parsem('contour',varargin); %Remove contour
    [~,varargin]=parsem('surface',varargin); %remove surface
    [~,varargin]=parsem('point',varargin); %remove pointcloud
    if(isreal(Q))
        
        try
            stem( Q, height*ones(1,size(Q,2)), varargin{:}); 
            %quiver kann option 'rx', etc. nicht parsen, deshalb kann es nicht verwendet werden.
        catch
            fprintf('Error during plotm. All options are ignored.\n');
            stem( Q, ones(1,size(Q,2))); 
        end
    else
        plotm([real(Q); imag(Q)],varargin{:});
        xlabel('Re')
        ylabel('Im')
    end
end

function plotm2( Q , varargin)
    [~,varargin]=parsem('verbose',varargin,0); %Verbose level
    [~,varargin]=parsem('height',varargin,1); %Remove heigth
    [hull,varargin]=parsem('hull',varargin); %hull
    [boundaryflag,varargin]=parsem('boundary',varargin,0); %Remove boundary
    [~,varargin]=parsem('resolution', varargin,1); %Remove resolution    
    [~,varargin]=parsem('contour',varargin); %Remove contour
    [~,varargin]=parsem('surface',varargin); %remove surface
    [~,varargin]=parsem('point',varargin); %remove pointcloud
    try 
        if(hull)
            idx = convhull(Q(1,:),Q(2,:));
            plot(Q(1,idx),Q(2,idx),varargin{:});
        elseif(boundaryflag)
            if(boundaryflag<=1 && boundaryflag>=0);
                idx = boundary(Q(1,:).',Q(2,:).',boundaryflag);
                plot(Q(1,idx),Q(2,idx),varargin{:});                
            elseif(boundaryflag>0)
                D=pdist2(Q.',Q.','cityblock','Smallest',10);
                DD=sum(D,1);
                DDD=cumsum(sort(DD-mean(DD)));
                [~,idx]=min(DDD);
                bar=abs(DDD(idx+1)-DDD(idx))+mean(DD)*(boundaryflag-1);
                idx=DD>bar;
                plot(Q(1,idx),Q(2,idx),varargin{:});
            elseif(boundaryflag<0) %THIS METHOD IS NOT WORKING YET XX
                boundaryflag=abs(boundaryflag);
                DT=delaunayTriangulation(Q.');
                AA=DTarea(DT).';
                AAA=cumsum(sort(AA-mean(AA)));
                [~,idx]=min(AAA);
                bar=AAA(idx+1)-AAA(idx)+mean(AA)*(boundaryflag);
                idx=AA>bar; %idx of triangles with area less then bar
                EDGE=[DT.ConnectivityList(idx,[1 2]); DT.ConnectivityList(idx,[2 3]); DT.ConnectivityList(idx,[3 1])]; %edges of chosen triangles
                [~,ia]=unique(EDGE,'rows'); %ia: ?indices of unique edges
                ia = sortrows(EDGE(ia,:)); %indices of "unique" vertices
                ia=unique(ia(:)); %DEBUG
                plot(Q(1,ia),Q(2,ia),'.'); %DEBUG
            end

        else
            plot(Q(1,:),Q(2,:),varargin{:});
        end
    catch
         plot(Q(1,:),Q(2,:));
    end
    xlabel('x')
    ylabel('y')
    zlabel('z')
end

function plotm3(Q , varargin)
    [~,varargin]=parsem('verbose',varargin,0); %Verbose level
    [~,varargin]=parsem('height',varargin,1); %Remove heigth
    [~,varargin]=parsem('hull',varargin); %Remove hull
    [~,varargin]=parsem('boundary',varargin); %Remove boundary
    [res,varargin]=parsem( 'resolution', varargin, -inf);
    [contourflag,varargin]=parsem('contour',varargin); %contour
    [surfaceflag,varargin]=parsem('surface',varargin); %surface
    [pointflag,varargin]=parsem('point',varargin); %pointcloud
    if(numel(res)==1); res=[res res]; end;

    x=Q(1,:).';
    y=Q(2,:).';
    z=Q(3,:).';
    Q=[]; %#ok<NASGU> %free space
    
    minx=min(x); maxx=max(x);
    miny=min(y); maxy=max(y);
    %test if we plot surface or points.
    %we plot only points be default, if there are more z values for the same pair (x,y) in Q
    if(res(1)==-inf)        
        res=[max(33,ceil(sqrt(length(x)))) max(33,ceil(sqrt(length(y))))];
    end
    resval=[round(x/((maxx-minx)/(3*res(1)))) round(y/((maxy-miny)/(3*res(2))))];
    resval=unique(resval,'rows');
    if(contourflag==0 && surfaceflag==0 && pointflag==0)
        if(numel(x)-10>size(resval,1)); 
            pointflag=1;
        else
            surfaceflag=1;
        end;
    end
    
    if(surfaceflag || contourflag)
        xlin = linspace(minx,maxx,res(1));
        ylin = linspace(miny,maxy,res(2));
        [X,Y] = meshgrid(xlin,ylin);
        f = scatteredInterpolant(x,y,z,'linear','none'); %linear interpolation, none extrapolation
        Z = f(X,Y);
        if(isempty(Z)); surfaceflag=0; contourflag=0; pointflag=1; end;
    end

    [COLOR,varargin]=parsem('Color',varargin,[0 0 0]);
    if(pointflag)
        %plot3(x,y,z,'.',varargin{:})
        [MARKERSIZE,varargin]=parsem('MarkerSize',varargin,5);
        try; 
            if(COLOR); scatter3(x,y,z,MARKERSIZE,COLOR,'.',varargin{:})
            else; scatter3(x,y,z,'.',varargin{:})
            end
            
        catch; scatter3(x,y,z)
        end
    end
    if(surfaceflag)
        [COLOR,varargin]=parsem('Color',varargin,0);
        [~,varargin]=parsem('MarkerSize',varargin,5);
        try; 
            if(COLOR); mesh(X,Y,Z,'EdgeColor',COLOR,varargin{:}); %interpolated
            else; mesh(X,Y,Z,varargin{:}); %interpolated
            end
            
        catch; mesh(X,Y,Z); %interpolated
        end
    end
    if(contourflag)
        [~,varargin]=parsem('MarkerSize',varargin,5);
        try; 
            if(COLOR); contour3(X,Y,Z,'LineColor',COLOR,varargin{:}); %interpolated
            else; contour3(X,Y,Z,varargin{:}); %interpolated
            end;
            
        catch; contour3(X,Y,Z); %interpolated
        end;        
    end
   
    axis tight;
    
end

function rotateplot(rotateval)
[az,el]=view; 

    h = gca;
    h.CameraViewAngle=round(h.CameraViewAngle);
    for i=1:rotateval:360
        view(az+i,el);
        drawnow;
    end    
    view(az,el);
    drawnow;

end

function A = DTarea(DT)

N=size(DT.ConnectivityList,1);

A=zeros(N,1);

for n=1:N
    idx=DT.ConnectivityList(n,:);
    p1=DT.Points(idx(1),:);
    p2=DT.Points(idx(2),:);
    p3=DT.Points(idx(3),:);
    A(n) = 1/2*abs((p2(1)-p1(1))*(p3(2)-p1(1))-(p3(1)-p1(1))*(p2(2)-p1(2)));
end

end


% OLD PLOT CODE
% if dim_inner==3;
%     c=padarraym(c,[1 1 1]); %add zeros at the boundary, otherwise the graphics look bad
%     [y,x,z]=size(c);
%     MIN=min(min(min(c)));
%     MAX=max(max(max(c)));
%     x=1:x;y=1:y;z=1:z;
% 
%     clf;
% 
%     if numel(c)>250000; 
%         p0=patch( isosurface(x,y,z,c,0.99*MAX+0.01*MIN)); isonormals(x,y,z,c, p0); set(p0,'FaceColor','black', 'EdgeColor','none','FaceAlpha',0.7);
%         p5=patch( isosurface(x,y,z,c,0.01*MAX+0.99*MIN)); isonormals(x,y,z,c, p5); set(p5,'FaceColor','blue',  'EdgeColor','none','FaceAlpha',0.2);
%     else
%         p0=patch( isosurface(x,y,z,c,0.99*MAX+0.01*MIN)); isonormals(x,y,z,c, p0); set(p0,'FaceColor','black', 'EdgeColor','none','FaceAlpha',0.7);
%         p1=patch( isosurface(x,y,z,c,0.80*MAX+0.20*MIN)); isonormals(x,y,z,c, p1); set(p1,'FaceColor','red',   'EdgeColor','none','FaceAlpha',0.6);
%         p2=patch( isosurface(x,y,z,c,0.60*MAX+0.40*MIN)); isonormals(x,y,z,c, p2); set(p2,'FaceColor','yellow','EdgeColor','none','FaceAlpha',0.5);
%         p3=patch( isosurface(x,y,z,c,0.40*MAX+0.60*MIN)); isonormals(x,y,z,c, p3); set(p3,'FaceColor','cyan',  'EdgeColor','none','FaceAlpha',0.4);
%         p4=patch( isosurface(x,y,z,c,0.20*MAX+0.80*MIN)); isonormals(x,y,z,c, p4); set(p4,'FaceColor','green', 'EdgeColor','none','FaceAlpha',0.3);
%         p5=patch( isosurface(x,y,z,c,0.01*MAX+0.99*MIN)); isonormals(x,y,z,c, p5); set(p5,'FaceColor','blue',  'EdgeColor','none','FaceAlpha',0.2);
%     end;
%     view(3); camlight; lighting phong;
% end

function dummy; end %#ok<DEFNU> %Generates an error, if the 'end' of a function is missing.