function [M, a] = daubechiesmatrix(varargin)
% [ M, a ]= daubechiesmatrixrix( N , [options] )   
% Constructs the set of two matrices whose joint spectral radius is closely related to the Hoelder-exponent of Daubechies' wavelets.
%
% Input:
%   N       N>=2, index of Daubechies wavelet. N=1 would correspdond to the Haar-wavelet.
%
% Options:
%   'jung'      (default) Uses the algorithm by R. Jungers. 
%   'gugl'      Uses the algorithm by N. Guglielmi. 
%   'double'    returns double values
%   'verbose'   verbose level
%
% Output:
%   M       set of two matrices whose joint spectral radius is closely related to the Hoelder exponent of continuity of Daubechies' wavelets
%           is empty for N=1
%   a       the subdivision mask generating the daubechies wavelet
%
% Note:
%   For values N>=43 the returned matrices from algorithm 'jung' and 'gugl' differ very much.
%       Wich algorithm is better is not investigated yet.
%       Algorithm 'jung' seems to be numerically more stable, but this is just a heurisitic observation.
%   For N==1 the function returns {[1]}, which is NOT the mask of a Daubechies wavelet.
%   
%
% E.g.: vdisp(daubechiesmatrixrix(1))
%
% Reference:
%  [1] I. Daubechies and J.C. Lagarias
%        "Two-scale difference equations II. local regularity, infinite
%        products of matrices and fractals",
%      SIAM J. on Math. Anal., 23(4):1031-1079, 1992
%
%  [2] G. Gripenberg, 
%        "Computing the joint spectral radius", 
%      Linear Algebra and its Applications, 234:43-60, 1996
%
%  [3] R. Jungers, 
%        "The Joint Spectral Radius: Theory and Applications" 
%      Vol. 385 section 2.3.3 in Lecture Notes in Control and Information
%      Sciences, Springer-Verlag. Berlin Heidelberg, June 2009
%
% Written by: JSR-Toolbox, Jungers
% Modified by: tommsch, 2018

[guglflag,varargin]=parsem('gugl',varargin);
[doubleflag,varargin]=parsem('double',varargin);
[verbose,varargin]=parsem('verbose',varargin,1);

if(guglflag)
    [M, a]=daubechiesmatrix_gugl(varargin{1},verbose);
else
    [M, a]=daubechiesmatrix_jung(varargin{1},verbose);
end

if(doubleflag)
    M{1}=double(M{1});
    M{2}=double(M{2});
    a=double(a);
end

end

function [M, a] = daubechiesmatrix_gugl(n,verbose);

digitsold=digits;
newdigits=max(30,n*6);
vprintf('Computing with vpa. Number of digits: %i\n', newdigits,'imp',[1 verbose]);
digits(max(30,n*6));

try
    a=daubechiesmask(n).'; %this is the numbering for the Daubechies wavelets from Guglielmi and Protasov
catch
    a=daubechiesmask_backup(n).'; %this is the numbering for the Daubechies wavelets from Guglielmi and Protasov
end

d=sym(zeros(n+1,1));
for j=1:n+1 
    d(j)=nchoosek(sym(n),j-1);
end

[Qsym,Rsym]=quorem(mask2symbol(a),mask2symbol(d));
ERROR=norm(symbol2mask(Rsym));
Q=symbol2mask(Qsym,'sym');

vprintf('Error of deconvolution: %i\n', ERROR,'imp',[1 verbose]);

T0=zeros(n-1);
T1=zeros(n-1);
for i=1:n-1
    for j=1:n-1
        idx0=2*i-j;
        idx1=2*i-j+1;
        if(idx0>=1 && idx0<=n)
            T0(i,j)=Q(idx0);
        end

        if(idx1>=1 && idx1<=n)
            T1(i,j)=Q(idx1);
        end
    end
end 
M={T0 T1};

digits(digitsold);

end


function [M, a] = daubechiesmatrix_jung(n,verbose)

digitsold=digits;
try
    a=daubechiesmask(n).'; 
catch
    a=daubechiesmask_backup(n).';
end

newdigits=max(30,n*6);
vprintf('Computing with vpa. Number of digits: %i\n', newdigits,'imp',[1 verbose]);
digits(max(30,n*6));
T=sym(zeros(n,n,2));

n=2*n-1; %this is the number the algorithm needs
vprintf('Computing transition matrices.\n','imp',[1 verbose]);
for i=1:n
    for j=1:n
        if (((2*i-j-1)>=0)&&((2*i-j-1)<=n))
            T(i,j,1)=a(2*i-j);
        end
        if (((2*i-j)>=0)&&((2*i-j)<=n))
            T(i,j,2)=a(2*i-j+1);
        end
    end
end

vprintf('Computing difference operator.\n','imp',[1 verbose]);
%L=(N+1)/2-1; %Original
L=(n+1)/2-2; %Original

U=sym(zeros(n,L+1));
for j=1:L+1
    for i=1:n
        U(i,j)=i^sym(j-1);
    end
end

[q,~]=qr(U);
projecteur=q(:,L+2:n)' ;
vprintf('Condition number for difference operator: %g\n', cond(U),'imp',[1 verbose]);

vprintf('Computing transition matrices of difference scheme.\n','imp',[1 verbose]);
Tproj(:,:,1)=projecteur*T(:,:,1)*projecteur';
Tproj(:,:,2)=projecteur*T(:,:,2)*projecteur';

M = [{Tproj(:,:,1)}, {Tproj(:,:,2)}];

digits(digitsold);

end

function a = daubechiesmask_backup(n)
% [ a ] = daubechiesmask( n )
% Returns the mask coefficients Daubechies' wavelet.
%
% Input: 
%   n       Index of Daubechies-wavelet. n=1 corresponds to Haar-wavelet
%
% Output:
%   a       symbolic, subdivision mask
%
% Info:
%   The returned masks sum up to 2
%   Number retrieved from Mathematica with the command
%       WaveletFilterCoefficients[DaubechiesWavelet[10], WorkingPrecision -> 50]
%
% E.g.: double(daubechiesmask(2))
%
% See also: daubechiesmatrix
%
% Written by: tommsch, 2018

%DANGER: The values in the list sum up to 1, because mathematica returns them in this form. 
%They are multiplied by 2 afterwards

    switch n
        case 0; a=[sym('')]; %testcase
        case 1; a=[sym('0.50000000000000000000000000000000000000000000000000'), sym('0.50000000000000000000000000000000000000000000000000')];
        case 2; a=[sym('0.34150635094610966169093079268823404586785065672630'), sym('0.59150635094610966169093079268823404586785065672630'), sym('0.15849364905389033830906920731176595413214934327370'), sym('-0.091506350946109661690930792688234045867850656726298')];
        case 3; a=[sym('0.23523360389208184037666455378558522915088772238769'), sym('0.57055845791572181288006281482971077909519097095524'), sym('0.32518250026311626425346741451708064158683105235973'), sym('-0.095467207784163680753329107571170458301775444775380'), sym('-0.060416104155198104630131968302665870737718774747417'), sym('0.024908749868441867873266292741459679206584473820136')];
        case 4; a=[sym('0.16290171402564917413726008653520017177903569516772'), sym('0.50547285754591443144136667385662406509120752378972'), sym('0.44610006912337981158583475265059490355842186177631'), sym('-0.019787513117822321547681334086981511835723044018791'), sym('-0.13225358368451986802584241445339979336920964891395'), sym('0.021808150237088626328869997057481357482564676810239'), sym('0.023251800535490882302747575267604718031752091969920'), sym('-0.0074934946651807362225553368271239107380491565811710')];
        case 5; a=[sym('0.11320949129177917882419730369511962204324288677342'), sym('0.42697177135251416622350956786261524367244025641184'), sym('0.51216347212959853661722721854642869318360365416836'), sym('0.097883480673904674028636932907746200368876210699555'), sym('-0.17132835769146744322379801365246758939474199074694'), sym('-0.022800565941773648742400897551521370702939433713188'), sym('0.054851329321066823522124845264915551111310360810817'), sym('-0.0044134000541791272721475866909118943449019014039474'), sym('-0.0088959350509770957397513538539962769434149110056554'), sym('0.0023587139695339357624019834720718210065248680057424')];
        case 6; a=[sym('0.078871216001450708360703821762941135168922297209986'), sym('0.34975190703761783105607104498368409053436335363430'), sym('0.53113187994086898454751440735467346135081182230923'), sym('0.22291566146501775627367242887951971083116640048263'), sym('-0.15999329944606139494194938783162139381986641787842'), sym('-0.091759032030147576133204962992087236279668680437360'), sym('0.068944046487372298805285738485359895499675313717827'), sym('0.019461604854164664143361603520751203546777542675306'), sym('-0.022331874165094534628441049888253468379143199958742'), sym('0.00039162557614857788770574331926166815743553498792021'), sym('0.0033780311814639378568864701169003701796001846001225'), sym('-0.00076176690280125322760585771112943679007415134280030')];
        case 7; a=[sym('0.055049715372811848891338245570773718753558169756039'), sym('0.28039564181276256271193000426791213878968219763325'), sym('0.51557424581809866812219531601564908084418941798779'), sym('0.33218624110553967423703033284669297777058403546955'), sym('-0.10175691123134624248731404007214892195697519072488'), sym('-0.15841750564033283937650333521608321209944284614898'), sym('0.050423232504694086902554278422120807582699961629514'), sym('0.057001722579871576316565526265901305109715752680936'), sym('-0.026891226294845438763215203541631905797728153661822'), sym('-0.011719970782103288571916893042018605196489167923010'), sym('0.0088748961896807639198633424723380041995351155080320'), sym('0.00030375749770106935426452217945706613716313790044279'), sym('-0.0012739523590936865854219388671007836252793204946755'), sym('0.00025011342656124532862984269813832948878689038780110')];
        case 8; a=[sym('0.038477811054076236586116598041731005973033235563244'), sym('0.22123362357612491986161112450067254866945112696623'), sym('0.47774307521387369558135499895601473783369137873194'), sym('0.41390826621119589290841281040149531482246366701361'), sym('-0.011192867666880217741098871761485333118318937421193'), sym('-0.20082931639048905042015417677290377687885350815269'), sym('0.00033409704622011878047354968026602685670689761282777'), sym('0.091038178423657745431264543005154633419943568172281'), sym('-0.012281950522848409255966458674470382034856128610301'), sym('-0.031175103325139428135588152336066442815558488268462'), sym('0.0098860796483507589781295863786448272554331604960422'), sym('0.0061844224098159223674466603000443163931943876053119'), sym('-0.0034438596284418090839115564922422990027347441122709'), sym('-0.00027700227447938932237662547818956319398291744757913'), sym('0.00047761485564962615490215387154141623704513773970758'), sym('-0.000083068630686612690616183620207030416657835888704623')];
        case 9; a=[sym('0.026925174794662800146933495667226733319426277390439'), sym('0.17241715190697794201117940367729789942456466903373'), sym('0.42767453217970756629489137813921718341359961357137'), sym('0.46477285718314734012271194615270142209324736669295'), sym('0.094184774753183779083317674885956202344982911723377'), sym('-0.20737588090093849234446413815720258012001648028231'), sym('-0.068476774512383098603765423918985463575794821554553'), sym('0.10503417113950619215454372045443005066266943245061'), sym('0.021726337730614543616172588029105197726916889888850'), sym('-0.047823632060097026094214736020971291285668829736166'), sym('0.00017744640661651891431296804713011036355564732878736'), sym('0.015812082926255863012538414932481389662714547659946'), sym('-0.0033398101131385774822430226016659937532607232270480'), sym('-0.0030274802875450659331670431375232387464592595392058'), sym('0.0013064836402472456978901998557222737764426451474944'), sym('0.00016290733567609221304511268860500591473957228931509'), sym('-0.00017816487951077766750985810370624361586844026871594'), sym('0.000027822757017154857827319410181342394208981431131086')];
        case 10; a=[sym('0.018858578796120688885281887971592801996541071494754'), sym('0.13306109139692089491682643666653660184695482506315'), sym('0.37278753574323338488883893804571954763780607858401'), sym('0.48681405536681994568330981564493636165998227343167'), sym('0.19881887088450868951105036816768441944382208563732'), sym('-0.17666810089705630024937705278760724723613314661377'), sym('-0.13855493936048315285629029382206656295954300881632'), sym('0.090063724266696666342768535453956910807500174256998'), sym('0.065801493550535007323661506109025565127999598799861'), sym('-0.050483285598389717069716017924831393956526295508597'), sym('-0.020829624043800806981046718807342133346119675347314'), sym('0.023484907048698560846021882595782361119003690000584'), sym('0.0025502184839072387326709511160807910246097418406884'), sym('-0.0075895011679282493938418689338363378631751621157121'), sym('0.00098666268248160259413354418237330832261460903522011'), sym('0.0014088432950973381057283828632621407138406074522931'), sym('-0.00048497391992820548148456440705240912385457774161982'), sym('-0.000082354503045388977551555079619756354521666135068194'), sym('0.000066177183425553383184381443984671876124076513402507'), sym('-9.3792078137502041685584985796407369252998315564355e-6')];
        case 11; a=[sym('0.013218864716656699017621057919059734435011275590744'), sym('0.10187076760095236152575432101157102928676795412628'), sym('0.31812717423038944182764353739296501220470107983571'), sym('0.48485376831317925854429780828948335097056448130913'), sym('0.29130279889030213596402624317216472904162614577199'), sym('-0.11474592617764701178388115574044354851446936678628'), sym('-0.19391049139549965435258466309428018649073232700292'), sym('0.046699869067766514582288921237691795045957911616174'), sym('0.10593308991817982937284211851153932897378103337012'), sym('-0.032866291452253755616514223448277455813604510208291'), sym('-0.046979315898752111614297533741059803421638815153143'), sym('0.022157254782979739220165042779232114556899579567976'), sym('0.014736744799144315165981509512462164824644326097659'), sym('-0.010864569054491985210224643396242814898711045455473'), sym('-0.0023623439640957613511559053634333178691923832709095'), sym('0.0034849175451188578275031124430190510080502346556028'), sym('-0.00021820810309393059144111440385402899643136704817187'), sym('-0.00063146279630336975271265705643151761493688879142292'), sym('0.00017617743895394249421498899211431897235418900716083'), sym('0.000038494238881444457353833653457579584275184307928961'), sym('-0.000024490632184905932850238897677951674123157198243328'), sym('3.1779318179462059696402269404163892064656583767811e-6')];
        case 12; a=[sym('0.0092717665182242590636048131800250740431363488369142'), sym('0.077475054501195340230846644859813869973736157622015'), sym('0.26683037502553632370051891896403300896594022793169'), sym('0.46470967332296464786392198997511156456813700405281'), sym('0.36478682721875598139161226100248962144173492703581'), sym('-0.031652847098043248813696948470887682655145945917211'), sym('-0.22357192871367185857229663642410871413145656215751'), sym('-0.016814474057346319427388712905481454391242824935437'), sym('0.12903185967285931873451056102577898789207623810437'), sym('0.0037897880609761817336597645521396001253950994803554'), sym('-0.068187806044435705392191947083441367879268631489380'), sym('0.0076714935738680096970856059842153885108552546087470'), sym('0.029377654549832257429109655183235663711529815168981'), sym('-0.0086398896141577099956792864444478569134854119473713'), sym('-0.0090798345737495069865947247042049231715973457049024'), sym('0.0047457464610460967070563222765255287968052871131970'), sym('0.0015900054283329058097888964629076757740419863773441'), sym('-0.0015411417883523083487890332232236869760210269532745'), sym('4.6281045428009215175697793971926301284311910500446e-6'), sym('0.00027481921624960187921846688089407646206545403301964'), sym('-0.000062581855783996040953244804876315653061327912435326'), sym('-0.000017141361391240446161834301529676097603774562577481'), sym('9.0346695572199413738774187640963767958926180742250e-6'), sym('-1.0812170090510800729791831296714034952725942747056e-6')];
        case 13; a=[sym('0.0065068910267844533284038379272700216369517305639107'), sym('0.058591747440081817553512278659503989574046673112154'), sym('0.22061471510490846607907295159971517693071507421328'), sym('0.43208173603809641335326661701682943409099893224612'), sym('0.41640780862194787550186824854574568590561571084805'), sym('0.061508196848064757567007974351288064176232215070641'), sym('-0.22271947893276750950622141462599339011446671586335'), sym('-0.088089051091952171595236396651347493036607054855297'), sym('0.12690875282526163181154853538105227094518861306901'), sym('0.051582685669034702116664918773827678538082657067833'), sym('-0.074817284321885443851855141577542514395784339312734'), sym('-0.018730131841541182438580836579563355642249743433603'), sym('0.039696604949877310044569624075603511102619830219992'), sym('0.0016828945198810073343378614921231923015025514866218'), sym('-0.016851359189579381471702451116565511078995015864984'), sym('0.0027746456074236188840955718962131540656079869944705'), sym('0.0051304764673890256658010816058900344103717159424155'), sym('-0.0019529661630611771576966531705639839634361260242017'), sym('-0.00093032194492927683094101516020801708487019809875569'), sym('0.00065925412941365848295856468017678894040202370890200'), sym('0.000034826087400578909978745281368594855516508157987518'), sym('-0.00011676382757878083728410148473058842954284362583603'), sym('0.000021693001959227387073991135433262077104516346890074'), sym('7.3835599157218147471143589231987172683170779710750e-6'), sym('-3.3236963669570675969930717691251899674302217195677e-6'), sym('3.6911222161492220708665731992066769441117422065565e-7')];
        case 14; a=[sym('0.004568725425915112941352971047152926387330934771901495'), sym('0.04409854388947371015485924296992841449430697937662630'), sym('0.1802063525433701474921683579162943576839785284448332'), sym('0.3919532612958056294198821234893301808158902895684553'), sym('0.4463172083045955395414495174891194875276450487870331'), sym('0.1546235261610489699523909905950665796722556691429287'), sym('-0.1921128176870585691101130891865254540984800828837178'), sym('-0.1541729875842859909081359451158967102296770796750505'), sym('0.09786019420756724233732028975826591412144091140357611'), sym('0.09898718291850822556853107121679514718663466120624253'), sym('-0.06134039007701433230655861550513660873001017246948884'), sym('-0.05059275162372557250287198294911337594871773290006976'), sym('0.03905854655114917769722555336599950489937821041829698'), sym('0.01907873678048827447448808135144537713158573768191363'), sym('-0.02134426676671001613275880179288207530959140842094702'), sym('-0.003970439599613744742914807257525043797363345585283348'), sym('0.009043537416564040790750420119442972996831031610709048'), sym('-0.0005276565075618465231083679213381965943891841771488362'), sym('-0.002722105748697793402035405360107591666773487509629809'), sym('0.0007507289661578486423587718977655222459310779054471581'), sym('0.0005006465593834682135078585427710787903106642384731184'), sym('-0.0002735314931245873855730635578864081946861751608535862'), sym('-0.00002954097378352747425941118587280987335029628599121613'), sym('0.00004861715681159354880059729037660145322781275608407932'), sym('-7.309510712953855614664292213558057453417313066657281e-6'), sym('-3.103990103457451340807199755457238617201905316383124e-6'), sym('1.219755432463267565019083691855328743535208018301262e-6'), sym('-1.263698790522473659048091926309963815082339752343489e-7')];
        case 15; a=[sym('0.00320923054504094117479653674774131457619798932935625473'), sym('0.0330525715043556633020542416906247141989837068827203943'), sym('0.145680871311459619384183178579916560606103927749523736'), sym('0.348343266402768718200494145963796211170139336328470387'), sym('0.456658850926114369920332566513566098486745305504556624'), sym('0.239710991659473665519671765394408295058136859096929540'), sym('-0.136615957270939176201528026352593954103769362874533543'), sym('-0.204270842999279062288354991574485364083927225368543228'), sym('0.0461620186552488996141210288453925244117824488702357289'), sym('0.134454030894775741890544246382944629637638869599430309'), sym('-0.0280482224263426287018500140402943142372173090701700528'), sym('-0.0785743674037231343738288446070048731781316123008686922'), sym('0.0239547581955449275563860635463136892189880144420431685'), sym('0.0387356987954380225754661791054965900696734593520153824'), sym('-0.0182200256128233631242470763222135516720316448093265277'), sym('-0.0147149275918222420850328395455318057079020166856715566'), sym('0.0106659407243447808826962793600532210475626691441128048'), sym('0.00360695194567919668805321087779061871781511157704064635'), sym('-0.00458752110213758760835648319020785410718360574793580114'), sym('-0.000170947654013407458646656704562161498559952047033060604'), sym('0.00137413756457069503060753081625016063033577139026826592'), sym('-0.000264091905264224590802730961486021680777994445834402198'), sym('-0.000254251022567699725179263142623098779063249646846092471'), sym('0.000110235465186036337617800852599797941626726124064697230'), sym('0.0000182381924778305566105777738696550812339611436505801352'), sym('-0.0000198932445668525858720759836545301905757071983298973735'), sym('2.37799104125007897466314101951921088939719910843162094e-6'), sym('1.28076158801730631160774122572830111199950387350569930e-6'), sym('-4.46671032858837547562276189970370574312624043577755090e-7'), sym('4.33694038615623249413678381712447484395817359745301571e-8')];
        case 16; a=[sym('0.0022551197430154216122526175461497910694318446792032211141'), sym('0.024683481513992199765259337113597207574847627992041143766'), sym('0.11671807418666671103130867271159963118715300869195896030'), sym('0.30427704435525668861280121400079664260120669510695177096'), sym('0.45067898444863223630663165956096709652901892934745985007'), sym('0.31133222633470996784373921984973227645622499150252582416'), sym('-0.063463603935380508286898570602278229594048391281119655965'), sym('-0.23126868475161214966667493157830846522683487564059039405'), sym('-0.019741154289441717119966547040837282271293879886753609322'), sym('0.14933437181349019900110699225724337262867791540553977513'), sym('0.019332485898974278536098984189118030816963754275789380765'), sym('-0.093612668613967065948037526997178244039764689691602318983'), sym('-0.0044121502709989711900356942753207754453983293122965266503'), sym('0.053686542163315877431154723161915602078183580248097488520'), sym('-0.0053662152384702058897505352188574309928812466244060021903'), sym('-0.026084036154928570534637946769647149267301588491061738238'), sym('0.0072815449624709942065491233971259827848909575646009649567'), sym('0.0098950888551420371474264324585412423373647749984804682481'), sym('-0.0049426866983827048996576491155265891159905936505972201131'), sym('-0.0025768948329014562432338010725307152245804475108983482410'), sym('0.0022118465445610256686085719454012517423143410671533666990'), sym('0.00028842672118435126039737443265978367506261318352184368846'), sym('-0.00066540286021617792710823065291775923184619628750163807713'), sym('0.000080780953512439459162146163735163510589107288702684974114'), sym('0.00012359324636375262825865365267052529924269253184742712200'), sym('-0.000043158945606269733562647772231866568300696626643885247312'), sym('-9.8610771097454476302731777515936999046017003044538193910e-6'), sym('8.0161928600465908424555584266652724497753416436218734583e-6'), sym('-7.3791637280048482267623220716864150076698986837428478667e-7'), sym('-5.2068916473228815060936743094752034864221658749485813565e-7'), sym('1.6325568841125616189331266451956384847757483430939642994e-7'), sym('-1.4915283562697592431439320568287476140890120441705522253e-8')];
        case 17; a=[sym('0.001585196932544961498981730878055742385345541762920239926500'), sym('0.01837444809962204829903622517387323225610869878095266240271'), sym('0.09278294792169978362485785511441636367141583006348456404963'), sym('0.2618775084656810472930444834177270436893356778189680445141'), sym('0.4320398502326276582417977003598709674157176602326467476682'), sym('0.3665045915605473389252707859838986318462531538902873743065'), sym('0.01931460080007877415350509902043914516604300060667093740134'), sym('-0.2321578275723994224311845027289868098872026468650218632478'), sym('-0.08951954328838730546877768031630304801544881590641716571162'), sym('0.1395196558813349153595256195134489726476674909174250931028'), sym('0.07151359021600791699899179907475536228772543499445048193718'), sym('-0.08967223561728935249106882210738746785945217325234508125297'), sym('-0.04036972996912554068419441826232471457862702746047475463598'), sym('0.05735059315798278472921013818727847145874508741768336718657'), sym('0.01577720421565112891810805133627420431928216779678697176734'), sym('-0.03317917437486061362662891079166749313796869903271239067911'), sym('-0.002312914840337499681271965269724586492095821656944742291874'), sym('0.01607513687381023456910400288366925669544968744943648860503'), sym('-0.002151718850898589684793985687921044835680589437124740564706'), sym('-0.006083184145035973673014352493967301576963013785009725284485'), sym('0.002098690587117339358244887607162100401632005705242048655441'), sym('0.001627197831628803295198854437062997437427122645243996935495'), sym('-0.001016003058542236222391289565192340802883397227671564935211'), sym('-0.0002320247296025228568563200542210320930128423470996950773335'), sym('0.0003107489840722548313824292432846709285749182274839617564291'), sym('-0.00001810908214143688854115636491943686496814154098747841172287'), sym('-0.00005801671982755894931488276715024807317841643349003080084526'), sym('0.00001639555327120307385156477803312335633413529115949861579002'), sym('4.943101361117129938573492586099135702692323539613471792898e-6'), sym('-3.186182481280885415351214537665413469146833822268378919062e-6'), sym('2.133022685012786783645094752785196245249997022755347060938e-7'), sym('2.091410386670130020080273006262948751475724610205808947255e-7'), sym('-5.956631070534374226876770395143314970859080484322835884948e-8'), sym('5.138893560295465733353394851150840461873380956082505515218e-9')];
        case 18; a=[sym('0.00111461964471310971687682508092312610080089409964160779006047'), sym('0.0136390515812757524976196562010708361891681276189806200281134'), sym('0.0732481066357466386001860851831957164328610289685681277366922'), sym('0.222511613316018898302799096005916529888236804014112811160012'), sym('0.404342613436024308462502984316548742162310748991387334743985'), sym('0.404324827665455342295419562150688510454828208120106285319895'), sym('0.104102460821322564063058035249262695292233320036909187992062'), sym('-0.207644763527651356992246763344587673778009164184406347417711'), sym('-0.153075136432634066074745432319175390645689576150754590866409'), sym('0.105736488140062136632369892392610099668462778278133054960088'), sym('0.118144329264449763650597170649660365438084195836701741110483'), sym('-0.0652885014027941163155308024337592591933097505754942819025755'), sym('-0.0754852375200636897985460212189132965477715661753371456304487'), sym('0.0458821905957560210980523770573870360950772094118069918346852'), sym('0.0403413241510731148101083765111040847040697463798374756480333'), sym('-0.0314847368796732874726394411725921602638178188646658240974868'), sym('-0.0167819140102396727023580766761135638902274424241664253036150'), sym('0.0188590370196395959627233421978801511522461819118598348413814'), sym('0.00442802142541865583907092843789822405256538515180795044314946'), sym('-0.00922879068187636688986944619147613985744202051757953144150287'), sym('0.0000838841013934645910481614250741884530402712840229414661976458'), sym('0.00349547178516068771400742424111747913590985835360118149888043'), sym('-0.000791063455165306365793801075581621565501804745144332042502625'), sym('-0.000947944733387044935546463025303215582850496451167176666041037'), sym('0.000444392346167689437333662020717673432941474285004658344469198'), sym('0.000151024970763216326422649221743342872752762079266499803304699'), sym('-0.000140465738412527596063201461335308397848287757899090860716831'), sym('-1.08605741339194004379012498839623582313908845295896937995956e-7'), sym('0.0000264545467550326896089592188280416401561108409645215398649903'), sym('-6.02497583402386191449571668564476700316411147171254919947556e-6'), sym('-2.35652843923626069408534776045766829688456291908281953575010e-6'), sym('1.25066894469577789618192117188888523446888622963752105781110e-6'), sym('-5.43880563341393320111484118091403801645065578225137942945912e-8'), sym('-8.31627413510800165853838841921926203218677025867632515432044e-8'), sym('2.16999464910771414411540785901466525504479329432220243238113e-8'), sym('-1.77337745986554180510879874908328134819276416023271202759898e-9')];
        case 19; a=[sym('0.000783947907642271279703213047928067306821610203622995713985842'), sym('0.0100982615573282035785308840355937372754200686856774462244911'), sym('0.0574723050520547321655660703800441193014634079640887877918645'), sym('0.186950852951264758886140974766861400302700058401309150837369'), sym('0.370832518806165386380885358192815785627117336326289656659546'), sym('0.425469366958876173369324681723937844370843047780517746405145'), sym('0.184480590196892790429039608827253803231748521901066471801415'), sym('-0.161284971580061839367363121570398218785872280899941803810621'), sym('-0.202118434839629161005305919424905634804188356704349547974213'), sym('0.0527871261415669064017542829705129026973779421859174729778481'), sym('0.150153943475092198232260569158995141844964730488195802486948'), sym('-0.0237011882746038060450466565391182142200933238022299365602432'), sym('-0.100964733218325007768960867697773466834605795736949487675395'), sym('0.0195050813820094163826479567101878047991121683284245435363560'), sym('0.0614523561844364936579963039589502688302577350769429829507967'), sym('-0.0187392038622887543200585866088573754342643609092590312613259'), sym('-0.0322965551260787748304478864277901306531602627121244040325144'), sym('0.0152903125701182515005385721123134033513281979581067769604150'), sym('0.0137005827158546992278013251698366229725957267074266506839145'), sym('-0.00989128449246532707347652649940610738504919184143191479743567'), sym('-0.00414854052959815612009969240643480286469627603862906205597640'), sym('0.00497856020790144776541562751087498264058692589343621218624352'), sym('0.000543732841853969100041996638835717772607135205541854700892726'), sym('-0.00190038610306620532754573238970174855209561369550439469156703'), sym('0.000241695216728815734468676725690121654560086432506735484059329'), sym('0.000520290951863546920892447977543699825211958319966829269571327'), sym('-0.000184325863231862387410711603963641797253615425803927897589057'), sym('-0.0000881060647764363886134823804961596250656664441275021826467034'), sym('0.0000615979842012131168718995533265863470320139856588608731447353'), sym('3.61045221381270009920935008986890958009704352545296841027313e-6'), sym('-0.0000117663814998579159901296439881609004182967832122580082625941'), sym('2.12907328596399052080807837615694381548461756758622962874337e-6'), sym('1.08323913548146408301483707266681421992924445936834151211834e-6'), sym('-4.85270106323490621558324478813926579157088263755646167824687e-7'), sym('1.02324594905566076361751635964501315447473229499975735768302e-8'), sym('3.27881014519594629110020092927064177088473159501679600983897e-8'), sym('-7.89415472131711046546105666029919717488205442984170118130329e-9'), sym('6.12838758557397308074155544106625936618993171186569021203046e-10')]; 
        case 20; a=[sym('0.000551510489234779724655551080584426257531704198365205337409917'), sym('0.00745954847671534189514874448783652949637067659612330766425457'), sym('0.0448473852511033815039774838373393846825623503334392372918602'), sym('0.155522559960694499343045277787596303294152027119033412552498'), sym('0.334246678074351485891825748447818991173269837573161594102435'), sym('0.431683909122018935229506836780124047219920623432483955634713'), sym('0.255620726853106104889395411540848181180026839682686738082119'), sym('-0.0984378114560587225591656375667821138884670467016028113501174'), sym('-0.231073162589161137269696651435421801581358448626745157065594'), sym('-0.0118278375728545728220849331771762365550401631275848322011034'), sym('0.161426150118965572200006042971009831931873766727852452217606'), sym('0.0281783795022452120441220377410000971300362603080907950491234'), sym('-0.109925936819898162714083440982178639391698027344078244948933'), sym('-0.0174774362205507113474033798079189589989154346227090074839899'), sym('0.0723311682874783215549459452130463648647580543480696293718525'), sym('0.00398259994611870875673195139468105468550850252126238217591754'), sym('-0.0436446808791081621593596292002488550429458789758493498252616'), sym('0.00415402734644541575663031142482993099839184449744917674006744'), sym('0.0228355181918767110399442743038332284852058212657151690535266'), sym('-0.00621499125574272981189968810459486482572636877729669484153386'), sym('-0.00976551668333417817602133391284055189075843889666654826614631'), sym('0.00475290824603630148282511510936685653346049336612335420637569'), sym('0.00312579549840264658629938606823816713709260390529380329306072'), sym('-0.00253249887775065756664803227727193212333209418797747977414476'), sym('-0.000588003251381049772084348115239115486589063936242198809453602'), sym('0.000984688350029946874386131521012639565457931845738607319306080'), sym('-0.0000378285146341025677450625877865876675520369639343289324143105'), sym('-0.000272310179272358036148016959384505350911930562239385396255060'), sym('0.0000717945948462886379899356030715413984916289810524697497149660'), sym('0.0000479013991140791511798380803099759557447171240573156884876748'), sym('-0.0000262378065245551173243126163689501888038906476685949064609818'), sym('-3.09440100039819241820956162838794415502132850649837515335649e-6'), sym('5.12033576846949243117564464609658179578813948554690957031867e-6'), sym('-7.15587827004521312250242121815104758080215370870287550423018e-7'), sym('-4.84161641436314678883517831843570522856749048219837497515419e-7'), sym('1.86246568152704814733508637789340258198547716483718760536778e-7'), sym('1.42434076234597600902306394771068055768334008357781183438022e-10'), sym('-1.28328796766333625804150849714677729512562393254199295823181e-8'), sym('2.86811494633492510675517373635655219121677678882482438888205e-9'), sym('-2.12049761747948249723431437960222184578551885619726874627702e-10')];
        case 21; a=[sym('0.000388076118384653442002659468872962283572625306591840792150539'), sym('0.00549891420876019437385730793969407289436933951176557053118903'), sym('0.0348234332129710013344948887378798774161508575897119655728784'), sym('0.128240620982346070078396404376127238396058344217762664159283'), sym('0.296764191848869797823532163659001239297397062472839181186845'), sym('0.425329038653580483150362481322125530844483307940129916679560'), sym('0.314372923408797969908871563595735576114040307455520228719633'), sym('-0.0252599187051431627108148632230574519188463183805652463055828'), sym('-0.237350353907845827608427402189040181109570819312485108413323'), sym('-0.0794767314915613900307496062972523374093705870786490262459092'), sym('0.149598712181673911540799345739016893445682063025680828760829'), sym('0.0814822467451959574500766336273013147255774097099578982423906'), sym('-0.0989528234319311501818101193545954932140115780289068650889814'), sym('-0.0578243238196932663529639700877113195790442302516542068606863'), sym('0.0683067910631727453035984294853581681190859175272906979344734'), sym('0.0323313302642236527708804351874102055650667033635341608812251'), sym('-0.0459460343349402009983585004278164041157416280832091171226069'), sym('-0.0131902703371170830179029769842869730132625756166272453209080'), sym('0.0280911147261150264180120494744657078394756094624852455437858'), sym('0.00237429231318054220164311786279810420005096137597806731433088'), sym('-0.0147729128286123583752749597359711834596660437850269668584998'), sym('0.00169951058648619845237814041907564831195698745161459598475876'), sym('0.00635605867538731597432484228707342114499084163251064615560472'), sym('-0.00204448212456480111830279845637653268749470857842442270191489'), sym('-0.00209188634421564035536243123158148696765574307202143803708054'), sym('0.00121382447906248563839045110944886316839183965117527277645284'), sym('0.000452137157728190475177749935386908041004252407324851805393579'), sym('-0.000488378230458441998027112062953204037831285284145530172791064'), sym('-0.0000226020055437508776879673067426445368106998472984434524358140'), sym('0.000136941587585634985749858374801810466918393233874755044577808'), sym('-0.0000257070102197706295662986410029220965964713866017144292425361'), sym('-0.0000247463754987252267386889847187003317308530905256649529492561'), sym('0.0000108575009470243030985357790554268827396870570613304954945387'), sym('1.97306164645474352680079640093208623799815233107597264386346e-6'), sym('-2.18497209103309173909918756441042166838574633579337641088812e-6'), sym('2.23876755718154926419918000735044441087846289013532829408707e-7'), sym('2.11576010163836427417025660895729356656061811649303525622138e-7'), sym('-7.07390245467518387361440295631872066962725480911092606120783e-8'), sym('-1.59382927348753590936769177799701425063360929444265238133047e-9'), sym('4.99078337884662803900582898260903506985559920114217188008078e-9'), sym('-1.04082879475457695774550057239476026010333372504938006442510e-9'), sym('7.34546463605226623013726438521998122431272414085887515912924e-11')];
        case 22; a=[sym('0.000273129350310384739167993752069986076432352457682771370018895'), sym('0.00404596221078030550829134031416731573687057031923145014440235'), sym('0.0269195107792125308156328129061755829930592086696807504784533'), sym('0.104911694271370164393456542192167003636065738059525537613061'), sym('0.260023445701494424431128996231003525221518465600353402860184'), sym('0.409013706557088892019578519408372446811992541691133819222713'), sym('0.359140305350975294979936569723415862548448966896824503913529'), sym('0.0521310947265405296152981550685809700346844586757228462905889'), sym('-0.221131085678128490893846245296838312714992434618316878347704'), sym('-0.141823280048543000990899425891734718565008735632221755669832'), sym('0.116031406056814309319518905360413252688019353217936070110991'), sym('0.127260261661534773338632030955473268419228779094202683986232'), sym('-0.0686677040815248024663941637606787334217594659487092562698740'), sym('-0.0931741437025765130228587848675277178298943119801390228445169'), sym('0.0481372235452883146651544260931021128065749472630118767856121'), sym('0.0597910942283258303349551260448876595918857086924628982124540'), sym('-0.0363200125243130115110881613363697207834960475491033294987905'), sym('-0.0329022525773452071786370249794204313905053380369875415345165'), sym('0.0261423363517033252147699851074287382866317812096499637604265'), sym('0.0145570005657562882317454712548999332077726464957047776950788'), sym('-0.0166028681729603868516449487329115257451608927983942196976591'), sym('-0.00439380798960641737415461160093118412516690585537957298853607'), sym('0.00888460240563622377477647920299194121371757165824210530348622'), sym('0.000212229189772443955114340849928436927087872027522892258803411'), sym('-0.00385775679947651860747013183533669072067150202224592436691611'), sym('0.000738403850005315182303185797426193693149406360737475679097498'), sym('0.00129189151077825737457628491278257416537257294560863942126219'), sym('-0.000544960823901218844413545181815292004402056088210197951210459'), sym('-0.000299662944207699034327296021351341960613080628311708752575450'), sym('0.000232361945152231288623541566518024457279018542610715908243305'), sym('0.0000307301529285254908932335288732563551664816747472565099971060'), sym('-0.0000665049741075421306546934562200282132626273174288747665627834'), sym('8.04287977767616197149949248012997906242264920067276852009624e-6'), sym('0.0000122851013593789755146079269388686794415927511513975591509691'), sym('-4.36053611741610772123493656989412342512154827363933028497082e-6'), sym('-1.10674877800545210014488142926825937068450844680671807098804e-6'), sym('9.15832015601322018247472473966948404756502009288760129813509e-7'), sym('-6.20831259645705164418217800125987517274933320712436730115437e-8'), sym('-9.07455749892735910934568304022773955254165093097612377761248e-8'), sym('2.65959035425024609765778464395110983391183095035617099433296e-8'), sym('1.18806059397677403992057274622070421888802616560108776221305e-9'), sym('-1.93013503706793914736946592149197643439964719545418271427317e-9'), sym('3.77307852479763801673102946244400970896890798381343315106404e-10'), sym('-2.54707887138000179068819862282178505165229589604334801387654e-11')];
        case 23; a=[sym('0.000192265299501176477983036499966675201885075043801667964112584'), sym('0.00297179224199454577627264995022440898971163894879530742432275'), sym('0.0207253023430923705554424873590515135736115995709802291496363'), sym('0.0852173497627494554900342586598186543444768445574710217736389'), sym('0.225178729949735955864036433326938149610544021114383234415935'), sym('0.385324509941135581774635318766690515083817085362732919226862'), sym('0.389628930101117759366138879913339433523365185568841457250787'), sym('0.128263955452002211040207403117312194117103009662518536667199'), sym('-0.184832160421386305425022427263907825601392436445769180994685'), sym('-0.191910264353865874053721162985311641187381821481529769616119'), sym('0.0651425000675489975004879106594008174444677922374920549871162'), sym('0.158090449837630299586746429611468771092626052312677550072337'), sym('-0.0233610028734638200584126158053257274717765995541501150906576'), sym('-0.115973517646556680234664734933065068137094179679593823806753'), sym('0.0143422995757540746797253950373061294506450955491774465626286'), sym('0.0794060010495651737063026284063194622491149367119881791185223'), sym('-0.0149384880178753123080255123668829450942860250609746141189537'), sym('-0.0496441226720318589979717943475266759322933639864668728360273'), sym('0.0153907849660104942055271014672858096700015494848245431238876'), sym('0.0272203106707397251451862240883452496923141539955564612686915'), sym('-0.0130981021134269979166897224270161933046765796014720930241505'), sym('-0.0124006030416000506309543654731805917300770711475563331316562'), sym('0.00901698602729429462262083996190553439639400594541720650705484'), sym('0.00426515542666875828208901757238225013837583253736455550502098'), sym('-0.00500300623749749895119430131913714246650233621507335807437244'), sym('-0.000802471071944686744236010060229548345348664221110876524878302'), sym('0.00220820711447418141523957738064220830134437231517434768900193'), sym('-0.000174302811877093063183686046880551407988497607467542394891462'), sym('-0.000750403798352684288125919058703769549449415583556529732403287'), sym('0.000225864396445119271422598118318438450364548233512901980624103'), sym('0.000181558470968852972978278850729128216563188029027593293925088'), sym('-0.000106081467707955431413025814888060432124777367302492658321755'), sym('-0.0000238923945012308545873524153839146284721874742563512742493883'), sym('0.0000312970496173304881104597702632755812868855805291411752765245'), sym('-1.86337336832438809508352831005220818378788933527818905193858e-6'), sym('-5.90283942253150686582661912512636220251945715867640827048645e-6'), sym('1.69533768493709153179991450240845493064193639572690392856279e-6'), sym('5.76120541589741207861832447841721276375162950213852633379957e-7'), sym('-3.77524692681521266459629421534705054865579125304501670892540e-7'), sym('1.31033376778286473419935374633206030798767454803633985823688e-8'), sym('3.83078576226384083819338604932964832478971178489500109490629e-8'), sym('-9.89903882047958049457063303362963194846018453512290712370988e-9'), sym('-6.69834185857774453488455806986200349932863428904107691405434e-10'), sym('7.42777810682186542732237788265338022536639756438414256441393e-10'), sym('-1.36641675820926275002383389124241191356057102893722460891768e-10'), sym('8.84027232954236623147398411773437792888889763288393153721238e-12')];
        case 24; a=[sym('0.000135365553011891673469593362354864483313496856699214776775189'), sym('0.00217936088078073860391590448737000098429651684291223740510292'), sym('0.0158974150353856979099852703390058210836142883197084675444159'), sym('0.0687747865113215936676883628635044250743194008198694617967142'), sym('0.192975745197759623661785629367123972178448032658735496759101'), sym('0.356644183212028066406605187958805092298949644426892028240653'), sym('0.406543422006769483216467412297595883839345668880081616830450'), sym('0.198686790107011309861159667937152830076481884133121088051418'), sym('-0.132420881730839081266326208614539909459682238410702021207062'), sym('-0.224819707191779580684785199990173260747327335974219926196469'), sym('0.00337757592730861630113513299574787519112459806607911749501901'), sym('0.169166379919920267619028222481375366335520077064934319288672'), sym('0.0300723531247425461507603673138780990752039481844628527328386'), sym('-0.121039251725962266058473899510871020612620469957117104892819'), sym('-0.0274196023921950610661284480512071050147657833536589577154966'), sym('0.0855714488172175740729975629088522436629171648754223586494842'), sym('0.0148351806738011489189318983543279821247356479640677041214110'), sym('-0.0580970628439822164043879663553796567921296778218941315235426'), sym('-0.00323744331382062334290845247197158936280720854456844787474962'), sym('0.0362757234161261609774445059273116009164898606747847943842871'), sym('-0.00349643756762468719338368892362607867832618208837060997047176'), sym('-0.0199496793451474229266959429091454763485194312893910808702282'), sym('0.00541765549807765558316989167886157887914721303869441328446767'), sym('0.00922772289723164039764302680383824410973338609134714651584524'), sym('-0.00444871661353675636628455983414570719803325803849521461447736'), sym('-0.00335633097617747448183942855080305870142766668012408533517836'), sym('0.00264178358748965735677275179170495552217384550087499371894573'), sym('0.000815835010734466073317277826672596462600432845143019183708596'), sym('-0.00119957612068725761730842587755347596475998535763523920073389'), sym('-0.0000312271425875100668466331878582783481231605969827180445156494'), sym('0.000414454418280907492780134187887410662979572214137911146168177'), sym('-0.0000835258032731214964921377315058388201518271270049467218227155'), sym('-0.000103243234021356489053941071549218182857629488007632390414826'), sym('0.0000463818818729101470546849505353263059930412855153981900862509'), sym('0.0000154378484166352513208700499211557414450463249484021254848928'), sym('-0.0000143039802928931554138025118740095011057822371709784524781692'), sym('9.48341740238022912882484422980528474357154548064471364511392e-9'), sym('2.75849450351157215744631220495018251305003292592892005467410e-6'), sym('-6.34999789485066114058380002146086652471685799407428153912673e-7'), sym('-2.85141358007718167766113474056826947742091971026789183855345e-7'), sym('1.53183345918659430870675805494060324865810474145754519759441e-7'), sym('-3.57629537317235981343920512647387725908195232159756169347749e-10'), sym('-1.59504932507568549366563643064796545801180378169861343250348e-8'), sym('3.64709894382365220205709026082079947530153341927287600304838e-9'), sym('3.35760874495384306004761863851286605197576832570372416446466e-10'), sym('-2.84586341954667397175815833156556349420808438171574840998685e-10'), sym('4.94395001127394626371383499291117644971886978830111512160973e-11'), sym('-3.07081095765789713389193472831676528952153074133338499934866e-12')];
        case 25; a=[sym('0.0000953201008204271780208467250814521549387987205561372688442358'), sym('0.00159591143226453719047101076928270924904584633749425417532564'), sym('0.0121528612872303815193737311286386038501853639628438756489904'), sym('0.0551796878126285478799502536538240979005701784595809932433506'), sym('0.163832050583104854344819181254795191010714442553837914677008'), sym('0.325045260048794396039765059955245511829152905267439425770371'), sym('0.411279393877437216815247743169144312247696602180827338438997'), sym('0.260134031090483899214967273702514442703495404556015602659661'), sym('-0.0687128475854637006642557590401717308108716546579724218657336'), sym('-0.237922396301399959914346707432904492655562092103226794890092'), sym('-0.0619337962228843826333041368630217462317571933815941803154092'), sym('0.158772214974604087171252516876840276072014660888893399611525'), sym('0.0835484044727244416089487576683520476729426348737296404600609'), sym('-0.106462148119930934874934522142846922790372826455627295366837'), sym('-0.0696561098768276921837557010100820924662471090128003981383027'), sym('0.0754014866322898631262973191579866056614089564190908919482157'), sym('0.0472009081726004417505504312016107438222537214664744642946917'), sym('-0.0545066976498405369856009975657533203512547231106470754157550'), sym('-0.0262859612226765980418504503694857138548409216541865361019946'), sym('0.0379135873289229055572858630967740318300026193678250625279761'), sym('0.0109902820497784703199632705668022133406779585080876090945753'), sym('-0.0240715556450535317133965320637565547852249922030508330827476'), sym('-0.00217777348258418141083625472344537238807264407026075714865981'), sym('0.0133804433644905543911075690353017217568973780105609912164236'), sym('-0.00140673646126290654651738015143196191303344775247450179691867'), sym('-0.00626546290729798220521488187126002090499268113538729441802523'), sym('0.00192823512041746405166929474018889552258194706732282234509227'), sym('0.00234950919887770212947717757984605205291839924345816385029332'), sym('-0.00130283313583245830055468825003893020756117854387069385804967'), sym('-0.000636380139245804213267896425185530862763030767303874061561241'), sym('0.000620315217598932167478660384844087727694104916566184668062433'), sym('0.0000815444336802408780623634024228157712957886627065857218781353'), sym('-0.000219118319427287346911319348086778334497330632410094425820619'), sym('0.0000250578456999775482909310333543576190276870158871588108633721'), sym('0.0000558942454964248294347447158073708536445033296124912045188271'), sym('-0.0000193255685893289032873638855370136655606807164937518421053215'), sym('-9.03113452721107935767667656405621646285539318088412081916202e-6'), sym('6.35735763838664859800451762441757124801202133430205186562718e-6'), sym('3.70016795721590037572557293851860412178681082985171189530984e-7'), sym('-1.25808532741552732750356968416348612357987525630581757577859e-6'), sym('2.27125351101330030616867445985617519337045421845815162747451e-7'), sym('1.35962972022120976128776906921205744094656510573788707551081e-7'), sym('-6.12138220323095004482766469677104124055970291301614477102680e-8'), sym('-1.84667904876362941740783607361679415004047267695245654437534e-9'), sym('6.56140255401775419132297524898570926472082436331628897724870e-9'), sym('-1.32965473185447216078043944700915930986853205916044091486754e-9'), sym('-1.57576972072642096361125541993678865947395748430789361641269e-10'), sym('1.08604641549708081262987157282708241076263955617975863313630e-10'), sym('-1.78730089338431312328791125395352513840123260395369350067696e-11'), sym('1.06751350926843032429242162227789429293660001316344977199804e-12')];
        case 26; a=[sym('0.00006713127354527489349485778881728614397430697953485943263433392308036395'), sym('0.001167094049616780040201913871370079256736547609304959876454893844580516'), sym('0.009261369457227090098406998453960136686734616333651603162572435741074935'), sym('0.04403489379683971945379118196043535185724328593657988674789981991380640'), sym('0.1379137097154462001743148454184292121841078925419341183475529480476772'), sym('0.2922422562437017767451245337485104930377511564455521827817517164765658'), sym('0.4056452704862959151365406843746377305573132981978008776663513258660326'), sym('0.3105318202805555810043183844044501220466090779114647575881612882513017'), sym('0.001254461722181286980450026128184848127283607151023951924402526784316650'), sym('-0.2307887594742737152167111874413975104193855623278410393385130561113787'), sym('-0.1236305222501229050822775941014536070594207784833140829515625999929306'), sym('0.1281926840009293026262180599176616804286239058350667248542491899434669'), sym('0.1292275894193823329158572574058058990761778217525487617561827265708098'), sym('-0.07376813733200821695762074170594990573358954526866273157029554698166750'), sym('-0.1046356768258848309831724154526566041987437599256655717881974032055650'), sym('0.04937244838475940813951496233392168916527899603148638825411818310176225'), sym('0.07529443082919371054578104543916577856282375699033333355700548455763345'), sym('-0.03779384040964422370805713266496578649489044185520943980084678801174185'), sym('-0.04854624607674604608649969710604719676879535190173657711325845899632664'), sym('0.02986266496094486114899349703574280920203513305293322239651885332121903'), sym('0.02724886608105194174203562820465007568208499697661498717859686936960412'), sym('-0.02218767461854509022014623918548225591441618921351937307743642159859781'), sym('-0.01255885535318642121044271866398073637723247399240901151711231610142130'), sym('0.01466180266661420884991961532888677263806353989045253233768935436926627'), sym('0.004122135942139225123227665966259941639430504002752788639059844494922040'), sym('-0.008333605489128948114715971558703799550894260566708568037839607796556194'), sym('-0.0003738745075923622886767248573368683775516474412957481772606955978114115'), sym('0.003961174880845832333861690762191684541950945096126445974189589200944110'), sym('-0.0006640144568392199158086664438136988872326426149841995789293462441838379'), sym('-0.001517119011337547467765303032831876192061171172919694495245929643117138'), sym('0.0005928021254778421271449326508500247633031349503871825994564531485742938'), sym('0.0004356755138336638595146687419006917423748522848176734244459261214348780'), sym('-0.0003054388098932847307740819133898924069870283542431324081143425988393417'), sym('-0.00007499395964666934368036353670751891972223850770860893369000996876726009'), sym('0.0001113548392199645868490827537273277586280228264206127273017286869201613'), sym('-3.731964982842875126446857478492600476177248333425788564256577616288208e-6'), sym('-0.00002905978351314433004656897157507210830572454620545448034216767071051878'), sym('7.595893360473911300758270337924542328932067478220133328289069653537779e-6'), sym('4.949803105563938654712161782375157212796698108208206199294390470912122e-6'), sym('-2.748807015634622304621320455162768721800765802446364824987373397778277e-6'), sym('-3.288374078973361274948918577010004690455030886456119089861832271095381e-7'), sym('5.613869676364654398289434618015537485652437706588338960607170658294802e-7'), sym('-7.629712133208991456798920579642700173663682791711250829365215564104776e-8'), sym('-6.296408553193772803476892696631001998657487156258546514042280552480222e-8'), sym('2.409675392712498998389278529537700948472384864207600306999916289765320e-8'), sym('1.533946723159776330478815648170869752779671464129149112291991014978244e-9'), sym('-2.670042615201666867032059462285576938632564334684287307661891396431796e-9'), sym('4.794217384290603981174379966740804740603869259748058861280897919819854e-10'), sym('7.087353831939148460302098461941515212746759630340168974801777960720770e-11'), sym('-4.129792232752160832301500700530243243826494647501850436371238965937554e-11'), sym('6.456245548268192181299719561817735779907151846761186576765677295602749e-12'), sym('-3.713633756581735192104629482153583052502068389179139017896189191677898e-13')];
        case 27; a=[sym('0.0000472851594932431134898575046450004518497229137449745359544347999713196676'), sym('0.000852439308848299920855069233557375437509940476831438412485693838174583236'), sym('0.00703754301731901176350299410365750955562423844974837454513839559529781226'), sym('0.0349682687952177848584141971563312705717594948866957203482394599284452982'), sym('0.115203270451603511654326422141899228243972915486051385719230172149266721'), sym('0.259586121850908629486140466405008693023271794740627064525488256575947895'), sym('0.391630992465595139381318584730234598057168812275693889732905017109384027'), sym('0.348890815224574661654864728075792284938066677925472349375882960788167045'), sym('0.0727194659972378616999868254104812417695226241192956042690047008586963003'), sym('-0.204860716247439244534443136024169352470207342245009793738347271838122924'), sym('-0.175549569392238575931226392931898080269841064159808385889600800121464624'), sym('0.0811921357373413265198257404728086805990418555561938179445104617396456242'), sym('0.160706483420226197246075235131126707716447815526593164831536051765533006'), sym('-0.0274261396324876664701427557541455635495833941798337197845015006524896047'), sym('-0.125887451298561791985756323405650990797133534153812862075585275952900003'), sym('0.0111718610827975628307876701407693746544447261821844634726656476274527951'), sym('0.0927709754791214039760266627918675049037160350750421331616677454458647649'), sym('-0.00994386698725404942155346879262919626797765132812677962090663042872187296'), sym('-0.0643629144503653354687143665868520420688515679542855411487532860038463721'), sym('0.0122407384047747888860640124563273012812927989473319243891062093996525633'), sym('0.0409905598963734256725664313999731343147382460751082930844295587123253042'), sym('-0.0130903097343611440747077820801229696444275179013986349074979338234524967'), sym('-0.0231500160245130749894433430546858807215372725413583033325496096774843081'), sym('0.0114176298064208534813183485540695278253607865606803686323095088022489536'), sym('0.0110772489146810430641459671014926137013147482814765584872718595244708774'), sym('-0.00818630705220320236993564995279982273173633017830812696879779928747651317'), sym('-0.00414512807784571310063064485657805621564044811015716268924438466503047567'), sym('0.00484837353573333679736691617144770482834506397876000487163141390562468293'), sym('0.000949380569544750567988496670432846929653540177252586778488271355074471431'), sym('-0.00235668399610548992332896576962394521994863404419164981294859164751803919'), sym('0.000103062908227661727046862676052156979498451124238971311703316149901481343'), sym('0.000920071398594649494512540404318642712413539655510436033889776289026867783'), sym('-0.000241713933302925353683570229781519904883342784620697245600972263368428942'), sym('-0.000274288033809562104225540319763033516376459169182595722369682943046050800'), sym('0.000142815762302630804430420574869696428553669003501823630868594745017647955'), sym('0.0000541647922978107835038974840611047545484773048847999830309928904323897603'), sym('-0.0000545260328635092308582625128404945400576740964428388606640463236816478842'), sym('-2.48723651681362518437707930592774833115160045507534429349996171648901770e-6'), sym('0.0000145907428880427603020008782814147915763417353481511858833732426161664757'), sym('-2.75853956886974618979202711427535995079923448587401015600699239852314555e-6'), sym('-2.58624369437505810008618634032961902181085330494591914394669389840609457e-6'), sym('1.15567384460881138542740903849371234699598050807589795579944030864107074e-6'), sym('2.15729842183985627360751102277437006916980004794848608135732358681382040e-7'), sym('-2.45540577447686047017860561651394434823288008797510853867589295822761365e-7'), sym('2.32394813308126512568312780226580635406236639479974361603318707769356310e-8'), sym('2.84699225066879250057522782553588937350197556206521317657682117594215181e-8'), sym('-9.34323011139010183959166449462564416529752845611628205428094937445452608e-9'), sym('-9.25932010339110651731626918363234443275512412353365847725768420406107326e-10'), sym('1.07594427409195801455964013387931054111984166516782025303161185329634718e-9'), sym('-1.70803547093537990206513437811870725944669939022849524757635133139179335e-10'), sym('-3.09358242679577280745419141387383700331448887297461044990743815687876275e-11'), sym('1.56529547372821651248015790089869039588514376091366738240960638238117033e-11'), sym('-2.33047554497083582620028491666890898214223728077556006756318984858120740e-12'), sym('1.29272438160942757765805371796481214895876648423536830206727293765252956e-13')];
        case 28; a=[sym('0.000033310441225787474322017916685593658923109762648305107169307153742365348304'), sym('0.00062189936469606522804329181244017381212489304444187469528861797346001226080'), sym('0.0053334592301533677028931701687941018584588625311915597412075758325431522444'), sym('0.027642648292671378756837782605708035108891980457562358524858914175869746191'), sym('0.095556935564155780270636674524129483292162373996391646341377153841516900837'), sym('0.22808674012731918087278841291594581144297085328405424995686550418855159507'), sym('0.37122980969675607754451480620425660622379732032654777127069476539300988715'), sym('0.37513166862242694241250284145627290918339110877233871922346193300520588128'), sym('0.14154590888669073607334080137202142167026570425538411275339471004499827924'), sym('-0.16298737346345200985389367412272380378550780969664354629704718875002461922'), sym('-0.21307093748061940157902639559559899745620850556040467518084748870209044694'), sym('0.023234029171839184665362641162004699003492066107799586365499249350003482045'), sym('0.17381261070731474791294674073663973316513058730488075274840219887582158784'), sym('0.026097108879213654395737672715062848751915203017687571299276320610402483320'), sym('-0.12931380068658864828093498079698610771194060893631326233902902122964430526'), sym('-0.033119632699573068573646095837177301062518745642250023189435230285523289255'), sym('0.095196066203973305400243446763486392877707040634820304203654379698097404780'), sym('0.024380073980653581340202574403832125996636236328179669061904663313196447210'), sym('-0.069073977512797520986135883708533083894978133292164339395622377657914918760'), sym('-0.012262591232830213102925870024730638342615573095701162920291150389658247081'), sym('0.047904996320517694640524985629274462419399478998774256744911510817833465703'), sym('0.0024381175851227634728303072418146234371837563085747709791047843672633566841'), sym('-0.030641318800090929693378603176639310588243150093297525775500128283447043659'), sym('0.0031337083931131309817184552636525452173636255947496744586785460692322961259'), sym('0.017457094647518809668429805584835337980628548191597257418599130672492722818'), sym('-0.0048193214560293155111400098955738632245115338493797185614678391665130025588'), sym('-0.0085302476861948640269462364829739384480467199604098946234679769999411976520'), sym('0.0041286668315860484666182689762343154909355118728229038862242759620650874780'), sym('0.0033834091538657642392431596598429177793360612942632122023033279741330674072'), sym('-0.0026342989108538975519088944434100592624876647317595979814532630046055641028'), sym('-0.00096192957120064817782220985155752483243050723319443694380608306807072604647'), sym('0.0013265313797831287777422749691493624537899934640612386491644009999245401279'), sym('0.00010010315491282103423215233457501010746543397051913748342408843065346690720'), sym('-0.00052939313822953498192307710543469452879483090389886974616726312961657787577'), sym('0.000081646513255138911862249567411227976491364438413631520944654011028182866898'), sym('0.00016233693717242015300373804581749037910147064725621731322099043169727851925'), sym('-0.000062960091226274623073193175830816098564052142671149090025291923547337947025'), sym('-0.000034702774367083233562716271242636175685602366444353305392345495007215660205'), sym('0.000025748594893549291633685253525021792636682386927128704077074133748810651813'), sym('3.2800314639964760533614736200487186857171312839637441482263230938865627491e-6'), sym('-7.1016575434910394036065787598826686293447637952508224288864445791455425491e-6'), sym('8.8239877680201141456659627757120975061788532875656880084191257284290286103e-7'), sym('1.3013336765273136591042960251136432234226853874038147966043845902651073963e-6'), sym('-4.7165545978515863086357416286548150334127748527114784544810473985686678916e-7'), sym('-1.2427127132487453967066797197100432445549157154732137565970379932751120920e-7'), sym('1.0540558040144853096457948003819729991392495175627271316399629436912627874e-7'), sym('-5.8423900996692535936887204201484701528582322421179586297339345190082411832e-9'), sym('-1.2615765668955040797429214070979717188194281218688857899667692423603138631e-8'), sym('3.5666798781927717374109862158306060614790068230460619106962739271052637305e-9'), sym('4.9105315588213344692815400470994148256707698203866881805591151820858569340e-10'), sym('-4.2971170754659875901986949954406425599448431992139650069316723430412816637e-10'), sym('6.0049063571460653798285551690127047733624364474592414771813276856275341985e-11'), sym('1.3204280551870038206257599329749173939175576067995170022166957428256482712e-11'), sym('-5.9152950401785451889602853451266150006142298107633878021896596230939007601e-12'), sym('8.4064427398855711469531278991866013060976436244448245748575919581720977791e-13'), sym('-4.5026950130711051121936615132573383336665863750649710987644115622998642195e-14')];
        case 29; a=[sym('0.00002346863563005430707532084966676813774147343712329899073027549966944336186434'), sym('0.0004532212795561840600347689021415766412418651013133810782662253572967979706500'), sym('0.004032012327901188376889246351844908619754185610663141367203197830436192860012'), sym('0.02176020725594607978952121634874142316163693852642771902122752622511354179363'), sym('0.07875056491811459846185105000033328366154450660868556875411953827944993941445'), sym('0.1984519618805221819066983550936957022150834153135116515480368677493547195478'), sym('0.3463117719931695202388956533042254095693746696045864716667705980732504324294'), sym('0.3898806003763331704622239252430562603204267728970942992030309674733390703308'), sym('0.2044282745038178232433908605332850832501598595017618708710011546586855939362'), sym('-0.1089147626341773800401488617600653964180468905630371011678486301596212686245'), sym('-0.2333741930488694437261277709392751444917372487036790055867065728500334604309'), sym('-0.03939065608977974081894606366424254539644651857401648913171757397567501775952'), sym('0.1669516135574558442754994654406993798307967411802656695882480232775576657837'), sym('0.07949236088822783486502815123249682695999112189490597986084600041819539758395'), sym('-0.1137579166786021958700322664713127776012777447497147210657498832980978589033'), sym('-0.07625860252517484560589265777195665082281666816525391154459036563162222319210'), sym('0.08094413668451186960608080668928921480784587819797840126309131529972083253265'), sym('0.05884595465397935674694448909715981065562901119249034785667811698954335870716'), sym('-0.06019281308036124023154772221572825497872182938392506878806174504956392830504'), sym('-0.03891031099502953205992942238574132737938276947065972573149995630134958817878'), sym('0.04488654774155313723091264954519531213848414783027780899986102695101618996158'), sym('0.02158906128821961091578213669452604873352373115735222882356398854654867619984'), sym('-0.03195272798965488958530413844435637648757871765623639499784504898236578312775'), sym('-0.009133799093675371931601591086791637447483043541249395943163121021215824532816'), sym('0.02083874222100891545853248620788895326331706247924553647032341546881183036047'), sym('0.001872650198060744801787180093926191434547196340466951372344172848882704817803'), sym('-0.01204996545572012216538297144915198478135219668053250332510558404587165620420'), sym('0.001228866968157406895928605749735971933250632952859285849596542742441341641108'), sym('0.005989000331285386101324002574551083863641231324113371347493181294987961267303'), sym('-0.001803693017558916412608736622921554890220124857369098027322084488734501334593'), sym('-0.002456346822082483885267006737933871448497871863828898679990325136292076821768'), sym('0.001327324935686362602937431278681520680617622685464356357040321519942516828721'), sym('0.0007686632140636185805265355792562186778321096037715893692161345762209086200868'), sym('-0.0007076571415468133979720842316544539258929253040296473490862896187321661078190'), sym('-0.0001414716572028571777912806737881007079061145876509482625655865452751861204286'), sym('0.0002907116410228666930630301731966574889819237006657974415270948977910492409944'), sym('-0.00001620701499544680644433464675516801856220678185805170574625910356098365279819'), sym('-0.00009143207747993524439608681208237585295360345167530880299267738323166700497335'), sym('0.00002577422650682479693280233545168421527082876364622247678536355521238395666011'), sym('0.00002060045828778757192737104824510107233494485932162854509200430649493371437788'), sym('-0.00001171908146974327461991142382650108640172046538140079322294090844838130097995'), sym('-2.541090610102011815441345709029036486482176684075281992843321546027676973477e-6'), sym('3.359188012934114709456743467382638668052980371126327731972763210329380934829e-6'), sym('-2.141865042624799155949190192942509316265222531414445929694509578834479577585e-7'), sym('-6.346779573782880113334372849744257364601840857921737397803191891561729896815e-7'), sym('1.862447410402380688261503670657865026808942641532580387458473472237964738953e-7'), sym('6.637750945722687825176257859393648494811807710463063110643974944856560064806e-8'), sym('-4.444984187156582065459349329523862793673798804473815859560141437356891530229e-8'), sym('7.612654377409879284838799719530347848440332123884662148943521307657281174478e-10'), sym('5.493497631102808977395240623288130224083813424780565600245354033756130637073e-9'), sym('-1.339256981098243773632938405249207288734759019544428798509501927375522930819e-9'), sym('-2.423114128189245237843493682851800912352526180550017800514930145755563113577e-10'), sym('1.702076346566889099606328378085740359696886620941480540220361603251581551524e-10'), sym('-2.079310599899870068665741796998999951518460236558955508967395516252037686615e-11'), sym('-5.538420746357868454460403742289059768695351272959003157207800330342342055660e-12'), sym('2.229339681964211587980487161195942972810699790477216812696020312431916434192e-12'), sym('-3.030415620450478354341233532560564939419004096990449789592447757885133707916e-13'), sym('1.569205225174357549307938136465554189125479722505011492612492519325974759388e-14')];
        case 30; a=[sym('0.0000165365135433091847179537555604004047567172297332971957255430168732854587714116496319895973'), sym('0.000329962859107020375643181325511764696699254985879495837803853271394643712336020649279013816'), sym('0.00304112283991336920560206596866299878675028833307866662879154424390361989863787126958048338'), sym('0.0170630754177580358943367778723391393306771508266026501646140689641612850122222886875250853'), sym('0.0645152235097469644612238955107063311708307923986812528265685275183582034514688769224042684'), sym('0.171134457609143392328340634061656698169746471814640795035879219430683127343625821819356702'), sym('0.318542993674438417616967073245468427754527888299229686817317000168555525573596872794990892'), sym('0.394263106893991741609736440346554085942632119199482150755004714093335803377955389663077319'), sym('0.258972684972035753095501056832345272684168480171298821078975545360517009433511128867149370'), sym('-0.0467989224094832070199485395961634722092287780864163511074456011822067366018024286301802536'), sym('-0.235443205948418780972158134379669578181364019593333865943978943904785457001826212181667552'), sym('-0.100386898490674406522537850551401247607722646058713602309815364454120689629453010227951766'), sym('0.141041018759543332300307336324275412912309685656116691350514451987326618815385401659988958'), sym('0.125744709268968251972374467798361279782845762248558593466630802310444486689210853022358842'), sym('-0.0810048938015241977173859084193619297904566004665033744378232953532989706755676671770806637'), sym('-0.111183220231706350804704241573509695502695079557782214382162732546840389259044660665781127'), sym('0.0514622832836078430322615200788187047875450735596382805512938676356854367717894379022118922'), sym('0.0867957636037903504503529015063105011336615999810579374887268352316865996402553088371129091'), sym('-0.0380469165972133143463865808332109063459554167712733407627846452567462682472329609088174590'), sym('-0.0619840541546565346617059411407879064571349473964683819471856649994453234809092428790792528'), sym('0.0309724541164187076456425430313595428864048399204334184894246381815093554505992594192993198'), sym('0.0401016983952342763674513786713559839076618135730083236056481714319676201850849709034229299'), sym('-0.0252249012779219295059370494699566126895042203788262538612492770005879367632331517174778153'), sym('-0.0228139227184419028712115391229821296783513105841474184485065378406610601967528454170098220'), sym('0.0191474755410034417423219866426933560063036027467366981222915717495820502090690243333657861'), sym('0.0108102207308800759678927737285938092382652257041898604307371425105973199808131617713967370'), sym('-0.0130105836612413683254243863162213527403310116351778871555559468626162663231950881391592152'), sym('-0.00374544538891480344819993324281072764502974879302512507603741147762160155455609000998801036'), sym('0.00771851716652194623519572576297064890643515971310418040412956879062136169205848794298548294'), sym('0.000438174101129319993549017999455303766212929391425010696380751598885463687891221609240295866'), sym('-0.00391081679269944443280959065014080242961831703520348331385827717499999344571942702527450294'), sym('0.000596362960376979704291105897836480844471791082049669984116800954261171572977532788621094827'), sym('0.00164368392151428750863148075430278326811303008652907760030658801090942907590448150361835099'), sym('-0.000608767812526404078737463341738454845989307257912914511819025216445746562798248927617640960'), sym('-0.000542971918010436768494475666241406861773118351962227608400195716692839154957331647073378956'), sym('0.000357155975124281585454662676524859967565512503007640301052973196584080096976380873027832769'), sym('0.000121963604949269292479036812081738203982939989906381945929195263368804081913331253639173849'), sym('-0.000152856566977211149182207242495310595741430230625463713741354657744896474930748190394523811'), sym('-6.04456476378273751079989732782165338975428773115745088008249376411704082458241034728595685e-6'), sym('0.0000493702546529980745169152574627502635659688354431213897462861678228906181641987232271674217'), sym('-9.47322878905138199742597154619271742031832195771694596148098810358393902560149173967307286e-6'), sym('-0.0000115693451276192750169317030148226256447884914171633318746479255968201611059364018544987612'), sym('5.12804128657989931060341436895135778699823674608413933686708529857727871692541241628054092e-6'), sym('1.64582575108952113884164350527564397622151741503043229648835192718699173864460070337480828e-6'), sym('-1.54663180667413635459834766642615108118472100365283828622935621730949704758534921179314461e-6'), sym('7.77445760512472118639773441709823126687765452181031987906403245335369472667892554399547248e-9'), sym('3.01345032985001811812139277266261517477070663301410456498411299765169729698532238817284170e-7'), sym('-7.07400005891453513825511307892555662708506424486751719022146304025895226327495091852263541e-8'), sym('-3.36892538149943436768525027074494775111622769189637768940037100787559015425299066346877259e-8'), sym('1.84232624003803934211264610883812473467625651001993578638121317749673241310445490534142236e-8'), sym('3.92684528642072762783602506728800888657277668059230775799262450943343242444703441491045422e-10'), sym('-2.35544741550765334653418979838541542656186624020002434785187594370672141576591881494272563e-9'), sym('4.93904377505145846277395257018105502017451650135024599260285697887566073219497559436001689e-10'), sym('1.14100375021378950030285104844795817669115669873029312934702394965387282314778271891994987e-10'), sym('-6.69021161231143482196048013579785178744629032008741327284378446382125281610272937516089186e-11'), sym('7.07181120307574463886770773902714828740458630123883339484801650123833362007163475808024651e-12'), sym('2.29062195747608479188461623584125598435553237535434100987258304783715611302471713934965034e-12'), sym('-8.38089538692244043771702929351619019758300940317624080968008954363021928356563489381707564e-13'), sym('1.09177115248190897676322994169370528732093352758277346730891242844997498036483525576210539e-13'), sym('-5.47155170678033482019301341204837072453312848178227130869952493598581303234839855676299692e-15')];
        case 31; a=[sym('0.000011653214410493434995104500169762648002321442319764051089278943072658086009088379'), sym('0.00024000067090818395637107395245750364429925790311483597709176273768158213183709958'), sym('0.0022888226748413637503083939039147331380553082028219482550532150123037478466311548'), sym('0.013331573189942276017920757962770266843882519900031668369409057283937791831196406'), sym('0.052563555455450146992188384053424423425771209721097431653471561505280964421041073'), sym('0.14638020734142952450215124008642666091508397892050601783897469974854686996556070'), sym('0.28934257945510748297477541796836308570577496998666078447877550684169982437024098'), sym('0.38971471889255921751307082558713843884286304679927767255001794142423140828122695'), sym('0.30368030659066447047372153567166353473957361163547396147021100398633781379938319'), sym('0.019211534396388242145762815556977664002513415068119144889829615153581781793727896'), sym('-0.21987847280838899857033710726774549988467920645668732588079582852129349020365035'), sym('-0.15413406526648532844910762648467093928624763323202452548455330119964788456169247'), sym('0.099121018561050757981853517472530632476058385029997476301090136023104525295740710'), sym('0.15907548722431698523099075875558461793724777326265177911214923282035041921347951'), sym('-0.035303260051254386314762501735965673671630883177829391867093186887257040944639226'), sym('-0.13220235321600699372492193654019160762719042828193711233386799685531488639359025'), sym('0.010915599199589897639844077606332573786076235379553480101803730060440159146268819'), sym('0.10259376998798429747449845295887728420483282143561794445173404395940399465876602'), sym('-0.0057557305982912093407493973165297943580294456715948198481298093526703660667924721'), sym('-0.076093721759334564940029544486035190148020691031830759286359298275624875239730573'), sym('0.0077366656235463662021316409809611412917001475126137883298735912053103028987808732'), sym('0.053283049850572543774672641137697209215465398858758254057431207985927856388761779'), sym('-0.010521767725898249152712848343962345494428920379569324890356415309510210447084065'), sym('-0.034378877956218992111852770005713220701089852746981762979486590679809605400490203'), sym('0.011422724258759547429247755749700155888172613860089842172037905975094062670365929'), sym('0.019832661850372427080208682966029405306753608659543012996992327015197814758426322'), sym('-0.010094851058992446916300633897557156591053907647122963863508639203769741679790308'), sym('-0.0098291752455979574247798567021289161019976570598894499398437979090023882628013010'), sym('0.0074370942035960321715914335524327426071839750749853894482314683179621325054470772'), sym('0.0039005166688224201244473630570708654384861393509468985562669314982815091030076068'), sym('-0.0046109389340973482099069369298660842263671158360237697338013657446763339106119518'), sym('-0.0010099353175642279866690610895486684397999288529867378270072240483678821825479809'), sym('0.0023992605268346165846428448572938818238909629235483501645564363765955636147007269'), sym('-0.000045239992574238365062106882013260767314188158705085141098561551186098690389195309'), sym('-0.0010316983097919403292174562477496274225460735076156025069278861557432678975351292'), sym('0.00024263650046933078897185980227903731441956255316901581589801126049776909300983762'), sym('0.00035346968156980838895001786609746415409112247045869701705981334670838919157758795'), sym('-0.00016946404228944084076069290061562596540844057331554812327481769790809142331698917'), sym('-0.000087922478636376862494911868871159909585076090953102077005179892471817865019684026'), sym('0.000077045248285443530207688253640690139018831445009857513282803861625121646011509145'), sym('0.000010616046737136675468569366678833967158936737956793786133950688373158992668515851'), sym('-0.000025676851463414940361600511947407068432997024144058259682968744473437129990661771'), sym('2.8528366171331481142027154690221693422598874100302311143867267470726086527053588e-6'), sym('6.2192172219973581419318977783604599113453282968739798836026361422560274068562866e-6'), sym('-2.1461697487884678643097798304258288589371632106646510974094301522964665563856936e-6'), sym('-9.6807177315257424790739825112668648566347561060386827045862052684772659411375955e-7'), sym('6.9367284284721861141408600627793554865625437143337770404734334575102713591341598e-7'), sym('3.7669350646274530672387010310695569295763027098900521687045721303444051546816499e-8'), sym('-1.3971900579531576824850541875650032446230857155639596832374677640412986477941590e-7'), sym('2.5574825567800775523295657169225334148471801480222957504982056658593084877666844e-8'), sym('1.6463635873456285956211882736784639006749477269129475518415458606915444781066776e-8'), sym('-7.5061478011070130614666903581884798629774397182185845345148289483370887796703718e-9'), sym('-4.5780296980717408684311041026134967444885978985877267623030349198479483396568563e-10'), sym('9.9600809135329996193758147160769605638040235037303497007493151094251534908214838e-10'), sym('-1.7847685959947433878174500579250026181409583267559306371781225828989458702832051e-10'), sym('-5.1964782604365122009715523235003305412126271057169199078522696658873814663987140e-11'), sym('2.6107151756313623201873216958141381297787813240707615605506929265424078932897037e-11'), sym('-2.3525506017232317761864773529526346049599161780955741137783962612136292094574670e-12'), sym('-9.3644620054533158478200046564332355959276471270532804671686834565143634628662152e-13'), sym('3.1434199293296960698789762958562549179156744427201655192352694541557480595122930e-13'), sym('-3.9311191735780661660279092183816104897344246634054134188074423627995497523158011e-14'), sym('1.9087519392990517434681454604578398860756475469072489528634508341289228324133770e-15')];
        case 32; a=[sym('8.212785770389889068164098556801848263759240191777669088580021840347262486899690404e-6'), sym('0.0001744126185752260879781862271171936396771132456007535740173194907591553290458799244'), sym('0.001719161790170171871664517559624031772792953450605736998399544604423265898766550958'), sym('0.01038106745121569606907301569200890372846866437772841976962425851296847816975559722'), sym('0.04260848624533156298665622435059904591045413475846495070637969337771263508187015758'), sym('0.1242745709102969779894840342885282836598340291484795443418484540697200276033553114'), sym('0.2598685505325354577928829146589205033005245501951326069787833786534493829593317901'), sym('0.3778198240754751909163397777776914096126911069927068478841001312626181959226596143'), sym('0.3378620997893483125474122324559855219847079326737524196734323245886602265924894326'), sym('0.08529867162580167287411586472581618283456927759051221195363281360096618182648523278'), sym('-0.1885840926522777627680921991703007535573832764313397120948960964384692949759204204'), sym('-0.1961666815674607409125795351290642224084030380758142322588657275098689142445389910'), sym('0.04575925201631158444925615640424644817428247940589402909085750041522760986318684867'), sym('0.1755821390513375283756623634038456891606457271349899598526538494838331475771664018'), sym('0.01743898198678921068188844182668148039629963857541871752878841623221339439077708930'), sym('-0.1358368706252491973762732880690215401267962315761827940220668521359201302122640448'), sym('-0.03464477960631093339466865068491366423215339678161163545431082515145905745624008221'), sym('0.1026945882427987942543622199946852589763663113634407634071463168189307497906523394'), sym('0.03139901170614351980141580857582263828415983810644300197516996793263081375054134660'), sym('-0.07739715982670860992642236209363050451907827792257991338692542502605674779232406359'), sym('-0.02095006956285385819690844543718402584440159633045298655699967901398508145428296112'), sym('0.05718665326810654659033658166452848307057876878968669978387746681604157663002925954'), sym('0.009974555393483880688021836470574425219496503961718722272696315791094623687932352287'), sym('-0.04025298270153362237332436488893501084595586441137363748769428774621249032919877692'), sym('-0.001683101144171192109470414845873138395243209636251212265438852199435502011771545906'), sym('0.02619933715058648060935960335079631796593066897356732119630877538721636147784550148'), sym('-0.002931599421144183280263231525396044506988199610936346823282611362960058250098182946'), sym('-0.01531792892725501244747133664732812125698004377835661626496426372827963811759794612'), sym('0.004361100384539071583150601490199500260344375860987951577018747791745262086687121635'), sym('0.007790478756913725674855899456797419386499697228720834737031507565254789098866369270'), sym('-0.003826556611573579212249156206987268311763388483833901637817374671014385146438582728'), sym('-0.003287492691968587048745103155069421595303987346586083887526078382979047935724618546'), sym('0.002564835140317327594880711162843989882035730009717311871400659498836206797213886247'), sym('0.001038708112799819776083474602766195754438373437576009286522430252336919558228753480'), sym('-0.001389281370293805917790090488707267947982035970237693197907438938507373641658827280'), sym('-0.0001563893027491428604236198313461537670841272317668248794156027193854107529510527570'), sym('0.0006132778492024138892506229060737683384540096172531738976570188257366884467014182926'), sym('-0.00007244572799091179714957547261347306318723303085989659788862083397215685957900766700'), sym('-0.0002163502391175428227339467784602351766450310332382757844366855112134356442589807849'), sym('0.00007452307697935838379947935861223080063532216195131980439804040031817913233420384067'), sym('0.00005730165899085715219627440138496551430797331602595482742814203803031255478463037975'), sym('-0.00003719246811534034922509177856385892195095677460554529275653173979118395496874301359'), sym('-9.150285457834692324966252450077546418778909538555246999430232403080522036568520091e-6'), sym('0.00001289952557744893347388331680054587041932434882646900948643638117066379638577570687'), sym('-4.498458861888571129934622642449445648617208346324549608223129717256628545958752339e-7'), sym('-3.223211612124151614804224322464059289263207314845571545989850688155785454932371118e-6'), sym('8.505709945979694710050476507052928046144400976071611943106591213050847194565340195e-7'), sym('5.345760942152152038122095611833948661961221435199840324770536423427138149921382640e-7'), sym('-3.030638941094202907799934139234311450753506529750540768790492416590483976830570758e-7'), sym('-3.537911106122070392716426823710626978223091581873967873366605903313973900556340346e-8'), sym('6.339895579075449940076814921132059314049354569020310504296390854459476778401471999e-8'), sym('-8.626167871253486401092148941889513587112878675568418782089241514768082816160020072e-9'), sym('-7.809167236873753134167263537350361417064479822489756472814328093253581243296710795e-9'), sym('3.005502439708080599162336738760675556919181023378554095045724232839920427966330110e-9'), sym('3.100230344689477513159953759650619338280569882046973054539361138459346778159308744e-10'), sym('-4.158559654007240818572512792822761664937651043039927304061924000193109691231304093e-10'), sym('6.296590580901513600666264724593556198663201850985531955477001446742265950656159544e-11'), sym('2.307480870044151215800431720992538627591302565539620924947012912370713460338411462e-11'), sym('-1.011812362178223926226911554434033394661000479937918712522712753343398623029114115e-11'), sym('7.605715870070949169656916856165837382666602595634557249280799187101477884926534700e-13'), sym('3.791140441769675172985803025362138011852754137626133088428814544209399276531741657e-13'), sym('-1.176484608619855526051275087956125932660025445008518421996317388289950334994767044e-13'), sym('1.414719358548125821365577633525754889381930054747310903624578093861817495322718882e-14'), sym('-6.661666519253506936891609539360630253930719535594390527548583989798457213725088230e-16')];
        case 33; a=[sym('5.7886294771760808093403379628298647760239814646443572842142310612513900088332904441531e-6'), sym('0.00012664396674978917433651853973109596990142354878745440803734988293241674678548730928198'), sym('0.0012888502017372257240235694918107105606417589510050198226451462413177430111041951693845'), sym('0.0080581488381845639691826580069571225773884405388593186067634451026415114123877288726926'), sym('0.034375760369699218802378441309782398348006994471901451769100632828830196719417776047982'), sym('0.10478354692864880447491358812649242304542618136007716362487914007497622408883237174029'), sym('0.23102460534281981520716391082608106792632335837768289987916174022650974040031168472546'), sym('0.36018334576016251798527578719319793396542563065798002083071070481042752730443417061448'), sym('0.36151171519340106181227459009227520025460260402152167019369844938817157843292448311853'), sym('0.14819710140621874256303267765319398340344506873301193076563977597194312720823045571262'), sym('-0.14439305903408510061789268057615978359496432113405037896062586077089559365978430979032'), sym('-0.22344391199042569989299114757778961996737251179919589063967562716873976741535471480230'), sym('-0.013631844545885344472394267276194042322156132660228332442980228460719071658363338968315'), sym('0.17353857907249564814311118432321756758450196538406248962418007112639630245609891703995'), sym('0.070605714254913583253970611824643262853358960529077818368669236214043689442033384325855'), sym('-0.12121797132548888892889066347797780549133558759495302230997664488672191219191726844291'), sym('-0.078378638181570563416252633690573859695004343202437732387375208335830546629956336703044'), sym('0.086244298349867821616323971600434308987028742332876691758660415228107482301867768253027'), sym('0.067025299836298098928328944503077652630022887328970455024547143794852349100962666117442'), sym('-0.064450639405822127645892486850670234822324591710420398381606709480929254881606308076211'), sym('-0.049711363915989016281473319818279558223066535928832089302500940245502534284260521828539'), sym('0.049632633859919696200640926377906412650490677737125685551168798602133435041408673104600'), sym('0.032339218849770212425969434297341737551877418914008831296237736535253183325929372820714'), sym('-0.037809884418090178463553401450684720180190666981658027303060129467232937268636565200651'), sym('-0.017853444238724863567005335162073365117403283503707135878499703541818052758882338300739'), sym('0.027355883007745205254720054540204329956579474815382898316474070150700191586399341764546'), sym('0.0075683518421780166938036985009000875914812432898001266485165897348459358393879028089591'), sym('-0.018192981908304781119264150824783766535537710786652116057934308668301609962275419841537'), sym('-0.0015328368183063100177314525351120869921171354586766067055461208676117254569789474964067'), sym('0.010830722122446154765868979040322492617969898876589984404774356438212692637170528185006'), sym('-0.0011273324092150112108782453573758105643718536130033797727831435508462865960886760714245'), sym('-0.0056240023421297467462425564402885815413301973613062654292581687039810972666440616848988'), sym('0.0016893222294919774049527387186684788266482959709694494620361340060771882074040084795475'), sym('0.0024612979581137787997068464626639246538795147123459744557062761715573306865757613731267'), sym('-0.0013157264673190178906942342464081300324755694472921084391470508165160166234330615659015'), sym('-0.00085157524269799101967877189769135490518001211838643073044022259839851558927719649305970'), sym('0.00075970187596592313444782158651633933584335365313953095457876731362751761129437836554879'), sym('0.00019284964590216712483038032307089876171080354054568164361590424818345038744265572027997'), sym('-0.00034707127255617746230944996884734537309833386170192236956888330552229307214630688788672'), sym('3.1064376475037574624561462039341075609641724824514287601644472964737161228086050079746e-6'), sym('0.00012589554686942927929580114949788581847625829060227968549365804517065548322928700790618'), sym('-0.000029418742875668382080939102598509290426416312002697575360706573083042122204068737957084'), sym('-0.000034857284320943273593274765217520969607591626929433026624986906384622302098659777482063'), sym('0.000017135568935928297743065353399122232511134094050037365742472451698712892094618901483202'), sym('6.4140282621864798753130625375330131307543568354695168216757011250718305038616559788156e-6'), sym('-6.2692945412573362214394234836214924872705275339316151843319653422406107196708940337498e-6'), sym('-2.5508990995859533464028158075027848111891189165533825833176577995845063525986410055200e-7'), sym('1.6181228472321874698767561767753430399496579464362792370429087183976160021330428106059e-6'), sym('-3.1303075615569354125177185095238541576502926563822411185624361040336953781072563278356e-7'), sym('-2.8183800509575515117913378326764594182880469532324694833308560956362994308990776423403e-7'), sym('1.2886620387892023419983210870373750015609546894645547246845707027039041422333321877622e-7'), sym('2.3885874054711435746398614293102108690550562696243908430461497975587904011943478845817e-8'), sym('-2.8198274324474460592806720298160325307042157544958421592597883344877738420617307406220e-8'), sym('2.5971067415553561733003472377750116422233825455679734721676694442207311537335378681008e-9'), sym('3.6141725644114732017062522808848047308442554143137809512626034003806579962389601141946e-9'), sym('-1.1818530961103800789342808214649056363099775471719817117260204817167319521917076799031e-9'), sym('-1.7652228571879568356346466068130872300444259544710923163928398683694020645926426061265e-10'), sym('1.7160301434483344035250297346190557659840482395538192840817277669970575726878669394879e-10'), sym('-2.1563747761183828033680519198456420810530032830880712665072337974157381645666926649731e-11'), sym('-1.0042591145192628898689438192682467678694769856421382265966356982596446718764501674382e-11'), sym('3.8957445094222915965013584349471472718684264535959202946081386380045090429645878933862e-12'), sym('-2.3641982426917272818860126376651677950540188766879016517206735340820754644294841092072e-13'), sym('-1.5220391347551207780462945939532360491480984116259352257493434436493905795470962715167e-13'), sym('4.3944849720899775464435949283694046815972943323008241077313756192180099809079833525032e-14'), sym('-5.0887014075069046142145847941875085110382760440190393162013082220467889176121501609599e-15'), sym('2.3259384338647080907664831194118369156443058023986115606064492844725342249089104322558e-16')];
        case 34; a=[sym('4.080367199312659984312681183742208280876068526359131587662141945423796335253510231102948e-6'), sym('0.00009188684334910265658998248165964867187449940289554230872404544952828113486264790761251965'), sym('0.0009645370588655025195915583201930041151882210779183532470675059671643579542473016172644726'), sym('0.006236603606802444789555778923353418260231948638423728610596944561021666120664244879700017'), sym('0.02761170051732551030156298779528647020788440486300806312555448097848031108796890843042146'), sym('0.08778906200209607650723298391052529608662880136419031511636908432849589480113212695160366'), sym('0.2034806247727080906777883276669704736788277618507444815007013487073068132256198038763775'), sym('0.3383355661477808240560542726485312129118542418093431517732706398186386306747789330467396'), sym('0.3751591087601896235542146689103904990631129510662914509506078608822754449734256928118455'), sym('0.2053200006228416266547249125254464377429331514031197398565769768750761985428978131031183'), sym('-0.09068421176730543676993268567228155103567695886976187563532431122023489247877497605006616'), sym('-0.2344237888314951591711516819644954262209756715763039251220813021862254034771292334608795'), sym('-0.07346267797156926883283606293263624094578720660045266767726663612219124083356717952673299'), sym('0.1533765662828536483647899987041212272026459641637776861672253633147623563817149709680236'), sym('0.1178052274740205727412220125436554061007109496841911898370159888255619211890868997417803'), sym('-0.09004110949843038214472389718725021173606650289716029174198267454616288900828642488978846'), sym('-0.1137911072694209586300072124025249415718277353296957752442721194363051403704129699419821'), sym('0.05514856384708637892611424152087689522778438935040983821148082927119580298218268335674042'), sym('0.09484137604087782086893137084624131333585240479638459109410918340212599872627609962384980'), sym('-0.03852527617732282542687843228224209768740099157399584020918993628404641009453333257385216'), sym('-0.07279494394047866730344529047289341720834562510243140691418901979917152984311517607902476'), sym('0.03081295202438112202534817582194572177042528285563408371394937213096028483283149403991687'), sym('0.05174977626009219409269577037685782004736840435214619024685149318402722323401890463089204'), sym('-0.02617202903623250301785158614883938491289070257438733099861662910779785348484295296980466'), sym('-0.03354412721489840633557312635310779031096208233433797170732646630036729771655622285846233'), sym('0.02173628325440259420857088333422970509387284309387976162093598700605104681638097091878524'), sym('0.01925335146034391232790717794188868751328342752536391589883766673392652798026130373456456'), sym('-0.01673844640770128137911472205872159975696200696192629206657416901092204921198860726294685'), sym('-0.009294197401558748329943667532778118020455764699328290457431608497372741997486175445481624'), sym('0.01160317977175225505150822469968050784193254946909167135755412896570231891349941755527620'), sym('0.003333053356587924024719593377827156128320236502184612748022234150874463551763644076706343'), sym('-0.007103245913937384015470645231109475565569228388002959025553726345734909299472089725367405'), sym('-0.0004380348916155837534227217450016569273116211133203861671796423375521784771487536719437582'), sym('0.003772379865886535588186379757146528671984037577376386252827864022695730000491290045667362'), sym('-0.0005439155852925213891848088051825919132785757920162490767365711122884519490170844187904614'), sym('-0.001696670154619855519243607055241180261751992179928543347535520255340371945940249003157405'), sym('0.0006074018877282889177951936783763735457373863753844149498994902468026643256104273041524526'), sym('0.0006188597887148358079379616124542732798530596318016707182177191421780433769079592223745510'), sym('-0.0003908430741442700022204871691035186292839872242387982382510331302395841386833643294009949'), sym('-0.0001645248074363819263694212589822012048300053597883072403433587579608850622593207786464697'), sym('0.0001874379137695425554053956444109178577489135850105142588431732661718396733032213304829389'), sym('0.00001880939406343829648158866919389931466493457539221525972549523838234505221478050136130496'), sym('-0.00007010750027133779162606017262030401213297836957756101902199022882889033577672969736686091'), sym('9.567983671285658677445769039711269292529843396960856663499045910590158264333494541591186e-6'), sym('0.00002011684441014615211142441468123567676742054579595624888190818654199045829480506546635170'), sym('-7.478767863625688365041457243296772895812555424307024354256078918983694006756751318720623e-6'), sym('-4.038164152106812342162738328452936631578731406856384004387458303252754657449102441563707e-6'), sym('2.948544597146877741475294726113645863878717140821303906336178376531767524425209361747449e-6'), sym('3.521192437912406530590739044898695506453090765794883448445232873633389439383561272921585e-7'), sym('-7.893479206519634745383945338374328259038944735243316435024106437974861494209007952824135e-7'), sym('1.024029005847650734910790354356327656818207028242070489212738168038626545895896313359158e-7'), sym('1.432591739021497470201332414069710221143468129315201442873772660349448005362921700507056e-7'), sym('-5.322181840614331286989889824730392191384338477585897270564774399472230529006137223302625e-8'), sym('-1.407387508144012272477629586437487024232723952101075281104184027662479138733779451519263e-8'), sym('1.230665140854386040537381713313976587381869800329638009517312282940151446108679785040629e-8'), sym('-6.127606531242232959192462968487423319275389514773259418128552935643370780336306909056724e-10'), sym('-1.638014235352346133494122028487844257774245025522162156591091693661896593083106041313484e-9'), sym('4.558277746612877875072330493607900050626041779559662594596942123577681814285553246096031e-10'), sym('9.195289546136763385700354850698995970867807093182543894512415807782456916262498319152087e-11'), sym('-7.003733241683727269954953621174431808230349493616903500889225508015988237073429379558767e-11'), sym('7.100828065717852912694497982919648028948656244580547253290880942464478549525392443710675e-12'), sym('4.299297868277776073314825675170452089594916899697507445278548542073992251791171954686929e-12'), sym('-1.490495611835466907542578053695146444026731605129305092730222095594516822693066598478090e-12'), sym('6.929258365877787940294476802470699695038867125216298038362366577743808750589035246225821e-14'), sym('6.066406291142883997140091419090182168917603548354849707528533502219710345962025637560647e-14'), sym('-1.638425599609063855119625343388601911199658666325408093597548281319974361444921065141325e-14'), sym('1.829524515090962631831831889363427480968065624560635498810240355981486663651776611200297e-15'), sym('-8.124266271019382121852588979407425070305629838568332078755005830345851449490446717460693e-17')];
        case 35; a=[sym('2.87646376005787776466689276508531064135534049754002719379573016307009970306992670250451046e-6'), sym('0.0000666198495492237899337168602764007743646564556976039814942491824977117087622438827782663034'), sym('0.000720628558154242309724694223647510661878440051671308335355095995890594613572776418361238918'), sym('0.00481348296002499044914032091071583133025175717861587042600229336710516759733158208082705920'), sym('0.0220873914255745329564355999675001962453161489398725201760473264454946704335417580217593600'), sym('0.0731179919445564037244348113724609644276334887174029045844960821760821024655974074816179556'), sym('0.177701151849223952591413974912403648610475580380487641999683394197151262621974000974323428'), sym('0.313667433990431263610121298020483892344719687995604390212197369206664615711830059618668241'), sym('0.379722300646055883726873272244174178317576953127541182782843123518979146384797704831514288'), sym('0.254802845981321205586229307124615283823882180713735291173293151619409736454847937311120102'), sym('-0.0310305904578481771427240151424662887835484951039599530521838580987076113754137593755629598'), sym('-0.228977343682612632164098048372985320908677840643537513738411137616571352122361080712170960'), sym('-0.128542804003154616082073851775599049771678126211530278725933076831347190847917948414592771'), sym('0.117408969839150937382927822780872545483909622716932732699656170972776206630353654122894795'), sym('0.153653801025959773234874348535167606279022048990671937246263002551516538118211015408478120'), sym('-0.0461478188634850725541164796804019938406672277989494063066890234263630296948539887719804174'), sym('-0.135707643035572942111949171312225848845962003529109085950363339053867240661596813037633210'), sym('0.0136539099749661039603532686498274862878662198094153575797165190081053080158002946621308007'), sym('0.109808365955458258266717132875901877858765061592313088984566610719242063415156050187890037'), sym('-0.00336065284661547285280140526849048369959133053383962442740897369973105541036398882081768617'), sym('-0.0852668407740675579717657880700081614707254945785602061165955856717283989446863367169029614'), sym('0.00334760555166630038768235053030469328119733953296557132871156434192548902959360221928065007'), sym('0.0635784792078020875151822863464710732779787687103045049059288478129131859961446480728700482'), sym('-0.00658921622436365865446364916137243805397155827430642168115973086261025287318973619756904573'), sym('-0.0447994837032463704816569366587768135775539360824095441985613779293904539668438017690693271'), sym('0.00935399711684183752281137965936679905188451988057186552908182510976148007662295304802361316'), sym('0.0291714732218226032697206224204414469754937770656256407494073478139718897058421508937175762'), sym('-0.0101588898356426879761743958718287719059743087268904902442175245729367764136903815933284348'), sym('-0.0170904157934273281198097335982785950570528449941331138325784519065073581474289847247953743'), sym('0.00902724811536542358366378395203457103405988327428140967645906054946808195004381569455152394'), sym('0.00868994353829887536682676627176736518238437291613012493120048042163292619271858604881115863'), sym('-0.00677252584338383974435348961022765647179925156244479080743270129639949662356380507409074846'), sym('-0.00359633918423111087259500121880685352741179132248921017316375448846824509601786800158728058'), sym('0.00434004788954305807346377892964188546073281871074838499112459500926647026916961493400631599'), sym('0.00100981127042395490910747264098701887375876549282340619905550137503282705756446100795586345'), sym('-0.00237421311056312446338749964341795932874105878965829252397093068141535786459148783036337839'), sym('5.38530363292012216259131403133854431516178680668960313611661041715468552829725646835400189e-6'), sym('0.00109575916320730394543479323632342193253044764913582200165329845158943948216249251020967822'), sym('-0.000236646872388566246657358448165728876047153355368771903818496997321885598534108147329653309'), sym('-0.000414704714693195310660465027461510074800511529867472886919977131113070660103568556906166029'), sym('0.000187265126740639482235518047272795606760498086745413303217120401097035341130931351032988844'), sym('0.000120209021387729778861653098771112995726536003027150982379185751836245323512680148307857143'), sym('-0.0000965825182703782477858290445990826017639098234238967989230072151494314641322244559906893921'), sym('-0.0000210505403289515654270823332352902400342029364401818300801483924084238124150407006354668432'), sym('0.0000375059557059599290216070523175298252440228043552013886147387846266488114747252654660366419'), sym('-1.72322030538190023174018215580821904978624446157197874610279714988106116646373596882965852e-6'), sym('-0.0000111188445586087733214796862905370851833040060605602883994904515295204503271276448725892194'), sym('3.04624985669610659016174974782862503830257123215061008628554402455491527263490702995991514e-6'), sym('2.37117359930015772231910537888566442667983301604734275760730346240027252546438994685793624e-6'), sym('-1.34062468932324724418537667577724493625276633846614983276345841009519357842772434564705140e-6'), sym('-2.76049660189680641194129249216230474802597798775292475497726834348762362155529457174730897e-7'), sym('3.74934080536409141459136568740718200301776066805852487288256155584274011899996029917986375e-7'), sym('-2.61651314671023953723354669414434362995839730196514936246917768724250385757618875137588782e-8'), sym('-7.06427742622599303336003052036749501954180657727978647486767961436988207274115460303347441e-8'), sym('2.12711059401186289666682169767941171974177706339246020361916007566526525116524830645514782e-8'), sym('7.67142079990686663698255500366698963822489240161936520062072284961133516909361044619109938e-9'), sym('-5.27368478943030525633948371672572387387984407108658208510209212635021120956569573780608767e-9'), sym('4.17048136668086600235985056487197259069158172917023086533790810837881317786022034371786697e-11'), sym('7.28902177798153225611932939782433296533091369209812267569137409777018106128852370852062246e-10'), sym('-1.72077657752631544555920026518235805146431783464391433056647415578297621226602072344346466e-10'), sym('-4.53109659459718808250560734386097593967189586276248998360638754343100462475611676049902956e-11'), sym('2.82880657751628235527684239466054253931089082317613580840880793356253095889154642645450971e-11'), sym('-2.21016078495502187432404617588250397448032924370194795429984010949930322244018364909194431e-12'), sym('-1.81518940593913160989151593238030830454719439440206149563453572037527205499353559605305211e-12'), sym('5.66752345398125660337908218136612247580391121356193452726972323455753522140630007473012881e-13'), sym('-1.83703112317378615816474075688540048728424153120505008018777108851121570402054819736403091e-14'), sym('-2.40255145841960705106655745340086735416462861380488713878435835082521643746257399569003377e-14'), sym('6.09811535129721274592739119547472459970510685894664451270561811077315321114782758772557987e-15'), sym('-6.57468771104261407762843136434203903221002504064068584309285183496579020706473043429545111e-16'), sym('2.83877118643722720852958704654457016817683125551156178548844844463365426271657627748219847e-17')];
        case 36; a=[sym('2.0279293446623982642992680901087125473025831198662809546461080202025895012821946109085206452e-6'), sym('0.000048267311671739782052657346301492608106791113939343918659200429150696402115811327628874338843'), sym('0.00053755325941803573867219100037691599574124384634292330987138677102965768187385442971745888498'), sym('0.0037054498110006099546470921863005830447325515403883783459428422514295770219985670998154627324'), sym('0.017600287755026764925037395450001089760241935581002833139436167199167304237081406697011203319'), sym('0.060565175496929312951205799433144057006998771957245854171874565013510816899268057084697545387'), sym('0.15397741818604664364780959877918824553479071775507572124104449200661956486413490316960528516'), sym('0.28739202375223071550046752786574399900892405103414223067755314404141295044250924715464048230'), sym('0.37636953103997109237658248469297025521195893765657662637138583193412522965248796259396289878'), sym('0.29548248349405005547726932882861932472868456253065855740042210056478128890151730124023029078'), sym('0.031095160377020326976797223318845815644021986029975347678553489478901772628815130171761173974'), sym('-0.20818711361745588308471063802429135647579156453041955143955764175579172089552373546206143353'), sym('-0.17451892949179155060098439768044348671934640574024826348100603194979033961491786630958606804'), sym('0.069377219094459846200662489896736453429055173281618984602215681775883089560575321722597966858'), sym('0.17432818081257604990015873852182571740523284356427074679475963859508805176075309819502445342'), sym('0.0051466873812033386722892897269636687577679663051954890085666389438627534666470612733303499907'), sym('-0.14095268982865324068359179355587305409668569428730640996924932865099901168057785326683538937'), sym('-0.032428907462488081824199635096558492234462885370145145627118561326615261702004300987617141817'), sym('0.10896956494254499238601844023977021261848179221994025267157443638649055653924849520121351910'), sym('0.035550627862154998470596462639924068400067152466584753302067829206964910547411268759416659403'), sym('-0.084006940303117428446856843836413192737602520944802937678717838392228933656291850111773251152'), sym('-0.028200022002753136697462333025538891450018756038581303543164805147747483336220346118441455042'), sym('0.064457578885789061979967366508101727984514445413140786204330252383798680981634609411114963472'), sym('0.017705049729230940232230002888752576992735398756592137600822707986883082191148902686002849738'), sym('-0.048231058201959699658987657259580881559784314465696340037528422341417702542309129476859760766'), sym('-0.0080038125909524017805997562685194003518262144311630810668988021364863849580995689985953683991'), sym('0.034303930352925337606858748741298759003392306727265376439690837835907723129216973711117318453'), sym('0.0010076078321397529560974876990066583502206927000707548661903485777531715613515533271729537544'), sym('-0.022613784458391868772510294200482813074721053500121196689829665107372224747483170145581838105'), sym('0.0028171418410325944708595889102815778687157550323980920990876979481927203949366081599149544284'), sym('0.013479997143172665414445467930345757814117597610677257035195781030262136976156673797432974953'), sym('-0.0040006781122681562750332904015905247155573518010384633651758478684060891448345125163336383593'), sym('-0.0070641830477975296273107458014418827223086618226047305500781750120740649788484629833713136036'), sym('0.0035517896591495661731506422010545742239506742553237993611557643152972807889956133534899750976'), sym('0.0031208050557403849428977551841884934858333419548494019303135706485948451165549888552675007845'), sym('-0.0024639428853713668601824209847340105154491501430659777103643079089173044869671963471431701899'), sym('-0.0010628338649039950989063702412319452035817899234063571589889508693426960958128271544284354526'), sym('0.0014077037760203080002648673080009525532768082744055013889643054556787216368805071759316996087'), sym('0.00019635031579335489088039850483339586431684883058852233289075691423748092762649995650825218703'), sym('-0.00066916370165346252883394037916692495102103328770774825033408777933318706771559884230922047339'), sym('0.000060914178651611772529021265230555588512359689774018876463344105037924221961532727600700358242'), sym('0.00026117040475624406814585218685186665324843563063681318175738286584460290625520632363187736417'), sym('-0.000081679240432767529554697787343306683400150615289998280493322363243771910340726852057485969711'), sym('-0.000080037378950411322223818573582423821057515227585697876974148307079444050959827373362195503260'), sym('0.000047338968986385641539226829705760709872317389299524163306612932140467564392405625167708105020'), sym('0.000016794540420580868318529783259960126397104904311167145191082342132350570291341315274997452218'), sym('-0.000019313849741837985158032261825635935189606497619071349872540329389566460351479719221980963561'), sym('-8.3684041185386039204405637587227808456444975419943623211508355173588647298397264858603325508e-7'), sym('5.9200522614929118076282529780460918496194395047454701708331718165484272749700211081310682642e-6'), sym('-1.1215744387099320049919065273477038217736455943311157321564868193114395111913154290687208574e-6'), sym('-1.3228635707042009439399734931071798216522742052819956919986131342495029093886349530135328104e-6'), sym('5.8770623481795429596926630921093298672170859513066498096840899646981982588797018223361465632e-7'), sym('1.8020075541350647398666972606432521640090327084228954278939528024409829781635243228127516806e-7'), sym('-1.7362141926527924077409604950683121700469872001117487986519245898846440186083966331845240218e-7'), sym('1.9468410900539472287865573043219673701643768961299955916690282454534492953117302706720855032e-9'), sym('3.3934361776293853513734549335615728706596538978226075708758674050289584782307979980228074899e-8'), sym('-8.1748168704947692817607311669026200280747812048887534671774833653835260174419029376775773791e-9'), sym('-3.9688378705047644271510308991914918973718502957831026322982445997729742791574065624945383418e-9'), sym('2.2194962481588342718280215815891227809499482612142271775003930434750628357183051181450749620e-9'), sym('7.7132307505475257960741542548546879531651991987310290463194411206448408415441620295448382577e-11'), sym('-3.1908517204368024976342855934002181018144855170614792151632883735105214123976383782004600895e-10'), sym('6.3373866877788889366583031612433012609481044549681321390988175402894153882809305806255934352e-11'), sym('2.1477867126487128352321114974848957542571969066961820579180308859695346939428244906992680441e-11'), sym('-1.1311705189539987053442096855264490987187474621545647931584031352662927074767268970167990298e-11'), sym('6.2768782052420326267648727215465730602419247165329960917825138216512479576947775546135550553e-13'), sym('7.5728969485831869484239403414910834751859439176944544121628407951869466169701606018013149863e-13'), sym('-2.1420279847208092559659255421026194040086737535534384178177058075943505483415674848366315669e-13'), sym('3.9189718795651425315442443923850534526895902516991261857914891042040951390642891543601653838e-15'), sym('9.4615935096378217790247179128605697862190304154982198531598734655738126002144964329340698746e-15'), sym('-2.2660145742233456549783246923311247741861806200389163676761617903116460125178842230635619743e-15'), sym('2.3617168394383905319008264532654054888101897475169043513406432956870267941141285084964757874e-16'), sym('-9.9226468527034361470413621705231628192888400671686299768591398586852352136643641219548244943e-18')];
        case 37; a=[sym('1.429812947844532119019302835560414682512843501872822521528103864129536104252822339461254771537e-6'), sym('0.00003494764781024107141314257779460092169007548449518886122809513494315117519112571616871527034570'), sym('0.0004003934432339205569954813254692844862349540526102184573275289086343959677385148184480983221346'), sym('0.002845496942841258051764974434491477803430236458991168226750317325922126096612356599018031471299'), sym('0.01397404655115727418860621971708878961927068577487143463545091365649914914632603064642459001128'), sym('0.04991100909733749105808034798223275041004597513660617561366926134900617955626561140991314967638'), sym('0.1324597195544677425964694606411100856689334732250132783696114965299591143961014896955470229484'), sym('0.2605271100512220700269739278190552976884941724876128150591009348617678979755563286170010218127'), sym('0.3663994283763777642937470659271718504129111428759378241516486679198005673359825152398095015598'), sym('0.3268300955097664550019835956378021606044352516172134203618188618191531660274667164875206576356'), sym('0.09254540241673339374659953442288541935436975996627994402430259126836691083644497455746883298996'), sym('-0.1740758512794806411696598619907331043843743863785109049704672490881628677916336250205132062918'), sym('-0.2081552059002246335247293317088257209636046543025209035525128736877366898121264177306476376028'), sym('0.01390985136597756166657667192909373532374009247525229573995819941039389230733778823973659800215'), sym('0.1778537987842548504753662653549629920140161151341003865819266968342462944464242450239271397445'), sym('0.05784559741454149440546064561139844704515268125239326454037044150386338650075156309433447273143'), sym('-0.1286667704368988831303344245825613476747265993126570039450688158984544911921862376747249681739'), sym('-0.07668694227575952639929576250180856480724995608734780023062775574320686399484376378500656822626'), sym('0.09187413444248667734867925964076240345535731271296181966266347657953371027604625773468847808706'), sym('0.07196953808590523290618748477675164609443760925919532378424905542220044101233163142803593758923'), sym('-0.06831184708381235166495000954820486659366667773344983710124496144267062637737073432191515562083'), sym('-0.05821625113565217925376762875224242406273217092234798999589677728185519250104689263385702302552'), sym('0.05306668097739630035225523552162306051819312996987624260407653368680691783147396892280616891866'), sym('0.04212052016498410225813449864218634536107429058834659616193268805345979150960447513391750447488'), sym('-0.04190089616537417568046437476190325310756965837485745162138991939808407873503677641467151606600'), sym('-0.02704954223122645923352297263749110786190019365364210988898230889757912929718784197111574821985'), sym('0.03239110794157648621681745351480931346808879758369321208993493126913375926276303734548790436749'), sym('0.01483000951949896921419193576064692281655110088562040876422559102621039796101829572631796100465'), sym('-0.02370475362140307613198892555512949866307649121578302826629995743343074851736279641396145584223'), sym('-0.006246223431479012614694663898465937647975216325036962232587250713538210847761019800152161574225'), sym('0.01599380188848186513827720868473278085010366120457941584681205832984289844048707177746803434865'), sym('0.001195344485770421875229200918088374121381736478850943843515029506822757635481207375707476067894'), sym('-0.009732604982092229156418311743003454002430550213679401239190598912216378238640716696959631094304'), sym('0.001074311418909005588517468160222779120674555661546350211470400542796017933041384488698294143119'), sym('0.005223933392675638869336976977804070230879187766548155002572884285603084688099242950974199985016'), sym('-0.001589613652998434279394921618134531999805489376979760844358938726417160041608770261518691589701'), sym('-0.002400290427643955890552001201697885709128610079745523734965883916667853892128972209383149680906'), sym('0.001284722047745501743658834384090173851147879907397724606214057394866719637142170679518638514443'), sym('0.0008937364848888562173053912515344401825486505845856090777728708594262938034255635683107155158960'), sym('-0.0007859384854530198881474157822702884245136722467723349799041156882446774061933043652903073825103'), sym('-0.0002319867775398032308313836220176878125339064557577507190098367526949231181137098646807518979677'), sym('0.0003882392956379476185996324113874259444935145236509975371923473052401027828854625901095616318037'), sym('0.00001085012238618802410385474059518784999567632124614359506967796669535237214684720967347243796551'), sym('-0.0001561959304610836824596650879928788142624879654399099463442603300993958478508246776308935032027'), sym('0.00003066528451805066646981016664826113504198464730547829132994347358342316718838634514297462729120'), sym('0.00004988736475010690215142926951789320986394558185341469254481594844997256813483484037822306004718'), sym('-0.00002191085568731412628910349231291782847234074241354370009740424500696952126625797818873306494626'), sym('-0.00001159062916501816480133267975225549127697500640546291268786074230459476254089583281506462278092'), sym('9.576543136414114856297488845569098877401313354607845684756954462923285255860128121619586604128e-6'), sym('1.308108656525202752754750515128504191504484379915520694313965104461546551467756490538255325659e-6'), sym('-3.047588901187508240855481441098617017384420089653032631848843856784147831593547016612287079735e-6'), sym('3.432813491655403856587851979970402922471847517571877618621025013170537937732622674894919125273e-7'), sym('7.086068370151862023619930383680692013900222112586222166145646588115040605337460393485899660509e-7'), sym('-2.471301857394927993772647910689743726868239245095271903613021001481389375725068112726741364645e-7'), sym('-1.067650197144162694062959140563317970359568864521376847303801709881769662114321247444188944403e-7'), sym('7.842035048479169831970168880327397702663931348369067332115643401225485914374637472587529735230e-8'), sym('3.783486212989544444308540801467965110070821361772939160528861328546717072758448777142494838629e-9'), sym('-1.592541534494658179397954100845745329512149251679717896271407271911257177300107800546458658150e-8'), sym('2.987162489994508858391881510233455167435747682546655903349697530931627347633022446067946723083e-9'), sym('1.975638291338123792584662788656436343334803365965794341657408782473878107572697708534672314764e-9'), sym('-9.172624531281429333046410170970046244199551349719386479489184778028994412760059330258961277095e-10'), sym('-7.293178035757445975906415361701036477303137640903955004851127205116323301318842291185706737314e-11'), sym('1.376146393912803966812438903030452091298955581010561039758125333538515495503999859416867415344e-10'), sym('-2.265144621260623383115164326267576522422227789046171463574668454072858792773002347701333481107e-11'), sym('-9.888292353745043466999112522869680101851544524833497757819804644660899828147629454320037256283e-12'), sym('4.479489950827269557609648143161736679173310444439476326801472212760599908154122171771953380325e-12'), sym('-1.482352630473318933287829100501992815118008957361495228420672255955139249446892161290082542186e-13'), sym('-3.126552118799157896909757988625519125278307910807137489672739282352023668498723378578691024310e-13'), sym('8.047248740930974160398401084898255168856431314045137993255191274979539655306305080700866358559e-14'), sym('-3.195337484871016814360526051782025486241644368403095549601842071745149969013409018769032264166e-16'), sym('-3.707379020666612073982068008741518488764383347762840563764171328966314369683281781117725656153e-15'), sym('8.407587221218926222319207204675712686646647991900622042852752504327877897567088705910204809051e-16'), sym('-8.480192580252513294094206315105718227458876190144428703546749931224335954477039637172573169725e-17'), sym('3.469500785087754751801677946327513786102842058094810818950596004218403578518424197947554656020e-18')];
        case 38; a=[sym('1.00817633178516080045011487156771653821565735552863029976524328090464791005465672795718269209958e-6'), sym('0.0000252879203637600473281385610246151758424976637213089407162641548373848911675304825639471668257758'), sym('0.000297812351456999624943567395511154342010370600071810083972638750066774106649494756985171157070372'), sym('0.00218007251612000699544679218145081707466271427092001156495309759566251867913347064531614929316780'), sym('0.0110572050527731876707884444881403376554455080660944018463239898391477540702414823058329696965234'), sym('0.0409343716911596447365259885317971174430035514671569921312985935646642180215847522360564849150831'), sym('0.113187992127232066454900309583646689850696029785681879221969555063695276576777579233547684642873'), sym('0.233893798088007611072044170492430889841112857763329742333427426896406768874058119786080854544547'), sym('0.351142987540313510009454335952315808593161718135961204737021796308381707891437369691507588394388'), sym('0.348855428659044753519743288401471278470590821807568626734216470323710223910042990988012256678430'), sym('0.150649503741197985964726510477162380882879517745203147455281421873188351672128303779617036311951'), sym('-0.129306967896332549312572821417089451769889855840822918735457993180684631924903024239977266790466'), sym('-0.227459024837250165527905462423569014690330460751022427956427585668361360599708630536819297585462'), sym('-0.0440290686672097511276363191840169573487346765098064501825427976210056356137932439858497650586482'), sym('0.164137843117441474454453757979497493390360520614718993814190783767680292439765700727275749725368'), sym('0.106055495159470292094703101486324105830508055937149915267711112852708331148724090327954001001510'), sym('-0.100264691094548675989856213356404633123471110485187038476109737562620447768725155793901120640100'), sym('-0.113075259220330219455058949112871191821089073197136449653669186469952757551572105311502335181594'), sym('0.0605552964804322645298988958278866828758712959739218775888052956616805024402430748941224877749689'), sym('0.0999953174229812132537028954217842088585730178999485773530378178793776791948244366293678606818480'), sym('-0.0400126686211193701626625654540104330838015051768729998180960519609295646199598234782468278966685'), sym('-0.0811271888230387531570984484199319716187187099692098539100515220464576056269183860567214467612673'), sym('0.0304733999020143523823619583189913097807132112883488591811075425891034075083459171250411662361257'), sym('0.0616628213603806868134150274886463072766042232634527839758333286559335011006593890737776438113680'), sym('-0.0258837168422071807817300014849732727326288111459769630650856173615024287159971562063305044079947'), sym('-0.0436753050259025304665969690670317512520382044273352840464049677825246344585368984447960876294944'), sym('0.0226202593318775566577759205417997029318057922848447295555917035371828809136972482393993703549038'), sym('0.0283231487597265183352789937694365724583937190309948674448041233001730673616183613674146757017545'), sym('-0.0190151576794170590786164472573105820530106121968794993150310246092809231462860068588975444780056'), sym('-0.0163441609071850997404593251223152312407844123418249866946928455843280396310460216389993253813920'), sym('0.0147818164185730110287930631979850706858996409292871915870849596327168292435797848564095067986747'), sym('0.00798358718872710682639468144866586043745131036554551503474006493071679909666215513054645414131289'), sym('-0.0103958005046482933109331112244667871988381935574767365410329675388492117864085316133451572420044'), sym('-0.00292127495164070280177223489509939855886134900639370797829968554057736415658278989051452121088131'), sym('0.00651583698344292500478088932500838349456763155608564548258133887437073920937541130643896005427623'), sym('0.000397798175472409075145791425477461921769799337781123305616915928656252371978233952534024054178408'), sym('-0.00358596087899802211399267274374412585345356048112322903531390569382013025517447970056781839586382'), sym('0.000506982962957364916340841685624032185421214534553258256610694404183827078607997433108802997859899'), sym('0.00169754968115461036906576944907510499203962802207380610536664995659257394805599936568852200003178'), sym('-0.000597408120691525010960801487405947335683067502486099029812115885105636814842869343733122794920376'), sym('-0.000666420852407367506174576233455017634527336017790945674901590179013715490323526066711969616809868'), sym('0.000410882762344763910511303486625032684544336482826311078439020767792039783508951573809786665123710'), sym('0.000199237182088155274265085616642380229517333206704577952249892864912060161819041613147660664331430'), sym('-0.000214325512169496089118778955756218321820451625336103520046281581795416963520695399313698149562000'), sym('-0.0000322135412774845741380950699322613687264056902302465423264121402398944903423588738032320606022265'), sym('0.0000892399411053887845239457529394212130483295520336131218428807586316061238539906496841252338146961'), sym('-8.16997612365493231334258823742965090957812454021257182307897761001643274755941517138714279822665e-6'), sym('-0.0000295227097209729637346376633425752340416249131890300637613861754429219093072983545869661314367543'), sym('9.43405002906746730058761445619334249044370090564819703215299847358518674418055016750167130651912e-6'), sym('7.33523713564787416401970576401855687706481694843928556272346200119386865702360750056050598045753e-6'), sym('-4.56559787026439023794677176150098292988289564766485465268657013981860296067918501867519922349580e-6'), sym('-1.09661255653370820987565359590613090387072015978686728146667494349237616432760287976510025656398e-6'), sym('1.52025148615599749521517917985521654753987987424603504741582443327302650773112743700576199351614e-6'), sym('-6.00127718463609689792773998418546078025310194048358690997121225810060298356218540166368491072932e-8'), sym('-3.66828170574814971544165322370288737288214392262280633438963812711776929599895957248366346855267e-7'), sym('9.87388031525585027178320943596715996352148713016483736398651048685750607662445028532433359136036e-8'), sym('5.93994518960765091753346124084531861313676010691322333004962581319890194634301679670187508998274e-8'), sym('-3.45404546203227508142225145959770777104983671354820188349265359974836256835953480922762363147876e-8'), sym('-3.83554149430245456105236232818708545686514076068866457087544523395476416601261682941490047647557e-9'), sym('7.31646596245754824964793505791387287309347467313730928760286321646599838424725131730094891665182e-9'), sym('-1.01563832083814090730418867617014613775602299247574450846490711956889717170988249649298185685256e-9'), sym('-9.54026881003355924629657745380652704931395420484123067084378452060678095760585118685488858424074e-10'), sym('3.72018250802888059148137534295453481658766990166211557134184906450635445811000626453369052185774e-10'), sym('4.76048078544250085106768028604671673750503642420948491205746495832142369587080426552075185078053e-11'), sym('-5.85361132348848265742175069008009664937871425899800241009703433881294577716800729679667590260286e-11'), sym('7.79014544840581921619494680798162129504852081637602797366869856563553495965440858653744826727729e-12'), sym('4.44878870096685407411280632006298816198834430594662129405644825495907329611733533709857352827978e-12'), sym('-1.75701131970060306420293345010521526590051041451077282420077954334787921598285942543723958998128e-12'), sym('1.85721348878730022619739982692660849204299761327808022095978315098631277522033906972130969544272e-14'), sym('1.27891662503896502841971837185459157982282388322649685282644498113933447721721628016992381533832e-13'), sym('-3.00507499902640899971239535490675552946936484340489449192472579498763260223510222528319364203778e-14'), sym('-3.22680907858771235461555902998134607523799156749563217419941669426496311120086890043571602362864e-16'), sym('1.44610384965991064469872356409642611270965024819190783937419736113600271381551520289790532861900e-15'), sym('-3.11502248294890971859796525313656259033655105238388714042301777654722508881821305606947293426951e-16'), sym('3.04380961552620135261018660122608627248295578951929667960885408235478742467873843345174216728759e-17'), sym('-1.21350303571476583987207322925910852870848297868096207982878811406010158168472549315466629755162e-18')];
        case 39; a=[sym('7.1092392361172766936822219352293034768973652229781398258537592281674310834336840198440368353022838e-7'), sym('0.000018287418550136759264865895165157205859844184027771558874705977528839325613905918333291746967177052'), sym('0.00022121819939742011546536780454686785886129447891641074854296221379727259639871206716121305417948023'), sym('0.0016666115204103765120209972537633448542508924802901173325754207398943982179198725141396697271133023'), sym('0.0087211702669660107161139094348248912524694231813187795580944262403567283589226568703600666781208308'), sym('0.033421673782314535884810335455350749946207884219214861591103369156294839577804150850118790019326935'), sym('0.096119285527844309661072933324141340229644347651390850775140389225491865212464382745581896434905993'), sym('0.20812656144351409617071766135695584473186080554375691340277145969613163513229704203727207482062161'), sym('0.33188824804256694867017257768373152241303490139155731212940098026947010282384956360124396940579250'), sym('0.36199715086259858218196537485846836880630519948585036756748045722401698727241595225367325117504822'), sym('0.20329811186665304979987375696016467923186536935147041324202001587345792749156027071208185447036425'), sym('-0.076890543859792644263633601750331426713889778036488931876536467323167776096335510740795547536178663'), sym('-0.23167173555799800118740275637765880117215073438766901043772374605894803165754945585872344140354721'), sym('-0.099639100891817402254194345860546965710895322855566510888049533303478722290626157987828011269346183'), sym('0.13478736195861793572487102121823864789980771617058609059230287113746196317519717126106929646427071'), sym('0.14466240992500969146337738386067091951914231578288389619314035170546826970789630262642481768500251'), sym('-0.059091224351210614133410459359777511377196180703020091386667120127080321767721431920165923560788740'), sym('-0.13674450186133176437884761504580304710387806426366851293379810138174257306326817737665555488774447'), sym('0.019115030710505923982335069791975984385707839768071226148448935217099289508239381342777363295527435'), sym('0.11502714480182454376436032861891294006288267937959206168730825333164946235196742692886178444157045'), sym('-0.0031220053762746799515608696602322341319730390776661483263555017232083762039961522032631388649179884'), sym('-0.092451724765711899740844044679823596168037189122517024277555563820827498921964123754648564970130520'), sym('-0.000028323562378698735566320894448739988835278655752842090159578652933352376213741174965624594765480100'), sym('0.071984775003269609985764140697530960209937046394001494240139192335125895991446794194247968088063237'), sym('-0.0023716034536523391130548444850952638400998398487205523782297876925377372686168610983563643059103219'), sym('-0.053871319525905310578531023886621960916263196712235573762506778434102697847903849620006077294362043'), sym('0.0059816501248995252193456454743566697759298090856207305478875217783076154529668302329004913666647521'), sym('0.038111880082384683188384214529493909180172242973590117455685292421516775683667032698724060992048784'), sym('-0.0085131393520021761381443307933510096523499392476672866550931951276123974570097358834473226620312022'), sym('-0.024974405711949783340645245526763213808655263545478413764038495340264407827236299415868601149260929'), sym('0.0091640455155573220456192477695280974532770022083360371943759389209748268220670023971479202424014165'), sym('0.014796274021028020580841108726077679599749205114481308516986099599099780460079076232977586142229346'), sym('-0.0081459104758777696064282049514415987471123546478129125147982762598295136781888376096446443989762523'), sym('-0.0076661109631829740528201715228960488000561793041989623757886723040962708240299956960591107436291874'), sym('0.0061849269049719667857226604738736061345392958690114074956195249986254368523247688563333129067617228'), sym('0.0032701267417323988497895752588344819271957454967574188951814621973875846107177205933359125532870342'), sym('-0.0040610716590598256601147238059901185640249429679721489661917602079491451900619425980787313012832343'), sym('-0.00096949032135780466040090070406155800744152392931079314135356126687701087984150839768660661168121815'), sym('0.0023071818080973687105426832308246261225034152706764890253852792969100594159706665212471700163218279'), sym('0.000017505074378346294899384668777870307269323157782602482551194861475775576829684110903153351267336848'), sym('-0.0011234534284276381753281034611703914466375303589671791339653211412349153866374469899697242251137209'), sym('0.00022854541107500962888929161261953905818767864044028868771930671687915574649566698693231360235305591'), sym('0.00045806924914497081727643666308895511846753893847635480841721029767288911464086040802906110496355863'), sym('-0.00019835916632967036971983287526681431957206062884217810797046787634859641534684863616440196552354112'), sym('-0.00014816708112477288453986018835511739699199544195677774706841345365002931560207415540006094107277170'), sym('0.00011240779618994104445711502870642416748323316515731148084746795341583385223439798402413177263049849'), sym('0.000032218111948934714905802313079881462941446606658375065351900308403021552699034997821809094215165157'), sym('-0.000048861333465612165309035539007005205133937511640443960626830603367603597257133483360546029874073998'), sym('-4.4267345501059861640806700765634892837863881064650292606224538142623537365803326407849328379087042e-7'), sym('0.000016723995741713689209490562595539529256773956897010711395956307531837315348436563470660513546114184'), sym('-3.6572102032134582947240251817597267311919402592519850143580608784511896226156750200850299028943972e-6'), sym('-4.3676475121760134136464298588551664267537765376255754654384666978568048431957596326225461219364604e-6'), sym('2.0850257503760166505542017115561654991187770475723403139676978668623214083708008173763813488851098e-6'), sym('7.5767491844006495044737841373678216864772063278806832160857800978383917685233079352637340899440491e-7'), sym('-7.3564252378524781597937890614220677044551229316104000139234600743717374229285321302737107116991175e-7'), sym('-2.4641227298442542979319340477052744982944454484661504429607554295668315708182392108580903332520839e-8'), sym('1.8430190630738865022766093734120737305879270242495961010122848549099596811305600046983351568773436e-7'), sym('-3.6809989546863512536947388067564104004288952256995528409199685169602678221705893042701085447718579e-8'), sym('-3.1566418547598889046833449689555106721781213719861488467789039347874496078529711155058427871948616e-8'), sym('1.4819393427909147090267983884642135121674987077932276038890225598615475797328744371930659903566102e-8'), sym('2.6710122194806817898329242494722365085871665243781442152219035459358263526503859712098486026685862e-9'), sym('-3.2951914497576738500863863966004605072701113412585372820500433510696474894277504065254330027231630e-9'), sym('3.0610301618766517525260972754338554203400354934634277105189871630347112711513469572507908589237935e-10'), sym('4.4928892947537750031133479145044412506845319768105933203228173311351735903154017287047837043033620e-10'), sym('-1.4789314886162086190999335758960391787316748011150826169177997579645630437735284519304050126640156e-10'), sym('-2.6949529702380594688436704286104603606315252537214027140172510417190305619090253049081632167315394e-11'), sym('2.4577908424569038853511117451451487885285072232938974873134927548234450787405402010461624258463032e-11'), sym('-2.5387663759943053009919540328454064252966415356523442388920338959366397550499270396056678421609043e-12'), sym('-1.9639880639991683322283700555997012647705606093508653467892924417802873610569146432871439049553336e-12'), sym('6.8262354140898231967716058141059451004625371801829430757500745767366970765961499398460111006945034e-13'), sym('8.5022746391981671601410497753437940839730540647243516912634636855937948665376602068415708411557674e-15'), sym('-5.1879984632214328716696075290108998024829172676951696830001812107213171389097949815252302819116505e-14'), sym('1.1153627535445574735281591100659111373987348853353288482726390539327910850187426449333148324772653e-14'), sym('2.9027368258274204007756454114939003589442474576016206293994257707072224968335108871604860631280316e-16'), sym('-5.6175537035743677267886299471926994701047338325343655561571728692487575732019057576346822533111668e-16'), sym('1.1525721529598474231643289796652834707170094455741269031278575273005604251689326334469056254275311e-16'), sym('-1.0921220889947098266087111897072609098585313287505999981349187140300227646947087184049901291297557e-17'), sym('4.2456277710412509097244040835241395149567777244788593045762143705980112780581755305400846954851289e-19')];
        case 40; a=[sym('5.013461937128117092174277443591562838199647856793979627344616537608293742942406435144199376060149993e-7'), sym('0.00001321747766966999569008086132274489228821523221975184710464129629249671156885450616517778382093471299'), sym('0.0001641160413008717265416224202331134462353485286424852032604083266622035689383118115528018616150752299'), sym('0.001271449236903667757715266776698255958661948282210635296382987042469745260481802255263509852244512284'), sym('0.006857859433128414054364956510806007066103800508839861712057928785309513374584095380873913731064749634'), sym('0.02717279609943704475010966610367022208032513540037466629150402536771966876325396607086086880720084281'), sym('0.08115147373962958116662852505863560879375257689099634805157606501608738700260470535462318568370881203'), sym('0.1836906884770794546800574037736879607527948023290975167558721738855596528520605326005375702703046510'), sym('0.3098265257450765848428866351741468306140784673357770820490806849075060701824957298424776032225431857'), sym('0.3670111046024498141521707113004468497066930920954701925453672413915457347803887681821881739579143263'), sym('0.2489693803457026392718887965572668431488990226486031557049621969238233884456704059404356585163040667'), sym('-0.01991959057356657575878645119006710169557908442521141713216550348456564976226959495738590255134196291'), sym('-0.2211533679777950244814891270302939169079323282244876371321546733811568912219548758723038189497387925'), sym('-0.1486874884160709864725580423545535418497065810936428638230797450571558434303819492124034011099089264'), sym('0.09277099898506891756451021822894959095413305748090520696055385450017454920529126917300779861254142279'), sym('0.1698310594602032902930614915102992584831551411605729970835745596436595442262058125182107619599165802'), sym('-0.009864669526042107305670269634070916431333925774895175743818617749583300738654046227611374179969468125'), sym('-0.1446212757067760450190907367716809306635593246374415967463902311413506064047817872904223104856920193'), sym('-0.02697420564876644161565847877849804384815451675773414723987793495195320568086147537724331643345278822'), sym('0.1144494509205875276902922488098289545767937435128474410799400876287897242834180379974352736870594394'), sym('0.03688695327807941939932658038535528455101705623119710514664977366006066360379414223522986893334774032'), sym('-0.08961327883238254988296523203734531288813662107368462367376576605258173989476270522134796367898086116'), sym('-0.03352462866915127458975360286767935710145210651696111328018451386054914841543462724228194514643857617'), sym('0.07030064353830064255290433893460469413331898040895499994670365122512258121542203412369151319156936349'), sym('0.02469808172538777097907899477194971508910630992625121445171757813285030265544076237883163346801721122'), sym('-0.05465665012553488999544413083805283450670055908455453765671203501297827210913129371433090077528170586'), sym('-0.01480947004455271092793449504523616540135948515180286526979354501707181112647226995084164129029208639'), sym('0.04125332782811829363909983348315339859386368151638117007510094363810302442493638604459140460301728984'), sym('0.006328664320680860747806049797640563730445888773471835550899815506104648190329179515995008415021826843'), sym('-0.02955344087882814043823794987902940925525025013956322159492898136622264860463838929294045777172492944'), sym('-0.0004205701059603185878866910721580848795849519449864050307918631819441891277494598765404347108477720945'), sym('0.01966490783306284928660233838763085599737070928102176183224866065974455050920327330598853855288698075'), sym('-0.002749282149077906475379254282094669034242049863885066486200419409464769105982304819805505516797875357'), sym('-0.01189476340984430833210729843324186500749937368382435348285961359217661280768137157055974354040205530'), sym('0.003708815034171168511424927729371883234746089504838519467348984129546749473333003386716210716069699416'), sym('0.006377091764775359155779553228737611310946135803790408594072452795863894138085675638282220192673848940'), sym('-0.003304715285743075544871298258337769837898646022870482832917635270698387576366697672955857871296984260'), sym('-0.002916455128012535919917418236395195602155979249355596540066195500744223186050033329001998992659664210'), sym('0.002348519458665756069747843210734328315202280571530093031184954077271167236871966514198409210623298377'), sym('0.001049296934846029598599988041128194730639153395367467563752642268010782054906354560603603656825139895'), sym('-0.001396365997678782530047161604066752263534874290220831765499162657302947926645146833004236067311674034'), sym('-0.0002205280324947811223998008090166931618969071146813564895781180811000379951739352118910586571001965664'), sym('0.0007020982646906082613565581240380436498782635457121604439915468184508010169311978329795020019169569453'), sym('-0.00005009762912365104138555910719741054732286421842451955169211932573363957036323983225619996689015292445'), sym('-0.0002956855194771251461425507948769138215843026025289921311146239705526273825999399292749576158429567035'), sym('0.00008563981638412683730154177229376460834484394528585095604135693613453188033703974445795286427298204396'), sym('0.0001007822406423127381770452614002278647891125895128715511384311987127033136262335265994377413303525442'), sym('-0.00005570928339541461702399999589968534358495062506555115801079754101563240650362186680261620943941538405'), sym('-0.00002521876463949329145115253040745397591302220833828088434765663475429957143369462905200359712883581443'), sym('0.00002566452374595398158348504772917872300626562089954430316281234810930010866837107428870586232805588867'), sym('2.875108876626817001948719525539394921878258137525691665491545424273673586953433347029023382329741999e-6'), sym('-9.113418640767755050483589009788737899670609612648867622428476417678044845079758623823822810781732497e-6'), sym('1.170279913196933520563387194736158105657983685030120251794958677399915944578023411063922374862127201e-6'), sym('2.480412704396231618543319909894925573092087995634684793407786087220389485501151598297989068409088728e-6'), sym('-9.054017595754776022935219278961080931906567334685827556554264095085731083930101731847510883067170858e-7'), sym('-4.727962639398220428418277647739412419034810384041809712899972081187033941465145703070935747435954785e-7'), sym('3.453131394437022545165462163780880829175683517178252717439495041862572138856232716091585837662771492e-7'), sym('3.790963275851321740024167846322693591427671518942886007257162374220624324642533844724128499473951963e-8'), sym('-9.012139070747403260877878477847211360094210941173224556504293573445756996391213125751621797328821870e-8'), sym('1.231122007756176498843842895720484718158990181373007781372532898405049611358051027303995273709837888e-8'), sym('1.618146088914767810123089139339466573652691209509988455499478593496125063031232113964643991074908767e-8'), sym('-6.179921269061117050067891680074337074373651826146081762847088368088727695442464332007134797230406486e-9'), sym('-1.607489833325158261594603048265010800020373037074865146912857348724939734554183144829272435321578448e-9'), sym('1.456207712697992325936770612627010580469890091709426778882504368031083337768659723180801848338252296e-9'), sym('-7.044703385244627490098230885642118641468441626827046165150836760776626188976741488252322195491366162e-11'), sym('-2.071000585255253207483488844148507139736631701299285832876253713575409802866476337680506489644433927e-10'), sym('5.751555424629192809408151353335102924493769204681632953905788801936368685395414098740485170360926763e-11'), sym('1.410889155627189537176432482735258621153031438324188471566118553327289966093550910662133888014889916e-11'), sym('-1.019271702972964611734707001195158418350149351447687685110515444741391215597686005192543651374442804e-11'), sym('7.603494937896098997225965406126393837290033920676816520717542508147583492145058832253629265808937132e-13'), sym('8.532533525376752167131615105269378464987593286289404420565866108156254819830055034776022478770078287e-13'), sym('-2.626673947708734077462157219567087732675296413169414488388195251653873043183975782269230470679451648e-13'), sym('-9.561990041843332051867114760506308060431552959189326375558072306626647032520251557905293898041571175e-15'), sym('2.088672983313987298314057873927526454704881258215504477496748994415343153026642826772693028112799815e-14'), sym('-4.114021407793633126013252771741982356905424393754896138289127873191407272776334724897060148240607897e-15'), sym('-1.732567377847835105083609227537109498412759452043446988372498510543241372259569688678724302237149175e-16'), sym('2.174047829886856404947932141554746667754708674527803310760046247363072901065658853149284712709843587e-16'), sym('-4.259171381661249838167036396372532813622613874246269129707521742456716306939064653556540289790894969e-17'), sym('3.917187111238007409096573412462276947640896031538310285343047228837709608654837344276882001128639064e-18'), sym('-1.485810604232397210363078332758423059946239714849840569437972292930849547544221780365515121326247456e-19')];
        case 41; a=[sym('3.5357290514580538139832437945456408865058172234808422709525245820760973039474873244601401772980427844178854390e-7'), sym('9.5480237268784095660528322257187044046079273964635181622980896749160181899752514053542299781861716581634165282e-6'), sym('0.00012160758175334167635499341326441724100926517158666388125015516583640098468120820718021574939544547675657400886'), sym('0.00096808058420969723287271551799665577260541106871178174182914387974980981288014988329058732916604787611341256749'), sym('0.0053772289370525376075239356459080725529724494640989025187594070912533873079743910767484701794196446094260102053'), sym('0.022004611243211395592794108785108163031444198030197123787545629781547834944738296172023376194227013602477853996'), sym('0.068143011379653244456480652908164149190008909608550957677023719910690997209287883999047379615098751964530236383'), sym('0.16090388432646786357762403294057514544796199466738474277168858583232449230151380046040528992221843071616982522'), sym('0.28601777337527079164740723667818208511197937118079246215140394213083115775552308881115284343025700928461172923'), sym('0.36486502595087635206725669822229223041352253039088291892183917362353238569131957249590075264598782367473154830'), sym('0.28670927567475652263254628314156815524561209410574172546547848419124634215937975119590940263267004108569800565'), sym('0.038647886072402479582081451081325558411013893899197631312230901323364589318950014132759392619095161662755265799'), sym('-0.19719431242486113329599005205874537346695507833634835195925974838365269920638015574156555285945839876796099728'), sym('-0.18779545538877858213816416338119640498162963018771210168809078042045128202020440280323822703505245805241232480'), sym('0.041989521238442455959455213443125169993340379053803800201905564542510035758982937730257930743029649262079973807'), sym('0.17929527009882913959988678931025288834016975991812316050854309863264704625785888643998244791614788980152806322'), sym('0.041987636203075244000462336147521348242011745454683447298799254342035117145582093445990880509714904861806960988'), sym('-0.13569077215611009144212896789113569083748805220082559062737547129332945656208356961470375583001202913248011911'), sym('-0.071710696802714366656912817019610740070037482459346467814987002794183956290494397070811199509862983796868028719'), sym('0.097955325255140211812576519052617348267352687154582355562386762159526324599060693780910303549748507765520521096'), sym('0.074010382571275756447980192076923646208501191363046747437849725278324448302044831938402714154733954809652099191'), sym('-0.072373865809206310540275721937222319526940064863456868161622507331315418644749674168884772568102667262441737770'), sym('-0.064387359257723469288386863609376043281680553986531908580705072968957707685950598914807092338860759541406511110'), sym('0.055973591824891340163635792598856594333637849801952762586620324958000225031183422165314806884435106976933446295'), sym('0.050398466509133533583341189330116562767895773719624506650429097014897763732581355579078564917599164574440959003'), sym('-0.044800469772808512597506103399139828776739828992126221688508422715618872794099711799626734444220264466232417132'), sym('-0.035813386536356212604556870415434668426988378752893338488486580255313394559484599017924007362618532525404249426'), sym('0.036005393059063497955969103736266097689601286999149381023634974489630081754247598665828894829356992783969261901'), sym('0.022733187232152565503065712425870133288843275595218149613785861628222392522471193850907661191074686540252627443'), sym('-0.028110577051896360650212900289575374116799048437409304645761357128169880600295168228805315409586831435727377619'), sym('-0.012350022136334703288619652338701135749889748855844209258950115889472247927374889094825180324385730157087234130'), sym('0.020736247081890325294501288325572403703299347510858701594290774918425410691205005634857345174144324419755652912'), sym('0.0051271530719983090740605737626069001502488227285253975246871169045029927920305305099398910029570655450189187825'), sym('-0.014141054043732822732891869096839856475048084312659392466909356731986407370567434570712153621965212448692824295'), sym('-0.00087833758503179703818908252587478633910465142720186207441278297793976868151791889177666281766616046582026321071'), sym('0.0087504969291535959811304839148549934481874151606923576636784058767704063609085356951697034792804582852170258667'), sym('-0.0010508563676439810644349121892763524541172973163678144490727854546745625097771966419782031741340660766971373549'), sym('-0.0048182065199901151549030395875599419818447948510081678143717773067266603106605499347113786725598233953052867738'), sym('0.0015038756894295643875055866930809402349505712571046350177176639952151296537006936838663236719158236141879272166'), sym('0.0022982835303599610498892681089819517707984980773174797137507660702883823203193904478586805765390748755333302184'), sym('-0.0012437720645346999205755159087031286932692839765891732750658765321612982435522557099189096128376056796768736761'), sym('-0.00090452603546127971404135992902915173157688736563419549368273984660037002750073956317861276742144876754720613397'), sym('0.00079396687601009536101906340838899534970495119436564083745351161417121753445915191492146507427026633064308179584'), sym('0.00025851293131452192142824035931488890153767932776538012469029657170171527265048313861375162389323970464773317536'), sym('-0.00041611672524642444037647180983181459017733011822726782934421624931244311122149714276009651121730706695550809727'), sym('-0.000023466467930457267733013525403955266433985368974774794316629579238107793357424332845292495740552792113023533036'), sym('0.00018104624303607783013940363655824459729803820590295009842993149142356949971226039946568161070520289055776925871'), sym('-0.000030309364596581327200385815621755958535925557130754476793178038907057087031548565712975679450593881398066176951'), sym('-0.000064248101870495513439596469525069625473593643041800669004030102136573005928655266208389653684228674822342976398'), sym('0.000025783142397555556068278691424457701915761044238940701351617603967010909916307656853147817512028870011724075465'), sym('0.000017483056476163031278535094199221447739392054748372706454036235158377436178665731034162845157162442681516067115'), sym('-0.000012918341754444727680439172942047537697992411514877290501189501750431726758127122060762605261550897209268437669'), sym('-2.9019909179490282157878683736856479263167558780056171016741282189938526277815423080325671453998750278144228971e-6'), sym('4.7909402888540602760219687791183552326461316572214277643568245617820384955663425518790258691260497493177345465e-6'), sym('-2.0652570589738034865092557201023308815181966579958575167485924744298186217296114272968292708345122090686089241e-7'), sym('-1.3543499873077495027216420961331298781403871853142204298611981790501353670225294602075336451321325450925455359e-6'), sym('3.6856693802313655398473911727055155454344798217623378672237228465799404568743862592848234601414492313195084283e-7'), sym('2.7607665316393014853006995383791746974903747084732191884320193603653727064155388178061491623029158455177722009e-7'), sym('-1.5705387174733975960774921565812288382577856218323618500896071588840704918198299022047644552504489710008378524e-7'), sym('-3.0409841516738981668002519249337579062026748158088322377268391834084060694362528883173601071226255725847699732e-8'), sym('4.2967871208039327352240889968236411678008942613746164373547765013680161383942764062001678883722539083073538873e-8'), sym('-3.3088966378064091757102083538402938049033942656598725717030706205729749418400597198295798369053126472928038174e-9'), sym('-8.0506946616849741244533924317799972841824894645619309290615984043774066095799055980561602428132632220904968989e-9'), sym('2.4952963881898064150431246238332189178940209602998993853479409664391971149605049112313150566705658684223511725e-9'), sym('8.9176773254201486538064488400442725301095211503605054632089888918537277654530972259389402626919295747512925346e-10'), sym('-6.3173945903528763366966731384653588744600647523416877646478904417931209220486727745116555254731415800981773993e-10'), sym('2.5033468379512565761117905829095240205087425302233014335506298369962700611026698148790916540735493029133052485e-12'), sym('9.3680009351393757434248603571894432211198388729608026013382487472238841130635286403962318606282673464343314235e-11'), sym('-2.1812863930582308563005311232642555306883598235567845703512514888980479547452489619540966447545927291071746883e-11'), sym('-7.0206959593688185375208385689202592748931055566236051942847809042033820273123012107694959904889602507525341061e-12'), sym('4.1767260685557667606474479568738904741402755787047651476419071015950554654697836778524203783984514650840060378e-12'), sym('-1.9388895859390810224498704670305457175065649098655429303670453760108622524989685154111636423885684644752609867e-13'), sym('-3.6558647453685705196581225523661181104420285607027612436791518461375410477847711956402667471290945727439879854e-13'), sym('1.0007887071875107362587580139840428764415532775789661741443794812112009230871444735046785307601619319332663252e-13'), sym('6.1808994378494389649277069489074343998108579250129279304626735856606542722090872098961394301391376890860531864e-15'), sym('-8.3507867643614665639197234027124713865835487741956071267115293298099627740398574248605488143646043122353442375e-15'), sym('1.5076764521772319993493970949261279620611289704656395570734487111208286171285695240431940243661835698287901083e-15'), sym('8.9355655531056773686938377025749170615701625914228576420682398116432552180320305024877332860843301480502325454e-17'), sym('-8.3849857318031057093301380703229625389945676903403433241025632654922184724528427729309463343837069436561497988e-17'), sym('1.5720354929083500417097436398749755117814694682577698213582761398517113585309392208554878342518545601301646867e-17'), sym('-1.4045390695105720193515767555078636638399072303295071193849335896254599713544641322303741393856205250744847494e-18'), sym('5.2011491948815863231566720590448319791599493000311714894732448806010134892259778156036998155509813708564411348e-20')];
        case 42; a=[sym('2.493708487965943980442794283392911423644996133496268624244024959695045626141853549916597430399532779082536694816502e-7'), sym('6.893790895306591580541716693732159413506414141556702747788396783002468603374915517553594929982149620879373876885617e-6'), sym('0.00009000675653711326153500710637167086668438336765790179695609694808664334422019199227052658511851955988842261673647355'), sym('0.0007357239891184814273374242903688055552653594240709860395616366716438590661072917825259758902678371656670282537260186'), sym('0.004204850582043740430725586748312574391289951532966392521216665747289176624239018078960628353925500402637026972650731'), sym('0.01775268855086220091819702305141148402441482050538335418921744324642326685956051273171881609886301283133828556166275'), sym('0.05692885953601494046806422831539195375219635667589902470124197426596533670156072127126043801387555893799693415060668'), sym('0.1399595193268664031378816210956496292067035313215510394989351292097916927375283861583139697510238774967967937102803'), sym('0.2613719782992325346426660227432552373326970201921568342306246351835016324596243153729061889363643497814697934654849'), sym('0.3566453430815168903218274289715510797325393094741080632464360539894618336981698924413015992996908216383834930976908'), sym('0.3160786855177268722506607781385108239010539207065861103922619332541644320437645382858862794393192476404912509544765'), sym('0.09615359823381754971910504860865448426526298579085025204782479364553965341463107343545204621712976702208567477874434'), sym('-0.1617862156752923238168338451855958787127429153881836615790585402984363874187328123503525639479044233578307420640647'), sym('-0.2146016715186492862926067158040747956531699895473739782485740099489781105179940075991453718124245673825854579583426'), sym('-0.01318274653039765931815298023983225464811962112332918337983893208976525935839819399792735339492691755113086804815678'), sym('0.1724397269122436086786276964555222308232584905933722020424543928523966915616593050658874214677508526952499178030239'), sym('0.09100176325069933673582550834158978151113003498331910851710244433006345777330888395215738594465596608890672099065617'), sym('-0.1109823871168230684497318060747933952494352907949483193920921889365630580541026094944651805282868160144650924625772'), sym('-0.1094061529117982682920535701087305216844723473878523834989894949249528395154948945410485744652895683445325966485304'), sym('0.06749410772457848453731191047990890386355377403080210443171966721133521956313141256263397369907024240471735364244371'), sym('0.1026757382605599335705305222704419350552030933818831657155122666451963916488716262249346223329401940743803343641995'), sym('-0.04283960071196555970421619566213560180875011349601307885981465938744160128167528772311981184635035652459041716935713'), sym('-0.08728617887071177297364578050090392882065474087878829930994606524797350017013948877040597637552307970741990279471400'), sym('0.03066988821605275747046449412902953059469663464232116774389141147981386662035593884107217349712121991888324171294621'), sym('0.06977841904630079729563709488853501340432398766946201584085809180287195398734682416609270493981555668291581582122165'), sym('-0.02517030267474666594959043350951119182791957393067076274397190955558448719460923902935793470945167902253166573250989'), sym('-0.05267173245839261150459863241437784263269726155524079416379562123963264823274000207304136507185390238791269470711343'), sym('0.02234418635138444156472588027959253855233433714452330648225119623411225920979703526781847775225546920174184712637552'), sym('0.03717243911636509325530648487370819402896714079685011023972877699118217052145592658850877269639502089006827673413198'), sym('-0.01988456922101732521335020503431458108262229720073046429254974826744980648094443820919141066977820080008342757437958'), sym('-0.02408566262007011436021060936103510086095313662795617561878314373355537949828143507001574204670300930599562662282728'), sym('0.01682968407748130596560821734151470326546800514239421380971508389289500423563859855952731246083105126084624686441670'), sym('0.01393446271141812382600210333358487737192327369456231234428587936178650997231420383917336814682685114357093668862528'), sym('-0.01315980995140140457948748242162643899863307347043904160214810952084808518213980549982313474034854586031889336032394'), sym('-0.006850680715823695834679650198945044525083000913937849037144095602995159462163784735914950116216145979006369360158918'), sym('0.009351893519137899307471479865039935910855530429039369030240656473122192570377996102277217585341891767660524788798583'), sym('0.002528760573619281704791136658773017272867632484680419340518106674684652017694235935040888020839889973702413648672239'), sym('-0.005967507100368421532962195086172286507397798034814179994873468136703116285866876212342630745172797834345048020625896'), sym('-0.0003343657778130749372222566322492026491688290906819342662003218156127165418332791484064901441182023138887014635850147'), sym('0.003375932201832213646623942276957734270564978687201907231818991801309973850083791477526495729800336259947896148774389'), sym('-0.0004830329640100308305306750940280879049075588873657223261440540159348756488578104319830575651980823206039373260764229'), sym('-0.001662761166903769494571993619184556696769074039567567550172410277332130986943212394089956424631807819083482615959421'), sym('0.0005835976714882950422843969504644527660014682912244382838376492749718073703335225349395367129706565671659728130210626'), sym('0.0006905788480215810580985157520121890380562954400443299822423718199956849091620026124430675361189115154665000823394554'), sym('-0.0004210858695921390905688864244729890191101859775802926229600700272312401875283124536334750132260255870290914967963495'), sym('-0.0002250822978706740328444375307496685099914160065677826276981413665427050673977341265648674827165167389221661220946578'), sym('0.0002340675789076223980069384442670985302809526008718443118629225791343033693430348199775418223095299164690355944113991'), sym('0.00004460045797495748207909486001556421090180231081261380909110737280693402452416320781804803088241747839576433932220976'), sym('-0.0001057263847799025364868664615179283124895757625753915736591458165635404016268028360031310527147684374618518082916282'), sym('5.834929087274710155820340142489547117179481452352513215489868645606759878406268637515471439926171651531454383648364e-6'), sym('0.00003888385716429514338477463342444564502728809997014123943546425061715858234848522589179778663332629526081824878351872'), sym('-0.00001086513777240026957504824609783673503540498257149295878720832807566958742646975403262528191419855836919897418044990'), sym('-0.00001121291639155594424062657733079801875967436421597450469880785731334754462016963775297439868687498181918235468980716'), sym('6.208832211886574097815724261044621060900649289407850365875787415386782477035009128258379215203287271828880869519735e-6'), sym('2.217546188597065037605952875159981784275602279596235487587652515759853609166127185346890103845268588753039486629309e-6'), sym('-2.432906889026032709005621256387826410008565767603770113092156992566177013900091978559703427400364067702574870298122e-6'), sym('-1.019085191987042427063291817720200239422673049720010366399069735560763017365996268646077028166331997502497581481376e-7'), sym('7.145672241151515198027710854839295913189378577949799364604350621445059727734240288660871804577724161018172768454825e-7'), sym('-1.365538840398099047799040272459839508696166159758509885186641379982170955972194265315037684512691511313374580657766e-7'), sym('-1.535265590895723288100552794798474841812708668447174423189609797420214819273823610351751018156048974871167488685926e-7'), sym('6.902519371193580323510565242556875679212928429159593590808866013714620539450243069439844106972593707206301852174645e-8'), sym('2.008689496124459731361268176523566311021120932007575891179556940067224335599280164494278411343202358400012811749693e-8'), sym('-1.999521446960484513859301706657128224323958200123254618379124899400128198629586019367193104713723664259471365996733e-8'), sym('3.722105846278505990020262452690378823982720482700711487725225641844237414822013883751110602647650119687365617156805e-10'), sym('3.903442936993792160729362315641679227541172256265371199253656566361321920927407353788928009934073882357191774715214e-9'), sym('-9.692120646783296473855608899843297211930323974310320275155508568421611329574776475517891075198333334175200082373510e-10'), sym('-4.687600089607104311183601925738872579730978289996099255175765276578880603226807680901256620580264159169315238932095e-10'), sym('2.690634609551043984588231841979367017919294270518673982691772326601170944952868213520107845891174394343187676728236e-10'), sym('1.121235097758050867079919671225765017080132391320505990395960734994582937143256477257007205842967001418364801049752e-11'), sym('-4.166245023682476677088177366223124734532833608409613836609415950897149153683273833998620519054968505406399172469447e-11'), sym('8.027177007196013787453566600801411014244693396969468384300574489643502761082857714193110787341997298351918971086040e-12'), sym('3.368487357959621932195293387911354677231373602115551388484696552659595195977150977123141715384762828983666506222321e-12'), sym('-1.691536618186834228187004052111861883834712404325639260685468336724268696180988582167118264683504512916093660241507e-12'), sym('3.082518509439678885608338100954861584543091722966934904758894912005816428495213450039657234277163636337525528256365e-14'), sym('1.547319720686717192714496802615941908403340759806972872921616338686957036717052909465074727098231096843152625155293e-13'), sym('-3.774048213998317483905675301929849365741225821645373049631380370154908476659336800737761858407549202667800712161685e-14'), sym('-3.363391877304916105399299110409098701707582418319884873692632989747765909540701339115546716057438488905789809251183e-15'), sym('3.317400337788608015465203637770462339429608827907703658336664531894786169685722974841991459439655408485692979043611e-15'), sym('-5.487836347945486284333190103946194213928971799976612607655290300122730697953237249712146166973890831405206177909168e-16'), sym('-4.269845559261969875230432208935623847299460079440846705578712280447272743975844393188022561450831929457854706949357e-17'), sym('3.223787971013681806689034256528662517894637565285898537719318587960471414267969540747559602916989505944516593518692e-17'), sym('-5.795722139058432180440397944503734254380194106615546082461727828787376253014246693171381568320360183094834177089372e-18'), sym('5.034498350082350954118634417433154093176926122153421164829598637479599991465089517673828757651947709006760320769604e-19'), sym('-1.821141873740072635833660634372106125115935218736854527947441562469724090133959869839960248663068295277173276272507e-20')];
        case 43; a=[sym('1.75888203420890753912502307460211505844173276774422709254557828559341139298774899892149363226906509013053143244102236092e-7'), sym('4.97499510733858882435668740076966588440991322103236626959611505031480052411216395556241626360936857799121710810383563507e-6'), sym('0.0000665453930372666005075639067402510331747872217939745070334217519121124124095325980737625298023116261403468421512942472704'), sym('0.000558147692487414689858442977954312426635903933255088317570237535073099228611790205795389441795914178559478834125562652753'), sym('0.00327963492365580293260040246555798742807307039280205923745527121743471315193425899700697447376361690326378617350206738418'), sym('0.0142716831236399928284790260065922517693319076399519961875403565625674192338857952200455637142680268970434885601851101084'), sym('0.0473329027683513112271254130205864689244648574595090300749166386175393203229813334793659678602476018082162800858764814136'), sym('0.120949695697913993453098017768512398218574480568403962022728573581797514341806415752490613614892962859637547331063149868'), sym('0.236643299116712412921659751300571766837224179811732662148646566922997008710698489345518592199763298513230459689980038132'), sym('0.343479149028211772270115487795616685126262322117580223110932899440391056974709217265362916584840227032403579184071321156'), sym('0.337079320262176050241574401884030316186482493079914831442947250459827394058625393959191382733019823585306557340756169853'), sym('0.150349614394021370767010588522869893267720878272175481058408586326814879576048912941927530960179620691609971778268706843'), sym('-0.117380501171341085142846939294350982193206843393480117265130460777031605275743863378418043915661650117633328701709594216'), sym('-0.227811559464351543186124091147290434152727782221777130918962776237931744763740271628318996308680605428336486156186275183'), sym('-0.0683332117299297148465956550806305000076222392438194646005519596110388306497205623165611382408397792554670937666507205075'), sym('0.150198221921321351435347418577448827409392261507461217557397905966388974014677507429318237916120434457404925696055791711'), sym('0.132274412119264633770077156826782537336950300688421428704161210276530072763646156094248792375791066013540802635374441670'), sym('-0.0733003862030862961386057788588445489299393727206205027668628876462535122476573930327575163655106456365250228951694307032'), sym('-0.135409465414083213649589997569633799217305824977451581213258863458276347489308348669629816150582362571774694499815588287'), sym('0.0268914157755752516939696100204351279865101005185479565119110481010964760897321679466622644775246587606726875396701139137'), sym('0.118574675734056995282338034951043303943433437292016118751540657515699206724637060010022168308346876859259519307976613853'), sym('-0.00507034077397101453387768995370319808473716656392044570282998397790331367110728019548570903719888663735366420807445984461'), sym('-0.0981116816776768673047985674948974921445315776612055347525181083598631107055599560249003806658611046698928743235932337757'), sym('-0.00192957425490181956730166886628055618740335392559579228567540348317674007918582324929183804990000574963588989563599376022'), sym('0.0788283797978159729284601904080712479376819389250982581630937081450492608181876229627796971652317547243464339985681002413'), sym('0.00136360343938396122454966153180122785113114948298880595695381327669272909461109730632985373397870481286822070019433450827'), sym('-0.0615310883008140498568782506764556209299914831355611649526523570096092036669277419168786552843362896607342349512446166170'), sym('0.00209870871881544710094118746764995112313467723915067517301336274127415199978704779306361434374904660390270059304270623514'), sym('0.0461451048953102133829299480737289685635608230452039962535128537933035875832654150954260563697495443055022323281529385051'), sym('-0.00565545734891780656860300709960664756359185088934275033201903922076671180609893676804948614151778571370258186702548605475'), sym('-0.0327195808236644027657784939026081105733421203674809899806876952592825061546734960761478839954060279807772098247709812441'), sym('0.00787388225173850977571951040204352447352179499571803431868900389821792815370911372251735107529809815947985122708814925514'), sym('0.0215300919231718493408666079814253989583303277997610094650175649238671905682257041751159886360329053482286213110289853531'), sym('-0.00835499275736542254336531785684539256558839097508740637472062115446272106307247796906053026955514685591069202076701233445'), sym('-0.0128576036973207288579810564537205477381862226891797305519980945167780683880982823118916228674596416284805264168611252485'), sym('0.00740365035430611057810400968146366382614380950590522295126114851980584788157309234285671502223952470721442337797524046825'), sym('0.00675305667735345123483655543909812657190358925607174388103672197527517274815280644929012649796870656089821436835709991055'), sym('-0.00566412027585508266523233766349497026204066760817124889965309115602335113614364496209601996646473036456243207534769018159'), sym('-0.00294339817973605155642156417283356218080521792043567746924715024632803799539176103426894174744623620452520918211667771005'), sym('0.00378694746813734952346628388706639198010654837494964864126932148008857631341397311487274366449960950246165675770247489825'), sym('0.000904037022059708698762156116001925996777763753434489318139770661476955651108889790904373941909760343343557582660064087820'), sym('-0.00221497568615695039940471097382886720020612802869319727849303227202217949876633236811849335000591822419570969401583043583'), sym('-0.0000264043334572671615720964484493357286058578588460585406954117500775813991720458324986624609007658049713802050425849434083'), sym('0.00112438698603184719004655176595338630799772571012624192568532312408700697973313895729784675189323035898852701793985824686'), sym('-0.000220818007662950852473114523004357857361841184043399549524084721040718741873258171364008271762740411367341027048516400939'), sym('-0.000485586455834840093108212593637273132867959949577474816010374380861295091417269980646929090689278365375543951539182280951'), sym('0.000204722154852524125950374467592395149654894114313406992219413916638998346820853881887172343901222332840822605522632661746'), sym('0.000170557131580508426922043064683794004497601756660540991669445832172033616107771868507728985295741220620704922841451042984'), sym('-0.000124611747658052228650647756080747786781658044742428443230371763826658056333550540220104700067529078603789622513061467864'), sym('-0.0000428977955424445397744400832254222813998940642121786273830748251694290080065945198291844342836259902262001984449594967190'), sym('0.0000590344586307622593452371361435775865786614479029009197288248343354416964897152868162496530680226177949698218617526803043'), sym('3.26416119386961743604097534551700828093982173071944209801737539379881307099976844198609293233382708263101275322779617106e-6'), sym('-0.0000225065469919237892637727215554145756821574491742463224505943853862973084142368165066048998624715502113727643451862057335'), sym('3.91699747658163038969491389483771013758803101447805793530673015072014268575234868378438002754354831428080469888696506845e-6'), sym('6.79125859729835712234610430404020816597354810468700560660993409873166269105984606171554930103914269265525776820401510406e-6'), sym('-2.82773033685671746618985878598591034941947551810309611709203506159586581725927816115678451621194438200723462388994443136e-6'), sym('-1.49043780807607674902859843420845083112782598264678446470280954552810461411226143846771167187842706880307844912922175722e-6'), sym('1.19330777452550777123328240202916466828893813684555162261485497209505044085832623618514921277141012292176946424406617416e-6'), sym('1.56646218706885789421325811181779481029548547065518876183978469890741586861417454378829084577275509424714256168819361275e-7'), sym('-3.65451231311552545655583731918047915369210542799887410072161404207281050024188308670874077624560351441158523241218473522e-7'), sym('4.27385037817458881106038399593675298806667518452211941701481423746948911845416718399785949698714630612079809665175398939e-8'), sym('8.21255154474562380031925026321537725336803370379092481330462027242097393673402679767916975324649936575469123197323897674e-8'), sym('-2.91697013450311008132184834535321465462161280613397141031734257321390478736127930358998453000408364882545513150634393638e-8'), sym('-1.20052963039428972086595641465834355278990565566529118893686877756031595983403578309265035319623997515177087309639484415e-8'), sym('9.08496995943924916784735748521214903735661825055838627001179261487207328710588667441220606801596315848232566130000895306e-9'), sym('3.70374293992769151185555818352070745009983053069220387906975695756281787985158571162528697465732502528351253751218281578e-10'), sym('-1.84960779642910221039538227770361225997727211950697730235763424221376470999797512455053517659067874689445092592619124508e-9'), sym('3.58014018263154398438274650896691305873804425065409857330918124926647564130758947022488898629036754872155530239960898114e-10'), sym('2.36864403325707450635383737327202376533406539562486004339582494990057085341992046259736625840276027211301547399545136144e-10'), sym('-1.12463851115000569162366642214480586927886601767390403839863025521309920882456085338467342977003964857979721231095338088e-10'), sym('-1.01242522184885579650004547894929014197292719683456108985828625920553113640401682704326123620844745836396977286267850808e-11'), sym('1.82422259622820177541177908370787898381017193404357400920410579106842663465654477361904563350144799762935148737700961453e-11'), sym('-2.84293051365778468384626919351810338745740444929276147256703665082601240234082684039575488911565947858487860884286473810e-12'), sym('-1.57157316309043115522947128734323937647577742818717159247981401247937029631529883203403161506326181057221503576160886833e-12'), sym('6.77091311167791202567464299442405243823290210480886190706119400425038105784460181287878464113297866330206461797848654541e-13'), sym('7.14565109980411325268196724167993226364065388510100867336014315056285368712592567334690898470766309198328766200342537595e-15'), sym('-6.47729394015594437885680386909655386693575821410567716014035458002962983847087218886214806807440438977707197180109571603e-14'), sym('1.40774998092623631406354213957269652151058660455048300206631149575390092851526310266398878844143532005902951280530142524e-14'), sym('1.67854270012161206253947152495865857238557107311757648924958886722769769408059592879154292491084337330065735937433429144e-15'), sym('-1.31000250754046864576838220472189374497208693113217833919367519948172756360553273151952200818461105406131747178954969865e-15'), sym('1.98314859612812764825660766363173754704553392694398886573713287584779025458975055145490069835743967999305156110793181967e-16'), sym('1.94606734101399438916202567537902073128360159804305477181003849455445641096454037856091076966647896865049892742624084456e-17'), sym('-1.23585170581636543822773736110235769213576455811329210215625911271268601200304428508150653512709035862225816383953731500e-17'), sym('2.13445153391861381631276073862744159273482359833993103948749238876669134645514914452024537713504405445569352827626162129e-18'), sym('-1.80404560060962881072970433876595275853128795673066251922285313582999536793756040551464665582026788456544342138107159105e-19'), sym('6.37810355054473590503076968589088130351718108306160436974449293547278560696633132906765508858081327480336136572578753426e-21')];
        case 44; a=[sym('1.2406549433812827743830863795757106208061710924633062761912492991753035116725927891160529286058241377862723940699556110902491e-7'), sym('3.5886133796162217915303596172911407071268494092733238958835344482078576070517357172037620454641609734387996653770861029966762e-6'), sym('0.000049148609605313751634648836519440083852014359166182449144226721732273705396695407625424954892090602359069916671911501798814492'), sym('0.00042271840797177749408677558374251167937838818613914693917862360216949256072952020478893244654833383592633939563590877429766394'), sym('0.0025517581037239442779768023515887440781291487138045280139625658009176150988205854623587161607851096793963245621848481764542087'), sym('0.011434814023728105552608161158742504065651952900802043359677499577851148847092436726216558946308549901309901245971561218959392'), sym('0.039177285698392510196608892763275108077968306775633285036373533697746331958524111509154352524690913512549563838997250545156701'), sym('0.10388689007774933476143296881718205443493047753687358686055331033490355252550839619445565730472378618147448147177980648840369'), sym('0.21243375600508617625859384491858671067136804012201863608196559409166905693222094601495708105897390929673293460072718736325219'), sym('0.32647244495336128658342455244231320760487177370910053005609857092898424472445509093034060457054501131531940629181138296296580'), sym('0.35006883551000681109273956230605418187197110089462360088351615032636382646934828803118996065229240510118356280233262019421271'), sym('0.19945888957303033117953673991201880299454179141376716130367419792364065988842735928550632930337097975107930772972432188231314'), sym('-0.066657241895617838719496115649264244333460885924703341193681985137815794001296871705502525907401212848802816633678418464371198'), sym('-0.22715048957593043061891241368620247447177154228485941259449994514911716349186688916383356629609718904817348238308092871652285'), sym('-0.11938414242514492866816520344426906543365151417180478919931072666331767320403990547661150987985862121882554606217816671837720'), sym('0.11480990555386067638630251416657893822363965288462936931313547969117271823260341201345353136418704001032574767385477922569461'), sym('0.16192616359163976936390488231613217208233702462594029865889364235090734953396557246328542533891052437426267403041627268467231'), sym('-0.026768963235383381261102579158049095810591440177725509045271919342897084267007321038719018941657533884224685167555862096063330'), sym('-0.14662669423116071957553720823437640012173013320423051129111455373176597841130869298682760150446195918375955209029906433634400'), sym('-0.018766158459903118887909134895313177735256580449966473748627174681838647948315054147540527572942055615466743357011962333548072'), sym('0.11924741911122106295601655251282525214774726731720053602665806124818936676164620548577326763416527002239021108546111623128291'), sym('0.035607843395001197033585300620085839953883916116060423927977630635605393538493614445937368253483830323651306510526631168874764'), sym('-0.094649495800941009812213910522494018895415505081514725320358086738244541618064292634724429446639712702321993058110769834948379'), sym('-0.036781514309800743692864107830657085408360321768972773229947974854058240133038478956813003147315497587364976361534106754987057'), sym('0.075251694213515638426314571543684304586037801910767955993522626711953108778391898468214898684680187782308051460273573245249671'), sym('0.030437088768507773147267023556986225356773114013990927529701758383872640601421393418559678782935570831429296712456708129621427'), sym('-0.059876006004720245882738663763561539162114191122477371122028851244651194845848917849144817436539597152622167560375582341121780'), sym('-0.021333016176253296867521078851975596711713877462634652317551949234888640354572520847100709265967666061881371132367875889528152'), sym('0.046991339597180449890518168094007283113568386372680754365135445866121514625738560987803444890020938874838086689415556578747321'), sym('0.012287460932791394978274888065695475604518718712782278605665795697377918560021949578786229697287876259694772995202771313240067'), sym('-0.035672005131785450681827479584892959320676481242372143262110757728396870818877705186580478720253707487635701635666565413263006'), sym('-0.0049132588636539963843237374188951183747155450279556563388223866039705256567297248755429713315302617319469731393821151331321042'), sym('0.025679136512589333022977678517947987680670388332350762836043723485381479806900659700013708562006991835440326958430002368677902'), sym('-0.000080219312184017809341093808736486916345763406988946632663359121446582240608294960374877318879703406834019872061197403858116603'), sym('-0.017202719162684136114909658849665817657983614075776227103216867806624884150190004777478382199554909722148899226722904047222526'), sym('0.0027044475068820190010069356034871920819981109127445999436439065202559402459035353503348877918414638532207796492809480279794098'), sym('0.010521002834275421919486444351361666052469474789135679958679766300949261501643922983029358592146985836249565037451327989888210'), sym('-0.0034654708206993647702807306205642544547840781665834279091585169459801679231497904316662244080691129083636963593959687053746640'), sym('-0.0057398882594180161481979665395021471544326095533825744394805941022398197719710530348598200630559473548044688705789482537778360'), sym('0.0030826157053108463017969027931804084822475475568508386905325255103997361154776874519214614823001081489482625268193698186036680'), sym('0.0026952425535339229504105518882528083766362835781781370246480428122457271410846482324844508135467148216436686527404813324971377'), sym('-0.0022278883176492421165813698621831399409299702870859693849131069459691035340801687762197060701145029684321570170202560363229679'), sym('-0.0010100344445678729221422418554616209085505381890130604416983011568112099936541575899623331619340394386051685264756537122952002'), sym('0.0013654425985671546171507054421625206177540733488771391023913258360090972807619062377461868560562753469296068382411345938198172'), sym('0.00023120243415859889966608642604236576008215044707442032052052621791576855389550598395779998344465898518970289324977728447925553'), sym('-0.00071710529132781799459143123589954226312533768690217920226336901551754241484304433752869933873744406762423288224334162227024418'), sym('0.000041513739973895576695642680085405367506042156454690000765636217113588902016308325321605416157862768322179320522359093793044537'), sym('0.00032035746013495407931411809290061341049032734987206142577230344525177425350189736095922097623713161320941741297587743267051418'), sym('-0.000087554110780098429174843680529591715396331978792884796270301968366974280981223350796249522567758828998662548383130118649287810'), sym('-0.00011839386222421292027564512886085481215805222442282112401027432948420191852443660540090109918875366235951208288760079269621683'), sym('0.000062300399944148711662305773530712655290345924802686665510123015389864017575347189540670212011153759160922470830201742102309368'), sym('0.000033570752685005131861588870656610727268808192226909494355097916463935745326706851159101496574661961339570667943650000257374610'), sym('-0.000031520557154718552069757511838985646773413053738266684442150301005075363969302132529928551161582092179291250868655376382543779'), sym('-5.4197557233132880800904145924780251782173310491355901470939474339884077889358552896377786451110633821964191528861062428948978e-6'), sym('0.000012511652499181906241556301889094391947649368126098321966299716928269402432650283358972303982244179400796412535859374492714929'), sym('-9.6331863301723620879806752916186911406600647028293040780013816101116766664634318372794024864663580705518726803066530938566172e-7'), sym('-3.9282198911419557863796076078732068287205660884767741020888900133618734840772703688088986862260920037819816718611309525632013e-6'), sym('1.2021501180025929549418271195298765734712557060258746890111040856357772198110180025708315579427315269729445796324928233322780e-6'), sym('9.2577830362701110795163851898129336590969585075270103569189012218781568076344864295444470750282288148015961395083446907400979e-7'), sym('-5.6439205499514097246706506307684216276050173453146821783881396992458769052938091836442750728488945950019935416003620044439654e-7'), sym('-1.3028821678844123566692952817437248736491644571814090711514961852683390277404273199182558058945397187064977859093026690199704e-7'), sym('1.8150919587541806415629263015123497814509801148088297129201550312529886726147208088330287221691659887132934004366151443408251e-7'), sym('-8.3475485277433259397870115065123210580964834755539494043332587943284879518659649126252839921832252137178159915534061559378085e-9'), sym('-4.2520678021224811949177462272974836947866633815156626624454336630747081660266421995465024688149072831421775845462642110907650e-8'), sym('1.1747903309351123505171071638951498756483247757500838307995451308335061357540402570487150519697280828827118832284810143266130e-8'), sym('6.7310781790209180585192346401534754569690064954414471676360652037428898832840987630891868534722768864102709026113506445226131e-9'), sym('-4.0288503162814818993831959132870242843148245914726157728104061626097547189529462556567906148850775684193882978619720206162228e-9'), sym('-4.1713397251363257036926209058571625228851331454570206843057945266393989266889243468220539152383966187386980317537657927977594e-10'), sym('8.5816825681968748810663215135771765761553014910407321147690717683295385493068698901831338998309273418988161802172196852754779e-10'), sym('-1.2304197769304402206563243677176295148404031327408641546659026852364307582991611126671095658835260836212682598872620778331798e-10'), sym('-1.1603637417131438591930045949745290699441291345984973221045372031041081887907034865255732439539958175656286108350169829154008e-10'), sym('4.6092128711909622713615579296961774629711466073310209688178773002637891379902795607245729024685873575318140926913967502418654e-11'), sym('6.5892800914761141530146528000104726102519576890763371107299853842882697599166624861692014677632654938141898897459930251890578e-12'), sym('-7.8720723874131522811313631369859167826684037766225149171447900021644347938530338873514232228215283666445111623026477712787801e-12'), sym('9.5504805204016821503179218811836085221838795183235188401800100476050683273767778667633860072413716255890058522526063322819034e-13'), sym('7.1687162006677300742234225053238661760728426347037242900348015793220556203238103202052647119552243497744720660119405064940539e-13'), sym('-2.6783849837952137324039209008383531789506014011568301322474337441447496554001503017906025605575533387143639197711453659377469e-13'), sym('-1.0852861262352516302506317909471679939532342382388537843457235942863706915524351604058250338217504370085982558654779319774515e-14'), sym('2.6844817072222281766212818093795472688683832872758015290164144622076307925998700017472601938583062831006810221985278631839282e-14'), sym('-5.1892007876889682967156668433052378269635622201588574537857869105653331535207137747441841454134225420719076973022993384563820e-15'), sym('-7.9455607089299634493267898590693822153487634828237806472427696216661942943223538509206030946222363297051741064909200418872899e-16'), sym('5.1441079797520128612267445992401214939851893684138173350123196011306192211868276864972017010730978816890419708843967190493202e-16'), sym('-7.1107713102750039794674415375949541053534206204407166598093652314289596687539818501438775561232108478746394063803086444946502e-17'), sym('-8.5867258202828500773229857989773170565945562109807888371907498141144862584949607261107934744536254692900491875830391679861911e-18'), sym('4.7249208456929042980252168907930453489671128703848882473059689811804978074920857822665360247319917508451955241562638916083762e-18'), sym('-7.8527571855093294790150713723045926337835906522358977976687514778705614035821282830673250661746799488407217843751514925465994e-19'), sym('6.4626913232764514667119320863818182091032347307585256617982539822507819054589419341464589790878344858925660148759144768796013e-20'), sym('-2.2342807902666075508816039355680067840206477999163542256498333430955114233524219638690263434256637817236220572271346968233996e-21')];
        case 45; a=[sym('8.751602602962380458005513815585493947939268738021620551366730084146001388345893129668848056562266488093554127015294227219943557036e-8'), sym('2.587432570794475397249438988274479090112907245361007550061120109323027045903770155108813627127424636339527932979763477052324080855e-6'), sym('0.00003626396411537474686241041577939420347968726399330767139488376158323397398041441649932386523622105794797316925715331481692708227577'), sym('0.0003196357761733893425259162041294492139289821055689287293839264771391739455577892803788088820721516927402762923024532367936673896189'), sym('0.001980817999640230291513700297132355782738564975041707498592938645585689134474171368405014423720474387924861390570771754284427818771'), sym('0.009132749555790763792291257175716604307176160672782583974822799550425123707088781033154635728018439570146276259504819831431841996706'), sym('0.03228913898037235236254224357247894053031353099340961465261470675958748301656422118049735895186589481637193445397775337655539275793'), sym('0.08872340767908962289075354451409598970837870366075847951285122134827010050497795868412402727708255504701614620952381506218550709551'), sym('0.1892036050374636714734930408981822417742299444118064871274182697567429357877742101158261482945838907250987621565243829623733753310'), sym('0.3066642768701906033764568415269180375319359582051900725585535353936305470285310041141876350877618691105359093464057914915134770748'), sym('0.3556736117038743164266249952963367654152521573702825531908117927473836165862765613498728426420212163041603457938206572796804112618'), sym('0.2421927672888682742603727732668714671089018471493094609242589184369964100826937524786475156320338879033151696158541496234846209022'), sym('-0.01232073243778405423180710716259285091798769087698242214491150060248701207448649802249500589214440681802633430773565688298514496483'), sym('-0.2132440474488970103459227250449265241704983201521423909881698090101788231321248436290403951872183411422434592048337274724052211998'), sym('-0.1628793789215074777887361001173748700520817731405927269597806200982143811225854655690153531340542634324213546681241155241013912307'), sym('0.06948277513417423553778685413026380505300179970001623337366887964485486271639962728350430612772931505266831824898414514378758573201'), sym('0.1773988634669774251395400596512474839746182407087923704279899196103759274054200427179766079703548657468155132313562163279145095623'), sym('0.02372652169992801984834044189277817735979102407037787273952250921691983019607972633972064526125470483096894903411071913511866128671'), sym('-0.1417939810069513327092931723457304003390201161049476388317384230097779550659610565325102509139367394278058331651287144064367200242'), sym('-0.06386730041377054323950072240162586372936048535755108387973889106599083144723888029294540903927292475074650570192620141462665679710'), sym('0.1043539237020180200081298052305484998778312509416385405547971366228603234939476097778033224236922045676318388036535540062542486167'), sym('0.07344944814844690332183814623303925869795642703295507813358880317532338744480549610765556270640194035508064809914022248447569024332'), sym('-0.07690631623467092754381175751475601174595552905468383476291200279937159354479756751939322623118421593084340209858870619056433350337'), sym('-0.06835463158434139143369639971998939567015804584610760397460870653587318616270931125082715670986677283546917895884638263264471246573'), sym('0.05889960289606798129485509519712232826865145185796335987070381704957879014340861927474617611127591188390149822165140606367556147740'), sym('0.05699712184524710143230866201501111698403714396593544231956547703571391218690999607386318551734422664841227218052473692827942384581'), sym('-0.04709195755322595724702130599071782735954958189486052322290873461794699464657148520031103612850235561258552963326569142873669632328'), sym('-0.04362256613380879571637636212267046230747395945825091546621942289969404123239715457933021505702655518260513247292619115088524011033'), sym('0.03855012400308890930323967877793408186485829467409376085208004544493264645401403277321740624036833709887008083291995031849921992121'), sym('0.03054706821368075651527486569797408801604696097751172987485542251477031590004327400891156081143885072626098588993008332464896690493'), sym('-0.03136324273519804525243916146404830923151441759712891548406231436503667703992663362171650560998097461275789796650554587682192882489'), sym('-0.01917055569000526510909956260517917729476849632273758424043007873519996639639633241368892802307533549312712339486059429576823835083'), sym('0.02464416538596342093916789015872635981489344714574524173530994696916893759817815833031786416089238966446078072748350491616054849019'), sym('0.01029449780979150008875436299194769828253919022306989734074245794312641031692322001643334094310681531909631795388017563447851392359'), sym('-0.01827671041304637636543728777531067345034214910618259659805920060070583136709761158109098624291700131308358239513046064720330642116'), sym('-0.004177977883764981581761055444766543814091958936098872984808477667921236493220839933539294305582455560359444422364740037108737797889'), sym('0.01256199466402463915022617056645591241045831303865498708094739882206843989540766753990094476691299592680356592265567811270083052635'), sym('0.0005900559325884733721933992240107829928577666428465101066776434294074171441821569029616078433252904164287720836169240483005335101110'), sym('-0.007874081289737594511758440835302023487346445908296618221622039517345133028193331971474944721233901211409273769742719261426527251394'), sym('0.001044371578139017612454881089760966557839064396885299842944543330820461967379938410815202242617341434404427860443442091896960973962'), sym('0.004422832351491135074493609337879861312848066811050215887001425593574758714808308193604944772584026876684711170365839119233204444202'), sym('-0.001429802678804008286674786438390182118391865413213151482687162037989003328490524076370817498811688869528642981542810646636190167240'), sym('-0.002172235550906637114731119045601362949638629618832378973979511415617785925610866733485679470860425779183233539089222358201323185944'), sym('0.001198143945484953538244284083633306243233681272629429733714603834648770183633870906658856255082946522317495128036182821764519885673'), sym('0.0008922194731182864399053954908121542480080443461328423438925393177908435321989591898147064233246666937791453505820845938493038632309'), sym('-0.0007893783938720890247878692383276666368303931346976982677488773297950611777096363371333963735934588632854370635886791457030155284688'), sym('-0.0002736494752191566367973773167164751958650190983942565318668428205443905019599219714119062824946786535565674849244141564944568447109'), sym('0.0004330718459079262433485960840041217789625057523542173740961561016839804122552604781355499604491657056329565520390355135277624341329'), sym('0.00003359615342086449990405237313272927364063033746799020536583786308981191849836904457890978204069112870952390541863892389339979779070'), sym('-0.0002002480437572049805432677538701142490619449905557252381951780506012815327677264126712720346541371716483706631747909281570449677930'), sym('0.00002911680041027554358266357936004164831075117375543891458686991416525910322671173326005327590588232259936942359251847760030212591236'), sym('0.00007704424574084579823788697918363271609998309588181475467750974734874147708312536696155444090325144425458920380678369711303633974533'), sym('-0.00002876200955205273122918999616814917495279874828926108007907935293726313043297678697916547465194690805771046401797727698099199112811'), sym('-0.00002353328427179710477444243317636959798035914601963601196789195903657584192612773481544966427601126055136224674510153530133006150872'), sym('0.00001605253630479745198292161382224652465870510709159834743225063787623153394638015668047583419828981009693408050691903383753287629853'), sym('4.890944539345561612251887108947858336696002558113906833656704865979070838426736824142823382069846028803532224210353985986909112650e-6'), sym('-6.694212890342093141852376023471430583654667104109526479736602912107463163415199011939723872768083552378146260426843658273371967731e-6'), sym('-1.192866589347986864228215143717570373317586599089180456787553809865848006689437193888543424212704535472157369318386061625611955494e-7'), sym('2.184480704781237112362835106141502106968950747452238981513159573875707545982552745663035267388035053264050901302142354100488580498e-6'), sym('-4.620309549173573681379825245921556171260205431618033523642882551220256167899958879309363156432054759938990648817175080166427290530e-7'), sym('-5.435960603915735993845102945144029848781896674016838404185798737515081680671000943511996414799309437075363614623729005354162051430e-7'), sym('2.564352924964581403059987079895982724307558963130305743384465059842598353487853494442101261140658528344460742695773314228111267818e-7'), sym('8.954061272163292958383065697738138529949783151204260916375486621841473819086027521378817338182636883450398074012795720422836699222e-8'), sym('-8.762163233866768714145473516973804545741473332002764192894023305393334778900075101989157701032256569600443608868431409488924549114e-8'), sym('-2.187047309057976685069541727080410077684396910481673078847633419032012116429654288686716652384682812638383030935472444006471040559e-9'), sym('2.139405481219044846301508894660041546021918921283796988300199857186703768611503531892319636401132115794659307888461133026687787796e-8'), sym('-4.434731063645350453374144841213312803938563930757977641408878568237837205402194277351663223096509246369355456427527463740890485699e-9'), sym('-3.603945394018843825035806278231405770887941018718112588740798780428143419910733372237255358166209969149796064726773111014150987887e-9'), sym('1.741766687481996826468156450664927731388912761344294401224916366551763155290870353901808124311133034812193324299910449684636530764e-9'), sym('3.007805657823139936189438928905185716426663809453892146958579913381260349330016856058905459457199018579531242281449492302489572357e-10'), sym('-3.903882378853546081074044920662060465999405319509955519913221827626390381939611073461137747810227462777752829517192021331201969175e-10'), sym('3.745894768798310014159592264389239091594698900554270692513806211748355268386582509908318525110774713633706978598718403918707562487e-11'), sym('5.541150727817204682677832721163054026831574854186326625649215273825360037662398043567571367669812474498685401422701387658235173667e-11'), sym('-1.849342859632824442597187799449336534780809241625029035234555605392836636625176431715923766072127318883557990725132617866520459285e-11'), sym('-3.754793782917193600532055585306985409062751120822522731082933129886179036497039621115665292025614599336328900548601766315795922535e-12'), sym('3.350380196434251043943829707809916223141134306127063657569256626803331702691948049876707612599083608066148120993534519636886181990e-12'), sym('-2.957060336533262245392192097488813029770093484225558624154878282347523939764838202039510818145148020188209019253341287470207100113e-13'), sym('-3.208972730100230273611166694761209017395467401471654704411404637854190890952641101630866396755071265842043662704432633203457165810e-13'), sym('1.046661646976448360638763585685881857791610824653411534534965303228036840658491890556372226269378999924023929445784376419814960911e-13'), sym('7.577275876684760567738266444159555427898149112306797555242497255901753092715461918948928368578138380682650922473549546911771204105e-15'), sym('-1.102358334888278014272035326242842511243215652026575580192835012203591193234620871280446063346588294973395432754744282434446291525e-14'), sym('1.887877132987056141216183125537845263401344461933704540462586604484518378167018675487543659398466160412299268860891802898320888814e-15'), sym('3.627702833254459190957145313850362898586310138635028831173095527397594987105384744708620741133275297789833305411373090606076030955e-16'), sym('-2.009307950485543329156733135531366731357298731910912403893425384619259768575082660054179904106935026903117608902477573656033712490e-16'), sym('2.527839963948661134542306570142765334863457993426383479408549928601878632857578584137880154841143859401554179476690915916262967602e-17'), sym('3.699924155864332344328953904314581310550173056336332517910259909617777076110685097951610656965136055725581388190615043309215187474e-18'), sym('-1.801910228493697821069154336231110253109901822708452817693721639979778725310383045997489907166089588831662021712370595913437879175e-18'), sym('2.886270320009962487414562647590755986302828112987897656163546703183146480189460505750964517776414022066410639199380719043251296908e-19'), sym('-2.314510814074195072072253170801073614439891281132757763640621400781487672131480713887246441011793858084290648821156681895271510910e-20'), sym('7.828485694147679570019101604080818800562630606324334449520634721778568964259928773765910281627471459744160527951608427845849103606e-22')];
        case 46; a=[sym('6.17370001992379395254715715395029347030809386250786791740617650792293200014050656789460598896907561220469009672345771458431128831246702351e-8'), sym('1.86478203989384215936476247838397299228037338554372535352843480765597123811066782199252662613043991531626604119081198975982339856481412035e-6'), sym('0.0000267318607076937545849508823416697284853760455382426825586775425311395146690741415857879114005404042268725037743749999859402827428591074864'), sym('0.000241320276760144376592219921760187756298733167582798644138009718540837808958762623204331690640896571023337629582963839010642451609014287917'), sym('0.00153422466142899768962249374373693744033215394501767326934286605276967579736192790554102749435286129847763794464581648581028233303268091802'), sym('0.00727214197634655413996011348910969123186848471551835220069674747244880478928799580100792056996555541363681459575667215845909944110936692404'), sym('0.0265051610982337018969712936955670203949763163179479054415085199425115468153140451576188116197387238804014617953569764390500265797036335979'), sym('0.0753682557982036563750861528457021248774118462681076851061942006358088350585990814502455722122713471564572327210570576811824262473480693928'), sym('0.167285950167145965590139617654974395068306772314905667482482146190106972645307134833585723712491219779142747111646703332891404829229183421'), sym('0.284995394867177995561814393344665428098277891546744730396545496418211761464105575416251498837585581393467354768645361626176900747261068528'), sym('0.354705522553329311579900784497527309467736520896401845392634825471632544067610609374751722289269184103514941391022312021892446612361382273'), sym('0.277734156908255357189807642935134261873044510272194119032803244031962648129043555394416493951709817274090046188569004771935420556023780796'), sym('0.0430681236174622861122468451633757873318113152003160957646360029216423844519660907698525632659173657672919273860973457618286204962744309118'), sym('-0.187450074658067546036817887602413204039852171789796891973510932056286186380557374613377453293731517054886714440337163336165108664074753268'), sym('-0.196174153289400917850081052025491557117156924853244307664570014074488306400939900115489510142086458875675288420683935811165651497248308051'), sym('0.0180139365461853733313658338198384018289613064983473986786464183223318977177764583140484827075436479902520828362658101775453440443483544076'), sym('0.177580423434703698024211333260878614734864796606776599846495456429361539631168883239584512448337920268359678404725318068045581078415220936'), sym('0.0731211984827950817989816271040775060230318484182844527968163950830095264375117655322758242160999964104192207836786252591740081311624111308'), sym('-0.121500147051785511352113706015921556964085420667203289099391949859498041177079208995299261197304601157489958172735880394533869512937186825'), sym('-0.103017713202583921489868128762116418499782659531263885182482406501753001573097659794148507052546066161729959389803773749146379069030094104'), sym('0.0756238663516504035554917833049537936240682755895372204021702603914509165197819859642266148651238461206140814864655354102761114168439287242'), sym('0.103147626412899666704237077655338098698130782655880396721209870890378369998191770330334879682179581440023871672180150752573463197138006661'), sym('-0.0470597470787247686676566093916305259246044127490769294601729328119782795066336271212183416616002299344494968568120675525560323532300070082'), sym('-0.0915562054550568472050662828802460712401081230316256837181793763188695182176675886631954189635701082469815172764388861519841138043022268267'), sym('0.0318044810294478409545918119763887567795987722854969517131391274148508199729935131678402988595661261472888337955810209312224729132731901806'), sym('0.0762323532328062085634522819339275218170664311897639101351941192154202344735068346001715910643437408040889498334272201954059784069920152648'), sym('-0.0246456417179894841341343162013397792701818653764481325055518552163294718191423694428541918268662139267753824941894897553457802254424634813'), sym('-0.0603619012890023661699411222909205416667299270408795805475915624743397877782163082306173748605865122614974332396429990193206542252562102139'), sym('0.0215051600447182696587682221842160547502352703841051596965412197576031168695252716105936947301325956377949294344059056389737873931848437368'), sym('0.0453022133801777555120340687840548316115852300569895239511055648216408994762241589648507756510389197296030789631823587375311761897774006233'), sym('-0.0196879865279965148645282585661250706617952071820486802708534949920257304465770273158380995338577799610061978805816385105271094942427139938'), sym('-0.0318476718889161636503702785926984337163215664759278422564840191742873816321697654817594762136831210102383177317361293101787788823185293057'), sym('0.0177066888405650242877466401483166357077578343957390562127967112502298982426467836777674814187194332779029211171878813379767885078133573999'), sym('0.0205938911447019820753225923736033145512070762232253232391748961953630622415069771453028283249038966728878340157797290180936710449133633033'), sym('-0.0150317418663316648960414070804657754706134876893921198153830971975029690874612656441934179653829157433962231210748907006031913394383477251'), sym('-0.0119166982079890626397061740046916315619524004294759235190463603249045057858526515402364885007506255019282595694176865156123259522442627659'), sym('0.0117920215344544341232252572179020869159621513186271478112521429138473777557847961635490550579867700326064369132223090955632089255575006000'), sym('0.00587037656046574793938100307871773835309011216851862505981629198302196365619297018497845558886951646500011492650947513052348144636450221263'), sym('-0.00844218495943381775687992164791630432942471404548309272255262260751987125201251759152746429025192712549664297641896718088115584181103617813'), sym('-0.00216450531634533906232433453172443760927697114516200500407181083512548275211092080177616950227075118721724381166348918114694104676520397423'), sym('0.00546145893024774266134319379880863097436847848871957600117329068884265813809038574298906349438069016120408603469522839399460043365893748225'), sym('0.000259126807616709918962929690561408734450655997350898346751903191064035292032504155949151098481259389774801305202479934432651147191146570592'), sym('-0.00315710069993099054918950815938940249677860341365514482981889311608428199291933945051054875662174685018398001121701128712158039251031730729'), sym('0.000468915873961465309867777215767333744352975684660276927753726822605910602171044193747145706269309065817764927814942413469476474177550716923'), sym('0.00160433539448308149487836320750909125660696707576150825285285443279571678852660264353392266462629890667537647602604036067237121526871049245'), sym('-0.000568457895732747198083404723017650185351261476758137410833479759629227392526824807068792937742889538716448874935869104219630615476769190582'), sym('-0.000696351168795829635985129158003718235188400898511048233761598731866421625951246799559756973060995343906688720301926609352626573314562597072'), sym('0.000424355573415088713077481720677345336907516538622028977525343605616566125812362496839493292333181495831030265809563339164092786650156456726'), sym('0.000242387163114690614635959107114871020132609527159046902994210717492113638563404556546961171389485241178099022158516778451250658448114040945'), sym('-0.000247689409639549287374848746373012002439817799079330679376206813165940611955202732966155781521461841104647148695722140267289755917212838385'), sym('-0.0000550140828594750442056316058473548050101102364508104649336902952947042824009968531196331615731187246303133893973318896662603737422560334263'), sym('0.000119201338107365061218531170251607106852232697571871931838957871858563718601792846922636072445696258523894257539388761998538255013804962277'), sym('-3.11379652026457758128374570413223755324204776404995328067672674953880559862429891295652020622998536696113020823854589256348876633817781152e-6'), sym('-0.0000475744526556493179913507406703042923426136099694060015969741494915511104858768418549095742584495382361375074350241297361806359062950212754'), sym('0.0000118023811912211426034476335256027362639493568373599592183560412710764559055251344760295996847096402044363887844884287209172308649435413714'), sym('0.0000153285037192763753079772673124089828516053287530463299966190776656953689389800999772431475176909091574621879624390179771780487736083455266'), sym('-7.74906706566088463598109581029350773868850331071293613319059471249034154942664060521871013850424764737215445616335223440041461644547385622e-6'), sym('-3.63587158595956863200550685067892975286307625949295624719131167082996177057949276590699214476054633960469974385584424016978555598753262346e-6'), sym('3.44835248132789966164772291816530774013505602259239871290818121664866734734068364848057024311244121872763085346862339942044412897839727815e-6'), sym('4.00363772594343387363556796859468820222121931457319638598907441939086815958865946797056651036871530597316723550982175525393515864635336783e-7'), sym('-1.17262538768045618185579100376334528704244610416736827542749809619101427847444839985338530594267242065924076487317596814362425719989740832e-6'), sym('1.47663776194210093231144056074878516432546737954947010476318027774231005258799682051093645971966627335596311555161900555162198499420911520e-7'), sym('3.05443443482098362317737412367058074819508328169644610856302104171864854799471396615875134115639809358819029654598827581380766166571612168e-7'), sym('-1.11137383456104120753111347774000274934018719177881608864340360849831206279786510406656594999086546583924845360861733983267633981497699586e-7'), sym('-5.56730928235198255968607733686273624673187289155302786548134871414013340303551201862292066287832744775777130488288645364354956800314047362e-8'), sym('4.11062274204806999454897253867883704982061009776948796849659771989349263123263795071254744389776759981973521001453982013500129931906294712e-8'), sym('4.07636141876642720942797698351924831538148236253766740028629510552021679059704856196018892390684156259786525817222996007515181523531975808e-9'), sym('-1.04882696299035068370531593314086320632089634708653431447862313095828517148952652871681231395438116536481332507139558005798666556624671095e-8'), sym('1.51518439814624365454952385265437678829099793508061729896425470338830784429702463395839665354534302472549043183935206993363181486796811353e-9'), sym('1.86141015543250944756935681239289567477488591456611071057440248915026202031240285914818191877568539831821969324891942094738573110124230075e-9'), sym('-7.32441964235968791740668597646381906324698852992788395923298217618873097989422241709455507677629995317527077773066039413409736817501986699e-10'), sym('-1.84889674227780549169152183427800713470998915664375570238714788508113925230287700563927273801740696465530609902331658631668675703150697696e-10'), sym('1.74263297049452018196789051453770569673327297335749430151212227630597091396052687789724079959863933000524522083666140426952404658617888473e-10'), sym('-8.67906303535649764289191817800346251207913768564603167826747388220858271196706671585802540264525162247567186241974191985208970611491629448e-12'), sym('-2.58892610139156952375840895508702237956154510097773012689575197593144286496606013084317168740358397235428624223659999242037434235813394567e-11'), sym('7.24589576005567862697844268127819672076535547838195741727395692258923263425035193410756559407727798183127617475476576638768196148399511319e-12'), sym('1.98551638201417454643828567170169790155764916917945366790102159243763614419298744091773771088836924079844980999880438571569022219593879636e-12'), sym('-1.40702141002034774443005047243208483193401360213226565663256189003653839073650228915093084772740068044386278989504065034234969622653973814e-12'), sym('7.86898958361828507123482816161486818778410005975609361705815814465202809996169367550727966427958435812124236234145620292592874730756804201e-14'), sym('1.41337234544708614501662314797358939717706457480339728282178062010096818961111020899063873868825326800138843588608398346161690343694552857e-13'), sym('-4.03816978642071219301496968053506094516581510953890321811853741176453443525905069705841566676785937875645637595981911162461422080320883677e-14'), sym('-4.32372078751443907229595232963757604755249898424689697804731069391838557393262836737960496794361731728783205190573105849685375864489947147e-15'), sym('4.48802080284374656365006121193114927169042801904016888862365985732147119452097815285923281364066631195342927752492561781813603654690556879e-15'), sym('-6.76640315216133678978888114627025119340233328422024924984366564058366795284873945462859632726026100311893272997603774110235019898093072372e-16'), sym('-1.61294402376042782561145766264359478503772957840732855542516951964613062456728851973581232054233134069656740979534692779768966844235114228e-16'), sym('7.80898931412756247910429689659676910250681686619571295553514966497655647232458961715105128689314241893567245873378059878746526966933552395e-17'), sym('-8.90037296540399138521176970594172270982887354403383429287955117355754305754525602297747463776175760484771686347365322992052005600518255730e-18'), sym('-1.56548807920605217753159312224023862786813611058594294073614943302839276987702598527815646912294776260060257645393972144318682458646498344e-18'), sym('6.85575863335503227601444976470842775271246486535539886571572256329760738228074689954152842400407213413994571936265156132121837831382307702e-19'), sym('-1.05986621011903152362741463048683456617302880405281946720872206233708707370289197067993837955795151917827406598367968945402526445067911616e-19'), sym('8.28685782983265720074639351485458183357429211247265029151272906626159486566419305354500359874573774405993728408369485713051775049667876157e-21'), sym('-2.74351496607378200196385740349501566953907327434044043698819712529405149593276067849663060227068683996288921553510489146666044919071644022e-22')];
        case 47; a=[sym('4.35535887273886062791605571798867852975909099226496688474314140579193500585313156414587662425818122775253178713665041253255072274475067311710e-8'), sym('1.34341924822124558703542220463178298373956170736668287156636448554784081790275753127556560919515710993798907689600151478957825507333352680005e-6'), sym('0.0000196875106387066151407690137959113662795124293213117535163433491117134875974821939307453907875995636974019421197133564528987979997304197046150'), sym('0.000181926644391694310738366760961257746631697281909266224309335668227471045843195032124522107972911711937237212878467544364164987559239712363562'), sym('0.00118581652565951237895087681542507381524076477385055393012993299811648129602014084283205531683625337646060423997959626436543984722968611240764'), sym('0.00577399069858496082224656924083369754901297054140165550910298008072445439088951030339344680094176855664297958307790272870456108245782705026296'), sym('0.0216744907578882969675100017887459741059306963843918661746291451647413046348679675089697395042720438830805219607828989774593487427365684430158'), sym('0.0637013196104195930299266913431310011188758098213411116321258671238282697811168789688002821955053824458355693428048968531570210598730542946307'), sym('0.146903608550676491850985615179071492831092327633038741950740004782606299878831927123065441983868670792296574387428305707867396149632292012919'), sym('0.262289476012045938169636986248181045346343744374654708403451341133644233940693146642373721971310671761205920588197858011503044809242018820543'), sym('0.348087026472846286018484407017164580863778645787397760206157066016881762921532828002534175776345514927757828314407250695117332754791953383009'), sym('0.305695519204699657919258400670834534354157276038470644125435359903730560042580561048328293769539277645989266013387980649708359161408838653857'), sym('0.0972179995078746814637295732821412408531615446050196538290040250304160403595007268915377436625935498760096246118813412509495780378574658767776'), sym('-0.151665905954697717793526747443384268496186305621301354600466600594408346699131746985419522023138340433744140272397117011427796398298025502005'), sym('-0.217529145563484680074234000131002281636072754373972298502459002655652967505970840138427784787835591385700747805547006232759345600235483365996'), sym('-0.0355891410543419249685327434535139156786735945671050323413722494868981839492736696325761002804035453705065696303265605021866323708038105499046'), sym('0.162770997823342512776176906516870482855544461710118151977303211194082999465641801834753096078142597930143507838775889675470088759654107018009'), sym('0.116705393724897842219272381692268532515165991778287292875771349179019859790630399745730164549981105081474658558967221361169545200832101625240'), sym('-0.0879888532246489433916371911469192061517007047375219061878475647370300550272066235548184050378929059794996299627263640213353787304455176694634'), sym('-0.131687881794743682394797555082138332428110152645239321552082137226636793247009022349959372847585139956640165048112116994190609010399318835024'), sym('0.0365264318561055658765312127100178850797672575609440082111310524235893899617005084608518088978990080374671066843114467235531642127040892830854'), sym('0.120571542736539644650816605828789686018831410317917901177397888890180584929375413274788285791635405794562714492079669626452456879938026512927'), sym('-0.00907027785975718828367094692597072664759154425111246716026700606031759977546844539247775717669707937296820078370955969205971029234930675524646'), sym('-0.102547594197376065612519453430877167640866802974585409546426096695493235984606373912495344304508592406669882768278658403938606736851683058137'), sym('-0.00217297724927190645988368145559801068489122482369269484860892271933650301354916058328337038930752423547944435910857568084808993056528428705478'), sym('0.0844448163381945398269601546511751237264795869950897508063887178772126112916701296788460001200984575914482987177508525597012096744435591868722'), sym('0.00414384103630426099848752300755047829685391496623268707974947084790610786614491407469571961108173824050122087771857163862421309163359165231203'), sym('-0.0679423102395322527911190007389302497336275248323341720945504542715592680641549553036932151305299734454163455029603914825025043920434873565117'), sym('-0.00165932973877834982288169646152249386427104821557959491384576979290424252399314360907442866749654967494928564238699223161523034553428550836336'), sym('0.0531189105831976180465014751860352008507609188961271513534117533691529350529915232295583490389042049586341951720745606046996636117564138534818'), sym('-0.00216809814211646936322738606381149006769325968999293293008109320160634216790655260455626000966751880043436132788699470522826738607958697121274'), sym('-0.0398674186308472654139253306004936369331880759148320002494886203232147515307138927268803109632570309028592230895375444245898232789325134424485'), sym('0.00547501083190115484796531093015021930811742232839210861724766002394047906378395671642880918417454705520593130655458191383362267319256973394940'), sym('0.0282929300884366336325832498750425104606281792024988870405328845845268966560178144531924766200951625821415503802731863657279219529050057222712'), sym('-0.00736287187080564795308569439666028369031962617186514443647351133054061874939468603583025501232328478269987311637122651949652579596091876696162'), sym('-0.0186624779413502368750294354732788518629571517796733310646656605692775566962165124232945316861002961371158086023266360278991003910921650594606'), sym('0.00767835271368755040163974049036511534255780294536665574128875230176103780582156950451128491527146886540664090123930820395193469890697356766346'), sym('0.0112054080716629516614398813490793526272054336855675609097129710997308044747263631235849766855463038516312243902637260717742126268419990807563'), sym('-0.00676750168378479504381232974394191816243471180995794626362493861818515514969947652174111912499534594299631065213203965943367120475952499786013'), sym('-0.00594187690379973626307209507701686586429371693546214316587336267867827186033171184369565158401051291087879852007620460551353097322063664512899'), sym('0.00520031056980410854274031097264712885189290384688505157350131475803817968709099196736108942461206863512506659890836731779564076070020747922748'), sym('0.00262825553007115661933686819158585925618406010442123078544303282443268933148716621138812726705407106359782861627884483238354678574615929480648'), sym('-0.00352405345225973718212253446823589170463608983603068131429865660890641746529580164524864566278800263144653211012245961909082555712687233663233'), sym('-0.000824124118557668061918673510390870671331046317080718678497476707330880324726603376399948730663767099086220309420389835320754309401777456801139'), sym('0.00210854975325951103089550527728559520533446458116622440032601063559820302191834774658998570134966031513576120880051942055863217422843802544907'), sym('0.0000246405146253510768251879916943255962359263447080942106412211424351378741148013330920952401941338636878909568273931183902612107179938823279703'), sym('-0.00110614997451607703013518752237523345484594928097470608309461490055668171766949326807294354744989662213776798509235879477868445418854125092020'), sym('0.000214529485519008301740721558780623158904620835940801019632585522959644118209232267692150345145655131509077675305303898674709485458425245178188'), sym('0.000499890205069734410176120222643051791033515436029342408188560578935696480051856746462365644849899153136071382732366659913699577517304973257828'), sym('-0.000207874564204222775777882119572700690461329852235328332891244273937631213441649053813596871822734985097776267036860268426652572024263962516993'), sym('-0.000187163733934100676884285641375662921748667443781692219353541799179108254561248577443062183417908073598857543203641725845402285238006782777185'), sym('0.000133629223437236304268978833690731182003385190924909855115342878600569621201620424861489470057155247321301882245925628843287634245487522535727'), sym('0.0000523040067954326180520339615183376049558503297918088581009244252289300916673465581682583964873564727396797079409053738587064670749293474856307'), sym('-0.0000677047581446448936361368193808205457368036764514513010704621462387938954935470552400505096410869294770427274522518276192291380107829760952262'), sym('-6.34343630741212694760429628237592440953338041753997615986949965340325751709767175327634167006909893359659645875302120492304032136391841492320e-6'), sym('0.0000280660340249147523522261410286186247449303507716649337455862675271643105262995614264232867377003999881548447148541539535188426279456489973577'), sym('-3.86163269753893642150113674109935675853728793411921970182171707863003703967103114912803945016390327457928169868483899844402747628824017922728e-6'), sym('-9.44404355341543533642432739108111080047991022439160553581186964386227837050885644158560994173263335818704496909459359156854611626976096569397e-6'), sym('3.50112083427029831696718069011708391174271865507480526377718543882978815736819441719415372136494730288918090383351876530111234477871266164830e-6'), sym('2.43595718914333947311208787336363830556797548318403468593905275630349401430339642485044886330227712970182957604892905203886220694923853462299e-6'), sym('-1.70767324394874745629352268929394953788348406611946504011912763251416663006711871600836682055570250441181152556484633190867567618588302848462e-6'), sym('-3.83459585590961642359664915062601449877493167891453235870690224981928333517665312701906507819339145792627481890389305577104848654042921400293e-7'), sym('6.09011910684271618307259471145024718297525265377598730383784177876788668748326434424707888229777533652338387443671304477096980180649929400544e-7'), sym('-2.70232098928257508506149636076414945543852505407715871636180178860810606301302297011705455062393752603863483786294942976338955693273047114456e-8'), sym('-1.65435738764180937074415822082566421537200874883890304188398611185914495961386162172541453783778819031991415534633874831648677289610608482705e-7'), sym('4.53373931484675489675451195413111193980105061220246165772156161028210765978775121743732169405355337555206677032703394915249025000056318377587e-8'), sym('3.24372816955320982493032624671923243779396699855143364036569757056441698280358681180329869185213354383629614115751692646813729014960756389224e-8'), sym('-1.87180345479363175600799627380635625034015307520295762525463478826088451229668372715387664682617530729362227165863266350591860221446556992372e-8'), sym('-3.38011280641430036411099202822706094088699187187856028407205733252037053205999370425074821403821801465202828489633250632995079744474537203481e-9'), sym('5.01836482080337888821055301062880828946908810113723697972777265380096769962495022888895047166958613170390195486502792138633537730790342868172e-9'), sym('-4.26960810841138713056728015016523755772216868372796247856218418016170334601784761610744063861975340619650935902172125798675073449364765738768e-10'), sym('-9.33256142522359237873509051057647918954987492647708803028952970856453920089907306248746368999722603961764386999394021788816332764400843922919e-10'), sym('2.98443833217353594493843470977347551254807050090800846975569310514799069006033247787543470585836694228033670013124627028996484132184266196732e-10'), sym('1.04255924908021878515555411163305847370010860170421861591863641959262566843442263921266644356390486922902446585465144298782574858234872988232e-10'), sym('-7.63595627117661722421356662461528571115541680981219635774717593369412781792492684660842102595162448909211924824992783723525614227176768713881e-11'), sym('2.72087511008882265713642273421623397480526633593689743860414586569158442690759232213283335390128566440666224430874831228457481078178227184401e-13'), sym('1.18653285923654472040484538003109505919097084585888259849699340001205727103057760093164719668629976276492189855512877780468835245871519606864e-11'), sym('-2.76136024403812262692442291378522939098723500556138095475837222454707983309179350033158652596131213981605133621910643340654043049351342856397e-12'), sym('-9.99454359861784819279510702422993686919570382284103743441992084955717608920606618630119293518644521002177120125937520492128355220704776031448e-13'), sym('5.83202291824897411197855678894581064001526539965349164974543742828267547188382135245510845959703005539514645956047837410341473110660203833413e-13'), sym('-1.37806654209082174632181113119071658618134967722218804352095576813230734413350959315107304013884736576481016784957905910516467626089277842307e-14'), sym('-6.13703872040970853132639414419715981955228180593005239819501841836566908492423899690345408861352925198153176868221937208282633261008444055355e-14'), sym('1.53675664806657557552451810134851947505019976209377987643644458394512066781965770441004743849030885927648345833536586751073936103193274912334e-14'), sym('2.23945506448975234667458957891843560836484071119743565091727447692012356045074978506093177657144606165957831754252555531761982392515918290762e-15'), sym('-1.81249373366730907282195385174075351881802927012848372140904181774787425938108762568627719001721156772310439374536711454445874501175538280976e-15'), sym('2.38301848807783818290637606400141403074881966973671324582074951748280933110259691417925203173787556495440391942468660144863488045436891696203e-16'), sym('7.02589624916033778966387831628995382119435488688482022326200905675335013125470399975279450805797513245884263598113249725478071406383925509141e-17'), sym('-3.02029973757402965347664068945285676317130727500037522282112524040543272208518605587445527141014702768590097235885229217847864588923317250693e-17'), sym('3.09952934113662405078523306343544514545523652078626170177862871776899042738730201764284848639467125073914475445471171437961824961697684182610e-18'), sym('6.52858121311650391029412134825696627312848704102113420150071651342525478506635582749937271151879605381217764431021205026879329860254091056127e-19'), sym('-2.60271967118807813188459619725547556657152119992235372376228748594817272622570465938266265559476660973663792763693614826923067948685349580806e-19'), sym('3.88850896865741507706667474757619378591589205985878942184958124689841921741555711627394971127360202935977009468135750947065673538232470802245e-20'), sym('-2.96626607959547051285212737851151089869119620995572186143279635501571150858034076549741439215947366857812677631592966305638763636805078692821e-21'), sym('9.61662065343790057527452924313979914257219944139392225398409455404966319777472278828237650496868701242482223659909529181804984261236088459299e-23')];
        case 48; a=[sym('3.07271330251204545079994858419551643342767624313565777068797381683476357633208361784420754898851783961315507870953681851607963670696795690495320e-8'), sym('9.67446159198299709388026568010428043996357417656574208254853177588183806405155929259574806705994174282928491321833464550667322322593370905371526e-7'), sym('0.0000144869451686009433807171977542129158371072469525526884485787022548614132246509275640596269663028731330690539245494287281893188599466637958352938'), sym('0.000136959055120734227099052862397215871973686811751404776181199962643336773157227926473068616533706787929350855290582074228728683927830142667651296'), sym('0.000914685807021242113866251306141605828094541947814937925681498035661940581940974232049420311825940502024477587043123951603883289119924391709777129'), sym('0.00457196166811674944899423061673129213685220784501124671951442745680726920830022587268101197572405954115807455953405681363854183157266478245963138'), sym('0.0176602569825155284801888711177487294149431156957760025562709418008524801227948422142959476627512716882308243935707495692366756152542146761489373'), sym('0.0535849153527647759984689356994557194249496324805999935116842887365279046305135824102787321480649396603256339893566685593046164628855154761429859'), sym('0.128186697179466975748596187757490370464212137971195432843456686188724318762486129030268881199866682533354985462776720226372447740833891134517244'), sym('0.239244686250339884812430614707763606101446668901779297187900511871614389287183021709130351697442030592918516406274068182606451706973600699001980'), sym('0.336787162178195363249842253854649653949034235930387105848262962971134966231596625967068434042106638864854502151723184204254684317111185332426272'), sym('0.326060345779617865567806671663516830722233968958382069719121286982017289608197084755648436362204754515801788363251616205317729328682196717873580'), sym('0.148191749580377422778321094861533984137698758403351053175590234713341922191618331324140541638747250677261665501084388155032582453613407443085359'), sym('-0.108131052128266402010454628263520313721059607288608135343041399572396511696889181290657617382635942527570008033900375893365321400229228976467123'), sym('-0.226118635750882075065051242286274739697492441921998732862972573771827156071103944280733680583419180609773627468425208993298898344341135972930833'), sym('-0.0874510802510343300051900278933791106812267544238327992002011854187741982723987725581359365088250776328332055372412748358165385185825921995010939'), sym('0.134519180916848644226273332300344016294502943550969556834871792300061561743356752224647877097145018378523496210138373463341581105878563769881827'), sym('0.150560110945048471772962930668676894371454144760357157184832740882969668051449587199427173282875333877020406062546634354741160385102570577975705'), sym('-0.0447932010268020461074489352623073935206927460721089192956131575400123535472555359649986707758749022901013788079883264873262695830473962640632091'), sym('-0.146694714942652182640341032053298241436517957154519375759600007331041847515989259409926738707740152306879462038794576291786323241399791302905397'), sym('-0.00826422914252315230090516105353710493966707923889014881815636807878107910618981359914057994473856881645931685197924099241305175981104602284986670'), sym('0.123285632153144411446716709808079974335370722590820618320629935798974849387824110896406745456553445514420449546564406143709646094462163996592851'), sym('0.0319553239588521610291667018969291749274609664650598369932716882829467743889272184446939541051800050635052452683939221591608487220879640980236106'), sym('-0.0993203613309945438548145703614958601767346190378584727916915778050820418510969765978729328176304501418269212970123273313384290282299286515792968'), sym('-0.0379976789735585042890315123430090973601555788521988599587608479607376148360112210618991242964796225483853688693831426921427607167326282981741442'), sym('0.0797028130817602431599526253584478687927969887703852034840825856544753767791972435215426829694220414229582609388862816984961302854731123943621556'), sym('0.0346849072775612535177506120756619399348388890029653909343748851620812731887237246716120627505326807417960041715943808087605830135593246408021339'), sym('-0.0642794439985271074843702262894698851998679668486432773192119178260673264701009876360722541788840261211470390680525847891566732460293680171594407'), sym('-0.0270820602991892216943165856579452728544342675540210447246950152099035818842823529269362749639552337980158562776128162473191982167735320312780734'), sym('0.0517061843306222887470528542479216911837538179768881736211147194270619469514973128149978340305488008511577264511272720679670623001565170468163324'), sym('0.0182615313872536679062748367726796719221145519518366979095002097947984926425528862574225959087691542777359182474409821958694889080289239463323555'), sym('-0.0408408765680190937843701563397674527376349933104367962815085721800260279330938286298365575698539422174606098735221797723587917709460681126567146'), sym('-0.0101149414647031050750984989977283197785233061453552501543835092613176237343797613926662589232187303823393912238818934625044735307066946387665555'), sym('0.0311129171848905482795652028238331780542167091323528316847319612062167325696050059505963977370110806471125109983963435326217911353824361391296610'), sym('0.00372096192637510208918433825648017500329799596952102368776834450407029168273487288339781491844485064353669966341679919195922216941752601678289976'), sym('-0.0224652222218470641995038427938507514124995913862838195652569352067962574876168267576014724625969239674449792485950821538761603293488295562053676'), sym('0.000501493634272799308466946897633980303040867975202660694663421311401584177185946502581896027302507292175943039528000812864656390959322638816230550'), sym('0.0151212322076617607164617139857622730913521793071442186728642201571584857119311271162621298719518079963780220526866014645614685352436019700131620'), sym('-0.00266901538825683461328697104937496609247666535513174571935539218636736885696425274863797470806886784611826926894808459685883572669761651445992974'), sym('-0.00932476819050476216527847421672185792652292292589725205198096191157147311784768589468248938438640598479898500001312505684370775942125777469071052'), sym('0.00325688357977030552696608083498665170104884987006133469454640904237950571801215944380155374074513264471681735839518653364428181636360160715518426'), sym('0.00515561649218387184481800181142414380796876290104430603426821645309451896651457952556428397453348715367494115878777557744277356171499247819487087'), sym('-0.00288205167717147333194620472547193223410554844328494670176335368939286869652206877042957405425022382088698257959094059644511275540844773061299554'), sym('-0.00247013200266900142392590799184943782905700174250083375739454483046898645815882136026392425140616186371370290376643874027697611657413205556811920'), sym('0.00210716105822632877151513772755452383356999413178475202905465156075881919201905825572865954672211554466774738448000570902807832322313198987315138'), sym('0.000954056602655891752295321067860918075564774667027176384460346838278785892691513641313317086726291601987485097075918256509530101207663962372187125'), sym('-0.00132191411778802544115877131728901454481347268660130272076228878803769006528642093706887010673577388449857797946906420759886414532188014604602966'), sym('-0.000231114029571173643007004826817020340499612187070259213525456297876328444915671552342974731297668039023460198770090259340259973383809269602970398'), sym('0.000718505036541537104421433155982984700170388991754399025758466528472920606775234268108417650039438633290483795787367983566895886185805962395164403'), sym('-0.0000356937346368829992529724757825456720154403063252822939175280382361906080372208858816713691307495546250999416977956136109214369330546682003772722'), sym('-0.000336338848904474600354466425546881397305764698988349709948244942883289971255416456177511756796376323908995323874478198558304185254911126732373300'), sym('0.0000882989326062619233876354750909178715999251923848701662247671975247252810411302061357126913090926394889404552411456600798531463657802752577697805'), sym('0.000132434648421328403603004406950900194931442689971568846432485872999800508168230424268379227714920498751218423961079110087995671321311025732175052'), sym('-0.0000673233994281666024173039013999697993143590400188359214771443234836154051433623375241465811336824626933800688093838638350187275019296297632115201'), sym('-0.0000412442323942967362319665409009519616630430932170090244132887472311587579971723805677971963373680083511540004937902406895101508616866669423556971'), sym('0.0000366627149193476427361982909861581946101111453013471107607060293254303570736711532033683739298268541624526338187125442875732288689307909698553678'), sym('8.19486647038569439485585730205876216478669592704954884599912774925785146898186295076002398938862532294625179718372027099090547985721470980497193e-6'), sym('-0.0000158761814787915357420252559237378459334947503702414092920014194313492971044851849541640519933256313402876571748961768086286069371479371236581668'), sym('5.30448497678827644166925778105546165439435675395769951268854091550431755570995618655101737864363529493901190038318984649480449984933964272654982e-7'), sym('5.55825799900561232264918451227840574201901569897760861221882764077514872869935150984854551365038938511086594647859979261430387446778057377809752e-6'), sym('-1.44173738921975671708055898286392100578684249027268049039114007222798502983586161351956363362928856573164151100816712846642571340662471709545293e-6'), sym('-1.52313493698217718781231652550196978184381498286654019954874316853748345740463194924727904350887083611396125270818143691986095386565488844111105e-6'), sym('8.09843059217503472421162282784639390960476873151233557384285877151649742480496199018473230596310874986176996708091577232469742665346669653574793e-7'), sym('2.85730599327137558340252179186445403526560113027669259406647547104253326685467131281715166175878993134468353721898688896746774161623921069693673e-7'), sym('-3.06308425321776051250891098260629966564560575776822210549982950307502468642953326513904821194138876519049128998710734836870350285808147155079113e-7'), sym('-1.14340001495414282571759567561546648328826035963756380321018835621469535257412639685687973506608638898139221357758220918102607848313402992855632e-8'), sym('8.67654968417998296396155843728789664213322259237480293507513839986828303430691871408413022956297137392811149644856943028765443987485231021363131e-8'), sym('-1.69462179436119930727838627991665670134574218519259067434567967894660418471231758916033552472668433767902028313387923983637675800458708412651887e-8'), sym('-1.80205369069161235978127987585811410238701309229224320190857990981939407008737726155685872952557631594853148021162744648297692035688907934919355e-8'), sym('8.25151372868284230330846922456838043762001479120720159349792963997350789914550548536011510434817111173828581127788581816635325493902260888632781e-9'), sym('2.26707072672789315356632706955871682833364084521580207071762092026690484564138091891373480945774058069084735161367062688000854292481835504363527e-9'), sym('-2.34569933790341097192645427467475841370387306576367609668320106762147500380743892279827547564972531401718790118811195565839899682512137169807924e-9'), sym('6.30239364081245139135675896150314721998712259351584975434462308366334269088170578701037727792889851975291953899832314678236276533560489894071507e-11'), sym('4.56085065308151680024114727371753925342657538992177785035167918918880970619575002210739383712954232595825917859398567726934130289813087743777780e-10'), sym('-1.17072597987511508593442776846676418628954537770079507463512990393807162689999333106825979861442930255155383019823797343994121498131188942455125e-10'), sym('-5.55801072771656994359388653061789090724274559764291529128684550387939452349829214802119277576268434036895517874778694078932736545588910959703518e-11'), sym('3.28433464465665687526797139241972809470563961214621046472950608712271707551142954912571472263842470609749529333619215225074961999100358910231835e-11'), sym('1.45103381814343591734564449191800019211140675073787017597422324033907068534280603061027337416448893764415706460730924076790691856792710938136535e-12'), sym('-5.34428848768470207790841385697310196686912068977869980778763172123315140898552853726610927170618526097027916475175538912914125538601813508902527e-12'), sym('1.01700937108266109279834417094677458303899670856194790308177224230852518954991286548754168071121634940381986732201649033758057748916334805073914e-12'), sym('4.85416778245670977722468381882936244754053895744102871346190187593462801801286055941927910935269217409063995560162735039250085757978864826484548e-13'), sym('-2.38596856722983394076673582839860792115743355995545412483381643853824010581080179379338458181365860839020727933832555197780065100250908648874528e-13'), sym('-2.22690679058914085051300581952327100167642706397721000856320831951007546957172884393531468683094402025525626886702173536579239080528897283865432e-15'), sym('2.63094149890824988114564586728762961134951984437794291205561517916572317384699599435923957996921632503570371430251270650052638261803989238189218e-14'), sym('-5.76072638792115433544659541441056344883106924046710414401755518272447607962009564369752330352756977979704920904987208610677421220136639874252690e-15'), sym('-1.09445897513359603471377206130152471530341488269866801553743013860337340046646182990568446340933155502670604078054800144060958447470188126628587e-15'), sym('7.26378147622081803141531446768760333656234659676235426885253127733589937143425696291824221817937275803995603215743568027121558834028696757021657e-16'), sym('-8.21542705425804077739001860901045218607289397978101162429212507520726311412426869911792637568894526713607224046280769344260597289060785965386623e-17'), sym('-3.01045496797566638946892071837722515420163933496635864180727608818339759571978014273975388086706258231805268702512326605428337512653600690156409e-17'), sym('1.16276224728798014407625989686214566226140271354203512833584689490194091567373810885690674824393855408976679345995904003385460688382636825276050e-17'), sym('-1.06561284427376505057486718451646181386794756838681053063293435699274228400348593951709343892180288989890798583722397206984915600718615898249933e-18'), sym('-2.69059894565014238875047396763187826036777974757499192766937209022334004786920676199960428838100264645792978546435196861487161840016640938726359e-19'), sym('9.86070582355111759046706432115994993106999066103173901397642996536967865613840191200996603173423297365757208971655610368176380778629676296007767e-20'), sym('-1.42544571199383351837782561100339429986848126423344056460950978761227520756021101039496240297196269173035618457500642628882826454149467597622984e-20'), sym('1.06151041066475485980645234465686889801924472492925312212321151803152436147054190696683057266903039861903807524741567417108060792509550390310101e-21'), sym('-3.37147150628777748565874016035338763722710950622442039449935757191928009771757168440482521533400636352464767857523866379441237191650332070235296e-23')];
        otherwise; error('not implemented yet');
    end
    
    a=2*a; %normalize mask s.t. sum(a)=2
     
    if(abs(sum(double(a))-2)>10^-5);
        error('Values for entry %i are wrong.', n);
    end

end


function dummy; end %#ok<DEFNU> %Generates an error, if the 'end' of a function is missing.   